#!/usr/bin/perl
#$Id: account_options.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# account_options.cgi  -  Created by James Pattie, 01/24/2000
# This script allows the user to configure the Accounting package
# and specify the account specific options.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';
my $ledger_db;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print "$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# present them with the main menu 

print "<head><title>$prog_name $prog_version Account Options</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#account_options\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Account Options</h2>\n";

$result = $conn->exec("select acc_digits, sub_acc_digits from version_tb");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the settings for the Account and Sub Account digit lengths.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  print "<h1>Error:  Program is not configured!</h1>\n";
  print "The Account and Sub Account digit lengths are not specified!<br>$sys_admin";
  CleanUp();
}
my $acct_length;
$acct_length = $result->getvalue(0, 0);
my $sub_acct_length;
$sub_acct_length = $result->getvalue(0, 1);

print "<form method=post action=\"$site_dir/cgi-bin/account_options2.cgi\" name=\"options_form\">\n";
if (length $ledger_db > 0)
{
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
}
print "<table border=0 cellpadding=2 cellspacing=0 bgcolor=\"$menu_bgcolor\">\n";
print "<tr><td align=right><font color=\"$menu_fgcolor\">Number of Digits in an Account:</font></td><td align=left><select name=\"acct_length\">\n";
my ($i);
for ($i=3; $i < 11; $i++)
{
  print "<option value=\"$i\"";
  if ($i == $acct_length)
  {
    print " selected";
  }
  print ">$i\n";
}
print "</select></td></tr>\n";
print "<tr><td align=right><font color=\"$menu_fgcolor\">Number of Digits in a Sub Account:</font></td><td align=left><select name=\"sub_acct_length\">\n";
for ($i=4; $i < 11; $i++)
{
  print "<option value=\"$i\"";
  if ($i == $sub_acct_length)
  {
    print " selected";
  }
  print ">$i\n";
}
print "</select></td></tr>\n";
print "</table>\n";
print "<input type=submit name=\"update\" value=\"Update\">&nbsp;<input type=reset value=\"Reset\">\n";
print "</form>\n";

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if (length $ledger_db == 0)
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/account_config.cgi'); return false;\">Account Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}
print "</body>\n";
print "</html>\n";
exit 0;
