#!/usr/bin/perl
#$Id: assign_accounts.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# assign_accounts.cgi  -  Created by James Pattie, 05/22/1999
# This script presents the user with accounts so they can select one
# and then proceed to the next part which will display all ledgers
# in the system and let the user select which ledgers they want to
# assign this account access to.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# get a list of all accounts from the database and display in a form.
$result = $conn->exec("select name, number from account_tb where active = true order by name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of accounts.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Internal Error:  No accounts defined!</h1>\n";
  print "Tried to get list of accounts.<br>$sys_admin";
  CleanUp();
}

my $result2;
$result2 = $conn->exec("select name, database_name from entity_tb order by name");
if ($result2->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1><br>\n";
  print $conn->errorMessage . "<br>Tried to get list of Ledgers.<br>$sys_admin";
  CleanUp();
}
if ($result2->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Internal Error:  No ledgers defined!</h1>\n";
  print "Tried to get list of Ledgers.<br>$sys_admin";
  CleanUp();
}

print "<head><title>$prog_name $prog_version Ledger Configuration (Accounts)</title>\n";

print <<"END_OF_SCRIPT";
  <script language="Javascript1.2">
    function doLoad()
    {
      document.ledger_form.ledger.selectedIndex = -1;
      document.account_form.acct_num.selectedIndex = -1;
    }
  </script>
END_OF_SCRIPT

print "</head>\n";
print "<body bgcolor=$body_bgcolor onLoad=\"doLoad();\">\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><br></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#ledger_account_selection\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu();  # This script should never be called so we don't need to pass it the current ledger.
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Ledger Account Configuration</h2>\n";

# create the form to display the list of accounts in.
print "<form method=post action=\"$site_dir/cgi-bin/assign_accounts2.cgi\" name=\"account_form\">\n";
print "<input type=hidden name=\"select_acct\" value=\"\">\n";
print "<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">\n";
print "<tr>\n";
print "<td><font color=\"$menu_fgcolor\">Select Account</font></td>\n";
print "<td><select name=\"acct_num\" onChange=\"document.account_form.submit();\">\n";

my $i;
for ($i=0; $i < $result->ntuples; $i++)
{
  my $name;
  $name = $result->getvalue($i, 0);
  my $num;
  $num = $result->getvalue($i, 1);
  print "<option value=\"$num\">$name - $num\n";
}

print "</select></td>\n";
print "</tr>\n";
print "</table>\n";
print "</form>\n";

# create the form to display the list of ledgers in.
print "<br><form method=post action=\"$site_dir/cgi-bin/assign_accounts2.cgi\" name=\"ledger_form\">\n";
print "<input type=hidden name=\"select_ledger\" value=\"\">\n";
print "<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">\n";
print "<tr>\n";
print "<td><font color=\"$menu_fgcolor\">Select Ledger</font></td>\n";
print "<td><select name=\"ledger\" onChange=\"document.ledger_form.submit();\">\n";

for ($i=0; $i < $result2->ntuples; $i++)
{
  my $name;
  $name = $result2->getvalue($i, 0);
  my $ledger;
  $ledger = $result2->getvalue($i, 1);
  print "<option value=\"$ledger\">$name\n";
}

print "</select></td>\n";
print "</tr>\n";
print "</table>\n";
print "</form>\n";

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

print "<br><div align=left><a href=\"$site_dir/cgi-bin/account_config.cgi\">Account Configuration</a></div>\n";
print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";

print "</body>\n";
print "</html>\n";
exit 0;
