#!/usr/bin/perl
#$Id: divide_fiscal_year_proceeds.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# divide_fiscal_year_proceeds.cgi  -  Created by James Pattie, 01/03/2000
# This script takes the equity account(s) gathered by the first script
# and allows the user to assign percentages to each account.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;  # The report module

use vars '$date';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

my $ledger_name;
my $ledger_db;
my $ledger_color;
my $business_type;
my $business_name;
my $equity_account_str;
my $leftover_equity_account_str;
my @EquityAccounts;
my $ledger_code;
my $report_obj;
my $close_date;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  A Ledger must be specified!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

$equity_account_str = $input{'equity_account'};
if (length $equity_account_str == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must select an equity account(s) to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

$leftover_equity_account_str = $input{'leftover_equity_account'};
if (length $leftover_equity_account_str == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must select an equity account to put the leftovers in (if any)!</h1>\n";
  print "$back_button";
  CleanUp();
}

$close_date = $input{'close_date'};
if (length $close_date == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify the Close Date!</h1>\n";
  print "$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now we check and validate the ledger is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
$ledger_name = $result->getvalue(0, 0);

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# check and make sure that the given equity accounts are assigned to the ledger in question.
@EquityAccounts = split /\0/, $equity_account_str;

# figure out the beginning and ending date of the current working year
my ($working_year, $from_date, $to_date);

$result = $conn->exec("select year, start_date, end_date from known_years_tb where year in (select working_year from variables_tb WHERE ledger_code = '$ledger_code') AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get working year and fiscal year dates from ledger <font color=\"$ledger_color\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Program not configured!</h1>\n";
  print "Working Year is not set!<br>$sys_admin";
  CleanUp();
}
$working_year = $result->getvalue(0, 0);
$from_date    = $result->getvalue(0, 1);
$to_date      = $result->getvalue(0, 2);

if ($close_date =~ /(-|\/)/)
{
  if (IsValidDateSyntax($close_date))
  {
    $close_date = MakeValidDate($close_date);  # make sure of it.
  }
  else
  {
    PrintErrorHeader();
    print "<h1>Error: Invalid Date!</h1>\n";
    print "Close Out Date = '$to_date' is invalid!<br>$back_button";
    CleanUp();
  }
  $to_date = $close_date;

  # get a list of all closeout dates that are within the current fiscal period.
  my $result2 = $conn->exec("SELECT DISTINCT(bal_date) FROM acct_bal_tb WHERE ledger_code = '$ledger_code' AND ledger_code = known_years_tb.ledger_code AND known_years_tb.ledger_code = variables_tb.ledger_code AND known_years_tb.year = variables_tb.working_year AND bal_date >= known_years_tb.start_date AND bal_date < known_years_tb.end_date ORDER BY bal_date DESC");
  if ($result2->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get list of all Close Out Periods in the current fiscal year.<br>$sys_admin";
    CleanUp();
  }
  if ($result2->ntuples > 0)  # use the last close out date as our from value.
  {
    $from_date = $result2->getvalue(0, 0);
  }
}
else
{
  $close_date = $to_date;  # We are wanting to close out the current fiscal year.
}

if (ReverseDateString($from_date) > ReverseDateString($close_date) || ReverseDateString($to_date) < ReverseDateString($close_date))
{
  PrintErrorHeader();
  print "<h1>Error: Close Out Date is invalid!</h1>\n";
  print "'$close_date' must be between '$from_date' and '$to_date'.<br>$back_button";
  CleanUp();
}

# verify that the close_date specified is not already been used or is not before a current close_date period wise.
$result = $conn->exec("SELECT DISTINCT(bal_date) FROM acct_bal_tb WHERE ledger_code = '$ledger_code' AND bal_date >= '$close_date'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to see if the ledger has already been closed out after date = '$close_date'.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger has already been closed out after Date = '$close_date'!</h1><br>$back_button\n";
  CleanUp();
}

$result = $conn->exec("SELECT business_type FROM ledger_tb WHERE code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the business type of this ledger.<br>$sys_admin";
  CleanUp();
}
$business_type = $result->getvalue(0, 0);

# get the business name
$result = $conn->exec("SELECT name FROM business_type_tb WHERE num = $business_type");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the business name for business type = '$business_type'.<br>$sys_admin";
  CleanUp();
}
$business_name = $result->getvalue(0, 0);

# get the last close out period value.
my $last_close_out;
$result = $conn->exec("SELECT DISTINCT(bal_date) FROM acct_bal_tb WHERE ledger_code = '$ledger_code' ORDER BY bal_date DESC");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the last close out date.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  $last_close_out = "Never";
}
else
{
  $last_close_out = $result->getvalue(0, 0);
}

if ($business_type == 0 || $business_type == 2 || $business_type == 4)
{
  PrintErrorHeader();
  print "<h1>Error:  You should not be here!  You are not a Business Type = 3!</h1>\n";
  print "You are a Business Type = $business_name.<br>$back_button";
  CleanUp();
}
elsif ($business_type == 1)
{
  PrintErrorHeader();
  print "<h1>Error:  You should not be here!  You are not a Business Type = 3!</h1>\n";
  print "You are a Business Type = $business_name.<br>$back_button";
  CleanUp();
}
elsif ($business_type == 3)
{
  if (scalar @EquityAccounts <= 1)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must select at least two equity accounts to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  my ($i);
  for ($i=0; $i < scalar @EquityAccounts; $i++)
  {
    my ($acct_num_str, $temp) = split /\|/, @EquityAccounts[$i];  # $temp = account name

    $result = $conn->exec("SELECT * FROM local_accounts_tb WHERE acct_num = $acct_num_str AND ledger_code = '$ledger_code'");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to validate that Account ($temp) = $acct_num_str is assigned to Ledger <font color=\"$ledger_color\">$ledger_db</font>.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Account ($temp) = $acct_num_str is not assigned to Ledger <font color=\"$ledger_color\">$ledger_db</font>!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }

    # now validate that it exists in the global database.
    $result = $conn->exec("SELECT type, active FROM account_tb WHERE name = '$temp' AND number = $acct_num_str");
    if ($result->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error:  Accessing Global Database failed!</h1>\n";
      print $conn->errorMessage . "<br>Tried to validate that Account ($temp) = $acct_num_str is of Type = 7 and is Active.<br>$sys_admin";
      CleanUp();
    }
    if ($result->ntuples == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Account ($temp) = $acct_num_str does not exist in the Global Database!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }
    my ($type, $active);
    $type = $result->getvalue(0, 0);
    $active = $result->getvalue(0, 1);
    if ($type != 7)
    {
      PrintErrorHeader();
      print "<h1>Error:  Account ($temp) = $acct_num_str is not an Equity Account!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }
    if ($active eq "f")
    {
      PrintErrorHeader();
      print "<h1>Error:  Account ($temp) = $acct_num_str is not Active!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }
  }
}
else
{
  print "<h1>Error:  Unknown Business Type in ledger <font color=\"$ledger_color\">$ledger_name</font>!</h1>\n";
  print "Business Type specified = '$business_type'<br>$sys_admin";
  CleanUp();
}

# verify that the leftover_equity_account exists in the database and is valid.
my ($acct_num_str, $temp);
($acct_num_str, $temp) = split /\|/, $leftover_equity_account_str;  # $temp = account name

$result = $conn->exec("SELECT * FROM local_accounts_tb WHERE acct_num = $acct_num_str AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate that Account ($temp) = $acct_num_str is assigned to Ledger <font color=\"$ledger_color\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Account ($temp) = $acct_num_str is not assigned to Ledger <font color=\"$ledger_color\">$ledger_db</font>!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

# now validate that it exists in the global database.
$result = $conn->exec("SELECT type, active FROM account_tb WHERE name = '$temp' AND number = $acct_num_str");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing Global Database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate that Account ($temp) = $acct_num_str is of Type = 7 and is Active.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Account ($temp) = $acct_num_str does not exist in the Global Database!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
my ($type, $active);
$type = $result->getvalue(0, 0);
$active = $result->getvalue(0, 1);
if ($type != 7)
{
  PrintErrorHeader();
  print "<h1>Error:  Account ($temp) = $acct_num_str is not an Equity Account!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
if ($active eq "f")
{
  PrintErrorHeader();
  print "<h1>Error:  Account ($temp) = $acct_num_str is not Active!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

print "<head><title>$prog_name $prog_version Close Out Fiscal Year $working_year - Assign Percentages for Dividend Disbursment!</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

my ($i, $name, $code);

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><font color=red>Working in ledger <font color=\"$ledger_color\">$ledger_name</font>.  Closing out Fiscal Year/Period in <font color=\"black\">$working_year</font>.</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#closing_fiscal_year_assigning_percentages\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);

print "Business Type = <b>$business_name</b>.<br>\n";
print "Last Close Out = <b>$last_close_out</b>.<br>\n";
print "Close Out Date = <b>$close_date</b>.\n";
print "<hr width=50%>\n";

print "<h1>Assign to each Equity Account, the percentage of the Retained Earnings that each Shareholder receives.</h1>\n";
print "<form method=post action=\"$site_dir/cgi-bin/close_fiscal_year2.cgi\" name=\"percentages_form\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"leftover_equity_account\" value=\"$leftover_equity_account_str\">\n";
print "<input type=hidden name=\"close_date\" value=\"$close_date\">\n";
my ($i);
for ($i=0; $i < scalar @EquityAccounts; $i++)
{
  print "<input type=hidden name=\"equity_account\" value=\"$EquityAccounts[$i]\">\n";
}
print "<table border=0 cellpadding=2 cellspacing=0 bgcolor=\"$menu_bgcolor\">\n";
for ($i=0; $i < scalar @EquityAccounts; $i++)
{
  my ($num, $name) = split /\|/, $EquityAccounts[$i];
  print "<tr><td align=right>$name - $num &nbsp;</td><td align=left><input type=text name=\"percent_$i\" value=\"\" size=\"3\" maxlength=\"2\">%</td></tr>\n";
}
print "</table>\n";
print "<input type=submit name=\"next\" value=\"Next\">&nbsp;<input type=reset value=\"Reset\">\n";
print "</form>\n";

print "</center>\n";
print "<br><br>\n";
print "<form method=post action=\"\" name=\"main_menu_form\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"open\" value=\"\">\n";
print "<a href=\"$site_dir/cgi-bin/close_fiscal_year.cgi\" onClick=\"javascript:submit_my_form(document.main_menu_form, '$site_dir/cgi-bin/close_fiscal_year.cgi'); document.main_menu_form.submit(); return false;\">Return to Close Fiscal Year</a><br>\n";
print "</form>\n";

print "</body>\n";
print "</html>\n";
exit 0;
