#!/usr/bin/perl
#$Id: federal_deposit.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# federal_deposit.cgi  -  Created by Jason A. Pattie, 04/03/2000
# This script allows the user to enter the first portion of the deposit information
# for creating an automatic transaction for the monthly federal deposit.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$count';
use vars '$count2';
use vars '$current_fiscal_year';
use vars '$current_month_name';
use vars '$current_month_num';
use vars '$current_year';

use vars '@month_array';
use vars '%years_array';
use vars '%sub_entities';

@month_array = qw(January February March April May June July August September October November December);

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $ledger_name;  # name of ledger to work with
my $ledger_db;
my $ledger_color;
my $result2;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid

$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# get current fiscal year and verify that it is not closed
$result = $conn->exec("select working_year from variables_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get current fiscal year from database.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 1)
{
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "There should only be one record in the variables_tb.  " . $result->ntuples . " were returned.<br>$sys_admin";
  CleanUp();
}
$current_fiscal_year = $result->getvalue(0, 0);
# get the known years for the years drop-down select
$result = $conn->exec("select year from known_years_tb where ledger_code = '$ledger_code' order by year");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print "Tried to get the known years for the ledger.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: No records returned!</h1>\n";
  print "No known years exist for the ledger: $ledger_code.<br>$sys_admin";
  CleanUp();
}
for ($count=0; $count < $result->ntuples; $count++)
{
  my $temp_year;
  $temp_year = $result->getvalue($count, 0);
  $years_array{$temp_year} = 1;
}

# get the sub-entities for this ledger from the database
$result = $conn->exec("select name, code from sub_entity_tb where ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print "Tried to get the sub-entities for the ledger.<br>$sys_admin";
  CleanUp();
}
for ($count=0; $count < $result->ntuples; $count++)
{
  $sub_entities{$result->getvalue($count, 1)} = $result->getvalue($count, 0);
}

print "<head><title>$prog_name $prog_version Federal Deposit</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;
$current_month_name = `/bin/date +%B`;
$current_month_num = `/bin/date +%m`;
$current_year = `/bin/date +%Y`;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left width=47%><font color=\"red\">";
print "Working with Federal Deposit in Ledger <font color=\"$ledger_color\">$ledger_name</font></font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#journal\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);

print "<font size=\"+2\"><b>Federal Deposit</b></font>\n";
print "<br>\n";
print "<form method=post name=\"federal_deposit_form\" action=\"$site_dir/cgi-bin/federal_deposit_entry.cgi\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<table border=0 bgcolor=\"$menu_bgcolor\" cellspacing=0 cellpadding=2>\n";
print "<tr align=center>\n";
print "<td colspan=2>Enter the month, year, and sub-entity for which to peruse/enter a federal deposit:<br><hr></td>\n";
print "</tr>\n";
print "<tr>\n";
print "<td align=right width=\"40%\">Month:&nbsp;</td><td align=left width=\"60%\"><select name=\"month_select\">\n";
for ($count=0; $count < @month_array; $count++)
{
  print "<option value=\"" . ($count + 1) . "\"";
  if ($current_month_num == 1 && $count == 11)
  {
    print " selected";
    $current_year--;
  }
  elsif ($current_month_num != 1 && ($count + 1) == ($current_month_num - 1))
  {
    print " selected";
  }
  print ">$month_array[$count]\n";
}
print "</select></td>\n";
print "</tr>\n";
print "<tr>\n";
print "<td align=right>Year:&nbsp;</td><td align=left><select name=\"year_select\">\n";
foreach $count (sort keys %years_array)
{
  print "<option value=\"$count\"";
  if ($count == $current_year)
  {
    print " selected";
  }
  print ">$count\n";
}
print "</select></td>\n";
print "</tr>\n";
print "<tr>\n";
print "<td colspan=2><hr></td>\n";
print "</tr>\n";
print "<tr>\n";
print "<td align=right>Sub Entity:&nbsp;</td><td align=left><select name=\"sub_entity_select\">\n";
print "<option value=\"\">-----\n";
foreach $count (sort { $sub_entities{$a} cmp $sub_entities{$b} } keys %sub_entities)
{
  print "<option value=\"$count\">$sub_entities{$count}\n";
}
print "</select></td>\n";
print "</tr>\n";
print "<tr align=right>\n";
print "<td colspan=2><hr><input type=submit value=\"Continue...\">&nbsp;<input type=reset>&nbsp;</td>\n";
print "</tr>\n";
print "</table>\n";
print "</form>\n";

print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;