#!/usr/bin/perl
#$Id: initial_balance.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# initial_balance.cgi  -  Created by Jason A. Pattie, 11/26/1999
# This script allows the user to create or edit an initial balance stored
# in the database.  If there are currently no initial balance records in
# the database, then num_extra_rows rows will be displayed (empty); otherwise,
# the records from the beginning_balance_entry_tb table will be loaded and
# displayed along with num_extra_rows rows.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$count';
use vars '$count2';
use vars '$count3';
use vars '$min_working_year';
use vars '$current_working_year';
use vars '$num_extra_rows';
use vars '$num_total_rows';
use vars '$next_ref_num';
use vars '@ref_num_array';
use vars '@credit_accounts_array';
use vars '@debit_accounts_array';
use vars '@amounts_array';
use vars '@fiscal_year_array';
use vars '$local_accounts_string';
use vars '$local_account_num';
use vars '@local_accounts_array';
use vars '%local_accounts_names';

$num_extra_rows = 20;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $ledger_name;  # name of ledger to work with
my $ledger_db;
my $ledger_color;
my $result2;
my $date_from;
my $date_to;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# check to see if the current fiscal year is the first year the accounting package is being used
# if the current working year is the lowest year in the known years table, then check to see if any initial balances have
# been previously created in order to determine whether to display the values already in the beginning_balance_entry_tb
# table or not
$result = $conn->exec("select min(year) from known_years_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1008: Tried to get the minimum known year from the known_years_tb.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0 || $result->ntuples > 1)
{
  PrintErrorHeader();
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "1009: " . $result->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
  CleanUp();
}
$min_working_year = $result->getvalue(0, 0);
$result = $conn->exec("select working_year from variables_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1010: Tried to get the current working year from the variables_tb.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0 || $result->ntuples > 1)
{
  PrintErrorHeader();
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "1011: " . $result->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
  CleanUp();
}
$current_working_year = $result->getvalue(0, 0);
if ($min_working_year != $current_working_year)
{
  PrintErrorHeader();
  print "<h1>Error: Access denied!</h1>\n";
  print "1016: The initial balances for accounts cannot be created or edited after the first year the system has been in use.<br>$sys_admin";
  CleanUp();
}
$result = $conn->exec("select ref_num, credit_account, debit_account, amount, fiscal_year from beginning_balance_entry_tb WHERE ledger_code = '$ledger_code' order by ref_num");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1017: Tried to get all the beginning balance entries created so far.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  # no records were found, so create a table of just num_extra_rows rows
  # load the arrays with default values
  for ($count=0; $count < $num_extra_rows; $count++)
  {
    $ref_num_array[$count] = $count;
    $credit_accounts_array[$count] = "-1";
    $debit_accounts_array[$count] = "-1";
    $amounts_array[$count] = "";
    $fiscal_year_array[$count] = $current_working_year;
  }
  $num_total_rows = $count;
}
else
{
  # load the arrays with the values from the beginning_balance_entry_tb table
  for ($count=0; $count < $result->ntuples; $count++)
  {
    $ref_num_array[$count] = $result->getvalue($count, 0);
    $credit_accounts_array[$count] = $result->getvalue($count, 1);
    $debit_accounts_array[$count] = $result->getvalue($count, 2);
    $amounts_array[$count] = $result->getvalue($count, 3);
    $fiscal_year_array[$count] = $result->getvalue($count, 4);
  }
  for ($count2=0; $count2 < $num_extra_rows; $count2++, $count++)
  {
    # calculate the next available ref_num number
    $next_ref_num = 0;
    for ($count3=0; ($count3 < $count) && ($next_ref_num == $ref_num_array[$count3]); $count3++)
    {
      $next_ref_num++;
    }
    $ref_num_array[$count] = $next_ref_num;
    $credit_accounts_array[$count] = "-1";
    $debit_accounts_array[$count] = "-1";
    $amounts_array[$count] = "";
    $fiscal_year_array[$count] = $current_working_year;
  }
  $num_total_rows = $count;
}
# get a list of the local accounts to fill in the drop-down selects
$result = $conn->exec("select acct_num from local_accounts_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1018: Tried to get the list of accounts from the local_accounts_tb table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 0)
{
  $local_accounts_string = "";
  $local_accounts_array[0] = "-1";
  for ($count=0; $count < $result->ntuples; $count++)
  {
    $local_accounts_array[$count+1] = $result->getvalue($count, 0);
    $local_accounts_string .= $result->getvalue($count, 0);
    if ($count < $result->ntuples - 1)
    {
      $local_accounts_string .= ", ";
    }
  }
  # lookup the names for the local list of accounts from the main database
  $result2 = $conn->exec("select name, number from account_tb where number in ($local_accounts_string) order by name");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>1019: Tried to get the names for the local list of accounts.<br>$sys_admin";
    CleanUp();
  }
  if ($result2->ntuples > 0)
  {
    $local_accounts_names{$local_accounts_array[0]} = "&nbsp;";
    for ($count=0; $count < $result2->ntuples; $count++)
    {
      # load hash with local account names
      $local_account_num = $result2->getvalue($count, 1);
      $local_accounts_names{$local_account_num} = $result2->getvalue($count, 0) . " - " . $local_account_num;
      # reload the local_accounts_array with the numbers since they will be sorted alphabetically now
      $local_accounts_array[$count+1] = $local_account_num;
    }
  }
  else
  {
    PrintErrorHeader();
    print "<h1>Error: No account names returned!</h1>\n";
    print "1020: Tried to get the names for the local accounts from the main database.  None were returned. Perhaps the main database does not have any accounts?<br>$sys_admin";
    CleanUp();
  }
}
else
{
  PrintErrorHeader();
  print "<h1>Error: Accounts not configured!</h1>\n";
  print "1021: It appears that there are no accounts assigned to this ledger.  Assign some accounts to continue.<br>$sys_admin";
  CleanUp();
}

print "<head><title>$prog_name $prog_version Initial Account Balance Activity</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
<script language=\"Javascript1.2\">
  <!--
  function debit_select_change(debit, credit, amount)
  {
    if (debit.selectedIndex == 0 && credit.selectedIndex == 0)
    {
      amount.value = "";
    }
  }

  function credit_select_change(credit, debit, amount)
  {
    if (credit.selectedIndex == 0 && debit.selectedIndex == 0)
    {
      amount.value = "";
    }
  }

  function amount_change(debit, credit, amount)
  {
    if (debit.selectedIndex == 0 && credit.selectedIndex == 0)
    {
      amount.value = "";
    }
    else
    {
      // verify that the value entered in amount does not equal 0 and can be parsed as a number
      var my_parsed_amount = parseFloat(amount.value);
      // parse the number to determine if it makes sense as a number
      if (my_parsed_amount.toString() == "NaN")
      {
        alert("The amount does not seem to be a valid numeric value!");
        return false;
      }
      else if (my_parsed_amount == 0.00)
      {
        alert("A zero value for an amount does not make sense!");
        return false;
      }
      // round my_parsed_amount and assign it to the amount text field
      my_parsed_amount += 0.005;
      my_parsed_amount = my_parsed_amount.toString();
      if (my_parsed_amount.indexOf(".") != -1)
      {
        amount.value = my_parsed_amount.substring(0, my_parsed_amount.indexOf(".")+3);
      }
    }
  }

  function validate_form()
  {
    var count=0;

    // check that every row that has entries has both a debit account and a credit account selected and an amount entered
    for (count=0; count<$num_total_rows; count++)
    {
      if (eval("document.balance_form.amount_" + count + ".value") != "")
      {
        if (eval("document.balance_form.debit_select_" + count + ".selectedIndex") != 0 && eval("document.balance_form.credit_select_" + count + ".selectedIndex") == 0)
        {
          alert("For row containing Ref #: " + eval("document.balance_form.ref_num_" + count + ".value") + "\\\nMissing credit account!\\\nYou must enter both a debit and credit account.");
          return false;
        }
        else if (eval("document.balance_form.debit_select_" + count + ".selectedIndex") == 0 && eval("document.balance_form.credit_select_" + count + ".selectedIndex") != 0)
        {
          alert("For row containing Ref #: " + eval("document.balance_form.ref_num_" + count + ".value") + "\\\nMissing debit account!\\\nYou must enter both a debit and credit account.");
          return false;
        }
        // verify that the value entered in amount does not equal 0 and can be parsed as a number
        var my_amount = eval("document.balance_form.amount_" + count);
        var my_parsed_amount = parseFloat(my_amount.value);
        // parse the number to determine if it makes sense as a number
        if (my_parsed_amount.toString() == "NaN")
        {
          alert("For row containing Ref #: " + eval("document.balance_form.ref_num_" + count + ".value") + "\\\nThe amount does not seem to be a valid numeric value!");
          return false;
        }
        else if (my_parsed_amount == 0.00)
        {
          alert("For row containing Ref #: " + eval("document.balance_form.ref_num_" + count + ".value") + "\\\nA zero value for an amount does not make sense!");
          return false;
        }
        // round my_parsed_amount and assign it to the amount text field
        my_parsed_amount += 0.005;
        my_parsed_amount = my_parsed_amount.toString();
        if (my_parsed_amount.indexOf(".") != -1)
        {
          eval("document.balance_form.amount_" + count + ".value = my_parsed_amount.substring(0, my_parsed_amount.indexOf('.')+3");
        }
      }
    }
    return true;
  }

  //-->
</script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left width=47%><font color=\"red\">";
print "Working with Initial Account Balances in Ledger <font color=\"$ledger_color\">$ledger_name</font></font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#initial_balance\" target=\"docs\"><img src=\"$site_dir/images/help.jpg\" alt=\"Help\" border=0></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);

print "<form method=post name=\"balance_form\" action=\"$site_dir/cgi-bin/initial_balance_update.cgi\">\n";
print "<input type=hidden name=\"fiscal_year\" value=\"$current_working_year\">\n";
print "<input type=hidden name=\"total_rows\" value=\"$num_total_rows\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left><input type=submit name=\"update_balances1\" value=\"Update Balances\" onClick=\"return validate_form();\"></td>\n";
print "<td align=right><input type=reset></td>\n";
print "</tr>\n";
print "</table>\n";
# display a table with all the beginning balance entries already in the database and num_extra_rows rows after them or just
# num_extra_rows rows if no records in database table
print "<table border=1 width=100% cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<th>Ref #</th><th>Debit Account</th><th>Credit Account</th><th>Amount</th><th>Fiscal Year</th>\n";
print "</tr>\n";
for ($count=0; $count < $num_total_rows; $count++)
{
  print "<tr>\n";
  print "<td align=center>$ref_num_array[$count]<input type=hidden name=\"ref_num_$count\" value=\"$ref_num_array[$count]\"></td>\n";
  print "<td align=center><select name=\"debit_select_$count\" onChange=\"debit_select_change(document.balance_form.debit_select_$count, document.balance_form.credit_select_$count, document.balance_form.amount_$count);\">\n";
  for ($count2=0; $count2 < @local_accounts_array; $count2++)
  {
    print "<option";
    if ($debit_accounts_array[$count] == $local_accounts_array[$count2])
    {
      print " selected";
    }
    print " value=\"$local_accounts_array[$count2]\">" . $local_accounts_names{$local_accounts_array[$count2]} . "\n";
  }
  print "</select></td>\n";
  print "<td align=center><select name=\"credit_select_$count\" onChange=\"credit_select_change(document.balance_form.credit_select_$count, document.balance_form.debit_select_$count, document.balance_form.amount_$count);\">\n";
  for ($count2=0; $count2 < @local_accounts_array; $count2++)
  {
    print "<option";
    if ($credit_accounts_array[$count] == $local_accounts_array[$count2])
    {
      print " selected";
    }
    print " value=\"$local_accounts_array[$count2]\">" . $local_accounts_names{$local_accounts_array[$count2]} . "\n";
  }
  print "</select></td>\n";
  print "<td align=center><input type=text name=\"amount_$count\" value=\"$amounts_array[$count]\" onChange=\"amount_change(document.balance_form.debit_select_$count, document.balance_form.credit_select_$count, document.balance_form.amount_$count);\"></td>\n";
  print "<td align=center>$fiscal_year_array[$count]</td>\n";
  print "</tr>\n";
}
print "</table>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left><input type=submit name=\"update_balances2\" value=\"Update Balances\" onClick=\"return validate_form();\"></td>\n";
print "<td align=right><input type=reset></td>\n";
print "</tr>\n";
print "</table>\n";
print "</form>\n";

# print a horizontal rule to break the menu from the behind the scenes links
#print "<br><hr>\n";
print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;