#!/usr/bin/perl
#$Id: initial_balance_sheet.cgi,v 1.2 2001/10/22 23:24:44 pcxuser Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

# initial_balance_sheet.cgi  -  Created by Jason A. Pattie, 11/26/1999, updated
# for use with cascading style sheets.
# This script allows the user to create or edit an initial balance stored
# in the database.  If there are currently no initial balance records in
# the database, then nothing will be displayed (empty); otherwise,
# the records from the beginning_balance_entry_tb table will be loaded and
# displayed.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$count';
use vars '$count2';
use vars '$count3';
use vars '$temp_count';
use vars '$min_working_year';
use vars '$current_working_year';
use vars '$next_ref_num';
use vars '@ref_num_array';
use vars '@accounts_array';
use vars '@sub_accounts_array';
use vars '@amounts_array';
use vars '@dorcs_array';
use vars '$local_account_num';
use vars '%local_accounts_names';
use vars '%local_equity_accounts_names';
use vars '$local_equity_account';

use vars '$local_account_name';
use vars '$local_account_id';
use vars '$local_account_type';

use vars '$sub_account_name';
use vars '$sub_account_id';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $ledger_name;  # name of ledger to work with
my $ledger_db;
my $ledger_color;
my $result2;
my $result3;
my $date_from;
my $date_to;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected; $database_connected = 0;

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# check to see if the current fiscal year is the first year the accounting package is being used
# if the current working year is the lowest year in the known years table, then check to see if any initial balances have
# been previously created in order to determine whether to display the values already in the beginning_balance_entry_tb
# table or not
$result = $conn->exec("select min(year) from known_years_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1008: Tried to get the minimum known year from the known_years_tb.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0 || $result->ntuples > 1)
{
  PrintErrorHeader();
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "1009: " . $result->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
  CleanUp();
}
$min_working_year = $result->getvalue(0, 0);
$result = $conn->exec("select working_year from variables_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1010: Tried to get the current working year from the variables_tb.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0 || $result->ntuples > 1)
{
  PrintErrorHeader();
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "1011: " . $result->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
  CleanUp();
}
$current_working_year = $result->getvalue(0, 0);
# see if there are any close out dates that are inside the current fiscal year
$result = $conn->exec("SELECT DISTINCT(acct_bal_tb.bal_date) from acct_bal_tb, known_years_tb, variables_tb WHERE acct_bal_tb.ledger_code = '$ledger_code' AND known_years_tb.ledger_code = '$ledger_code' AND variables_tb.ledger_code = '$ledger_code' AND known_years_tb.year = variables_tb.working_year AND acct_bal_tb.bal_date >= known_years_tb.start_date AND acct_bal_tb.bal_date < known_years_tb.end_date");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "</td></tr></table>\n";
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to see if they have any closed out periods in the current fiscal year.<br>$sys_admin";
  CleanUp();
}
my $latest_closeout;
$latest_closeout = "";
if ($result->ntuples > 0)
{
  $latest_closeout = $result->getvalue(0,0);
}
if (($min_working_year != $current_working_year) || ($latest_closeout ne ""))
{
  PrintErrorHeader();
  print "<h1>Error: Access denied!</h1>\n";
  print "1016: The initial balances for accounts cannot be created or edited after the first year the system has been in use or after any period has been closed out.<br>$sys_admin";
  CleanUp();
}
$result = $conn->exec("select account_num, sub_account_num, amount, dorc from beginning_balance_entry_tb WHERE ledger_code = '$ledger_code' order by ref_num");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1017: Tried to get all the beginning balance entries created so far.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 0)
{
  # load the arrays with the values from the beginning_balance_entry_tb table
  for ($count=0; $count < $result->ntuples; $count++)
  {
    $ref_num_array[$count] = $count;
    $accounts_array[$count] = $result->getvalue($count, 0);
    $sub_accounts_array[$count] = $result->getvalue($count, 1);
    $amounts_array[$count] = $result->getvalue($count, 2);
    $dorcs_array[$count] = $result->getvalue($count, 3);
  }
}
# get a list of the local accounts to fill in the drop-down selects
$result2 = $conn->exec("select name, number from account_tb where number in (select acct_num from local_accounts_tb WHERE ledger_code = '$ledger_code') and type in (select num from acct_type_tb where first_digit <> 4 and first_digit <> 5) order by name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>1019: Tried to get the names for the local list of accounts.<br>$sys_admin";
  CleanUp();
}
if ($result2->ntuples > 0)
{
  for ($count=0; $count < $result2->ntuples; $count++)
  {
    # load hash with local account names
    $local_account_num = $result2->getvalue($count, 1);
    $local_accounts_names{$result2->getvalue($count, 0) . " - " . $local_account_num} = $local_account_num;
  }
}
else
{
  PrintErrorHeader();
  print "<h1>Error: No account names returned!</h1>\n";
  print "1020: Tried to get the names for the local accounts from the main database.  None were returned. Perhaps the main database does not have any accounts?<br>$sys_admin";
  CleanUp();
}
# get a list of the equity accounts assigned to this ledger
$result = $conn->exec("select name, number from account_tb where type = 7 and number in (select acct_num from local_accounts_tb where ledger_code = '$ledger_code') order by name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print "Tried to get the Equity accounts assigned to this ledger.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples <= 0)
{
  PrintErrorHeader();
  print "<h1>Error: No equity accounts returned!</h1>\n";
  print "No equity accounts have been assigned to this ledger.<br>You need to assign at least the Retained Earnings account to this ledger.";
  CleanUp();
}
for ($count=0; $count < $result->ntuples; $count++)
{
  $local_equity_account = $result->getvalue($count, 1);
  $local_equity_accounts_names{$result->getvalue($count, 0) . " - " . $local_equity_account} = $local_equity_account;
}

print "<head><title>$prog_name $prog_version Initial Account Balance Activity</title>\n";
print "<style type=\"text/css\">\n";
print "#InitialBalanceView {position:absolute; top:425; left:10; height:1000;}\n";
print "</style>\n";
print <<"END_OF_SCRIPT";
<script language="Javascript1.2">
<!--
  var isNav4, isIE4, isDOM;
  // check for browser capabilities
  // getElementById => DOM1
  // document.all => IE4
  // document.layers => Netscape 4 Layers
  if (window.document.getElementById)
  {
    isDOM = true;
    isIE4 = false;
    isNav4 = false;
  }
  else if (window.document.all)
  {
    isIE4 = true;
    isDOM = false;
    isNav4 = false;
  }
  else if (window.document.layers)
  {
    isNav4 = true;
    isDOM = false;
    isIE4 = false;
  }
  else
  {
    window.alert("Browser support for layers or cascading style sheets not detected!\\n");
  }
//-->
</script>
END_OF_SCRIPT
print "</head>\n";
print "<body bgcolor=$body_bgcolor onLoad=\"do_load();\">\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
<script language=\"Javascript1.2\">
  <!--
  var initial_balances_array = new Array();
  var reset_balances_array = new Array();
  var accounts_hash = new Array();
  var sub_accounts_hash = new Array();

  function create_sub_account(name, value)
  {
    this.name = name;
    this.value = value;
  }

END_OF_STYLE
# create the accounts_hash entries
$result2 = $conn->exec("select name, number, type from account_tb where number in (select acct_num from local_accounts_tb WHERE ledger_code = '$ledger_code') and type in (select num from acct_type_tb where first_digit <> 4 and first_digit <> 5) order by name");
if ($result2->resultStatus != PGRES_TUPLES_OK)
{
  print "</script>\n";
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to retrieve the local accounts for $ledger_db.<br>$sys_admin";
  CleanUp();
}
for ($count=0; $count < $result2->ntuples; $count++)
{
  $local_account_name = $result2->getvalue($count, 0);
  $local_account_id = $result2->getvalue($count, 1);
  $local_account_type = $result2->getvalue($count, 2);
  # get the sub-account type(s) this account is assigned to
  $result3 = $conn->exec("select sub_acct_type from acct_type_tb where num = (select type from account_tb where number = $local_account_id)");
  if ($result3->resultStatus != PGRES_TUPLES_OK)
  {
    print "</script>\n";
    print "<h1>Error: Accessing the database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to retrieve the sub-account type(s) this account is assigned to.<br>$sys_admin";
    CleanUp();
  }
  if ($result3->ntuples <= 0)
  {
    print "</script>\n";
    print "<h1>Error: No records returned!</h1>\n";
    print "Tried to retrieve the sub-account type this account is assigned to but no records were returned.<br>$sys_admin";
    CleanUp();
  }
  print "  accounts_hash[\"$local_account_name - $local_account_id\"] = new Array();\n";
  # assign the account's type to the first element of the accounts_hash array
  print "  accounts_hash[\"$local_account_name - $local_account_id\"][0] = $local_account_type;\n";
  print "  accounts_hash[\"$local_account_name - $local_account_id\"][1] = new Array();\n";
  my @sub_account_types = split /\,/, $result3->getvalue(0, 0);
  for ($count2=0; $count2 < scalar @sub_account_types; $count2++)
  {
    print "  accounts_hash[\"$local_account_name - $local_account_id\"][1][$count2] = \"$sub_account_types[$count2]\";\n";
  }
}

# create a sub account array entry for a value of "-1"
print "  sub_accounts_hash[\"-1\"] = new Array();\n";
print "  sub_accounts_hash[\"-1\"][0] = new create_sub_account(\"--------------------\", \"-1\");\n";
# build the sub-account hash from the sub-account type table
$result = $conn->exec("select num from sub_acct_type_tb order by num");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "</script>\n";
  print "<h1>Error: Accessing the database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the sub-account types from the sub_acct_type_tb table.<br>$sys_admin";
  CleanUp();
}
for ($count=0; $count < $result->ntuples; $count++)
{
  my $sub_account_type = $result->getvalue($count, 0);
  # get the sub accounts from the database for the current sub-account type
  $result3 = $conn->exec("select sub_acc_name, sub_acc_number from sub_acc_tb where type = $sub_account_type order by sub_acc_name");
  if ($result3->resultStatus != PGRES_TUPLES_OK)
  {
    print "</script>\n";
    print "<h1>Error: Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to retrieve name and id information for sub accounts associated with sub-account type = $sub_account_type.<br>$sys_admin";
    CleanUp();
  }
  print "  sub_accounts_hash[\"$sub_account_type\"] = new Array();\n";
  print "  sub_accounts_hash[\"$sub_account_type\"][0] = new create_sub_account(\"--------------------\", \"-1\");\n";
  for ($count2=0; $count2 < $result3->ntuples; $count2++)
  {
    $sub_account_name = $result3->getvalue($count2, 0);
    $sub_account_id = $result3->getvalue($count2, 1);
    $temp_count = $count2+1;
    print "  sub_accounts_hash[\"$sub_account_type\"][$temp_count] = new create_sub_account(\"$sub_account_name - $sub_account_id\", $sub_account_id);\n";
  }
}

print "  var site_dir = \"$site_dir\"\n";

print <<"END_OF_STYLE";

  function do_load()
  {
    var f = document.balance_row_entry_form;

END_OF_STYLE
# load the initial_balances_array and reset_balances_array with values from the database
for ($count=0; $count < @ref_num_array; $count++)
{
  print "    initial_balances_array[$count] = new Array();\n";
  print "    reset_balances_array[$count] = new Array();\n";

  print "    initial_balances_array[$count]['rf'] = $count;\n";
  print "    reset_balances_array[$count]['rf'] = $count;\n";

  print "    initial_balances_array[$count]['ac'] = new Array();\n";
  print "    reset_balances_array[$count]['ac'] = new Array();\n";
  print "    initial_balances_array[$count]['ac'][0] = \"$accounts_array[$count]\";\n";
  print "    reset_balances_array[$count]['ac'][0] = \"$accounts_array[$count]\";\n";
  print "    for (var count=0; count < f.account_select.options.length; count++)\n";
  print "    {\n";
  print "      if (f.account_select.options[count].value == \"$accounts_array[$count]\")\n";
  print "      {\n";
  print "        initial_balances_array[$count]['ac'][1] = f.account_select.options[count].text;\n";
  print "        reset_balances_array[$count]['ac'][1] = f.account_select.options[count].text;\n";
  print "      }\n";
  print "    }\n";

  print "    initial_balances_array[$count]['sa'] = new Array();\n";
  print "    reset_balances_array[$count]['sa'] = new Array();\n";
  print "    initial_balances_array[$count]['sa'][0] = \"$sub_accounts_array[$count]\";\n";
  print "    reset_balances_array[$count]['sa'][0] = \"$sub_accounts_array[$count]\";\n";
  print "    for (var count in accounts_hash[initial_balances_array[$count]['ac'][1]][1])\n";
  print "    {\n";
  print "      var accounts_index = accounts_hash[initial_balances_array[$count]['ac'][1]][1];\n";
  print "      for (var count1=0; count1 < sub_accounts_hash[accounts_index[count]].length; count1++)\n";
  print "      {\n";
  print "        if (sub_accounts_hash[accounts_index[count]][count1].value == $sub_accounts_array[$count])\n";
  print "        {\n";
  print "          initial_balances_array[$count]['sa'][1] = sub_accounts_hash[accounts_index[count]][count1].name;\n";
  print "          reset_balances_array[$count]['sa'][1] = sub_accounts_hash[accounts_index[count]][count1].name;\n";
  print "        }\n";
  print "      }\n";
  print "    }\n";

  if ($dorcs_array[$count] eq "t")
  {
    print "    initial_balances_array[$count]['db'] = \"$amounts_array[$count]\";\n";
    print "    reset_balances_array[$count]['db'] = \"$amounts_array[$count]\";\n";
    print "    initial_balances_array[$count]['cr'] = \"\"\n";
    print "    reset_balances_array[$count]['cr'] = \"\"\n";
  }
  else
  {
    print "    initial_balances_array[$count]['db'] = \"\"\n";
    print "    reset_balances_array[$count]['db'] = \"\"\n";
    print "    initial_balances_array[$count]['cr'] = \"$amounts_array[$count]\";\n";
    print "    reset_balances_array[$count]['cr'] = \"$amounts_array[$count]\";\n";
  }
}
print <<"END_OF_STYLE";
    rewrite_sheet();
    calculate_difference();
    blank_row_entry();
    f.equity_account_select.focus();
  }

  function account_select_change()
  {
    var f = document.balance_row_entry_form;
    var account_value;

    // fill in the sub-account select with the values associated with the selected account
    // clear the sub_account_select
    f.sub_account_select.length = 0;
    account_value = f.account_select.options[f.account_select.selectedIndex].text;
    if (typeof accounts_hash[account_value] == "undefined")
    {
      f.sub_account_select.options[0] = new Option(sub_accounts_hash["-1"][0].name, "-1");
    }
    else
    {
      for (var count in accounts_hash[account_value][1])
      {
        var accounts_index = accounts_hash[account_value][1];
        for (var count1=0; count1 < sub_accounts_hash[accounts_index[count]].length; count1++)
        {
          f.sub_account_select.options[f.sub_account_select.options.length] = new Option(sub_accounts_hash[accounts_index[count]][count1].name, sub_accounts_hash[accounts_index[count]][count1].value);
        }
      }
    }
    f.sub_account_select.selectedIndex = 0;
  }

  function debit_amount_change()
  {
    var f = document.balance_row_entry_form;
    var temp_amount;

    if (isNaN(parseFloat(f.debit_amount.value)))
    {
      alert("Debit amount: " + f.debit_amount.value + " is not a valid numeric value!\\n");
      f.debit_amount.value = "";
      f.debit_amount.focus();
      return;
    }

    // enter the value into the credit field if negative
    if (parseFloat(f.debit_amount.value) < 0.0)
    {
      // swap values
      f.credit_amount.value = Math.abs(parseFloat(f.debit_amount.value));
      f.debit_amount.value = "";
      credit_amount_change();
      return;
    }

    // round the amount
    temp_amount = parseFloat(f.debit_amount.value) + 0.005;
    temp_amount = temp_amount.toString();
    f.debit_amount.value = temp_amount.substring(0, temp_amount.indexOf(".")+3);

    // clear the credit amount field
    f.credit_amount.value = "";
  }

  function credit_amount_change()
  {
    var f = document.balance_row_entry_form;
    var temp_amount;

    if (isNaN(parseFloat(f.credit_amount.value)))
    {
      alert("Credit amount: " + f.credit_amount.value + " is not a valid numeric value!\\n");
      f.credit_amount.value = "";
      f.credit_amount.focus();
      return;
    }

    // enter the value into the debit field if negative
    if (parseFloat(f.credit_amount.value) < 0.0)
    {
      // swap values
      f.debit_amount.value = Math.abs(parseFloat(f.credit_amount.value));
      f.credit_amount.value = "";
      debit_amount_change();
      return;
    }

    // round the amount
    temp_amount = parseFloat(f.credit_amount.value) + 0.005;
    temp_amount = temp_amount.toString();
    f.credit_amount.value = temp_amount.substring(0, temp_amount.indexOf(".")+3);

    // clear the debit amount field
    f.debit_amount.value = "";
  }

  function rewrite_sheet()
  {
    var bodyString;
    var count;
    var db_sum, cr_sum, temp;

    db_sum = 0.0;
    cr_sum = 0.0;

    bodyString =  '<form>\\n' +
    		  '<table border=0 cellspacing=0 cellpadding=0 bgcolor=\"black\" width=\"100%\">\\n' +
    		  '<tr>\\n' +
    		  '<td>\\n' +
    		  '<table border=0 cellspacing=1 cellpadding=1 bgcolor=\"black\" width=\"100%\">\\n' +
    		  '<tr bgcolor=\"#dddddd\">\\n' +
    		  '<th width=\"8%\" nowrap>Ref#</th><th width=\"60%\" nowrap>Account</th><th width=\"8%\" nowrap>Sub-Account</th><th width=10 nowrap>Debits&nbsp;</th><th width=10 nowrap>Credits</th><th width=\"4%\">Edit</th><th width=\"4%\">Delete</th>\\n' +
    		  '</tr>\\n';
    for (count=0; count < initial_balances_array.length; count++)
    {
      bodyString += '<tr bgcolor=\"#eeeeee\" align=center>\\n';
      initial_balances_array[count]['rf'] = count;
      bodyString += '<td nowrap>&nbsp;' + ((initial_balances_array[count]['rf'] == "") ? '&nbsp;' : initial_balances_array[count]['rf']) + '&nbsp;</td>\\n';
      bodyString += '<td nowrap>&nbsp;' + initial_balances_array[count]['ac'][1] + '&nbsp;</td>\\n';
      bodyString += '<td nowrap>&nbsp;' + ((initial_balances_array[count]['sa'][1] == "" || initial_balances_array[count]['sa'][0] == "-1" || initial_balances_array[count]['sa'][0] == -1) ? '&nbsp;' : initial_balances_array[count]['sa'][1]) + '&nbsp;</td>\\n';
      bodyString += '<td align=right nowrap>&nbsp;' + ((initial_balances_array[count]['db'] == "") ? '&nbsp' : commify(initial_balances_array[count]['db'])) + '&nbsp;&nbsp;</td>\\n';
      if (initial_balances_array[count]['db'] != "")
      {
        db_sum += parseFloat(initial_balances_array[count]['db']);
      }
      bodyString += '<td align=right nowrap>&nbsp;' + ((initial_balances_array[count]['cr'] == "") ? '&nbsp;' : commify(initial_balances_array[count]['cr'])) + '&nbsp;&nbsp;</td>\\n';
      if (initial_balances_array[count]['cr'] != "")
      {
        cr_sum += parseFloat(initial_balances_array[count]['cr']);
      }
      bodyString += '<td align=center><font size=-2><input type=button value=\"E\" onClick=\"edit_balance_row(' + count + ');\"></font></td>\\n';
      bodyString += '<td align=center><font size=-2><input type=button value=\"D\" onClick=\"delete_balance_row(' + count + ');\"></font></td>\\n';
      bodyString += '</tr>\\n';
    }
    temp = parseFloat(db_sum) + 0.005;
    temp = temp.toString();
    db_sum = temp.substring(0, temp.indexOf('.') + 3);
    temp = parseFloat(cr_sum) + 0.005;
    temp = temp.toString();
    cr_sum = temp.substring(0, temp.indexOf('.') + 3);
    bodyString += '<tr bgcolor=\"#ffffff\">\\n' +
    		    '<th align=right colspan=3>Sums&nbsp;</th>\\n' +
    		    '<td align=right>' + commify(db_sum) + '&nbsp;&nbsp;</td>\\n' +
    		    '<td align=right>' + commify(cr_sum) + '&nbsp;&nbsp;</td>\\n' +
    		    '<td colspan=2>&nbsp;</td>\\n' +
    		    '</tr>\\n';
    		
    bodyString += '</table>\\n' +
    		  '</td>\\n' +
    		  '</tr>\\n' +
    		  '</table>\\n' +
    		  '</form>\\n';
    if (isDOM)
    {
      // get the element that contains the CSS sheet
      var ibvElement = window.document.getElementById('InitialBalanceView');
      ibvElement.innerHTML = bodyString;
    }
    else if (isNav4)
    {
      // hide the sheet
      window.document.InitialBalanceView.visibility = "hide";
      window.document.InitialBalanceView.document.writeln(bodyString);
      window.document.InitialBalanceView.document.close();
      // reshow the sheet
      window.document.InitialBalanceView.visibility = "show";
    }
    else if (isIE4)
    {
      // hide the sheet
      window.document.all.InitialBalanceView.visibility = "hidden";
      window.document.all.InitialBalanceView.innerHTML = bodyString;
      // reshow the sheet
      window.document.all.InitialBalanceView.visibility = "visible";
    }
    window.status = "";
  }

  function calculate_difference()
  {
    var sum_debits, sum_credits;
    var debit_diff, credit_diff;
    var temp_diff, temp1;
    var f = document.balance_row_entry_form;

    sum_debits = 0.0;
    sum_credits = 0.0;
    temp_diff = 0.0;

    debit_diff = "";
    credit_diff = "";

    // run through the initial_balances_array and sum all debits and credits
    for (var count=0; count < initial_balances_array.length; count++)
    {
      if (initial_balances_array[count]['db'] != "")
      {
        // debit
        sum_debits += parseFloat(initial_balances_array[count]['db']);
      }
      else if (initial_balances_array[count]['cr'] != "")
      {
        // credit
        sum_credits += parseFloat(initial_balances_array[count]['cr']);
      }
    }
    // calculate the difference of the debits and credits
    if (sum_debits > sum_credits)
    {
      temp_diff = parseFloat(sum_debits - sum_credits);
      // round temp_diff
      temp1 = parseFloat(temp_diff) + 0.005;
      temp1 = temp1.toString();
      temp_diff = temp1.substring(0, temp1.indexOf(".")+3);
      credit_diff = temp_diff;
    }
    else if (sum_debits < sum_credits)
    {
      temp_diff = parseFloat(sum_credits - sum_debits);
      // round temp_diff
      temp1 = parseFloat(temp_diff) + 0.005;
      temp1 = temp1.toString();
      temp_diff = temp1.substring(0, temp1.indexOf(".")+3);
      debit_diff = temp_diff;
    }

    // determine if the difference is a debit or credit and place it in the corresponding field for the selected equity account
    f.equity_debit_amount.value = debit_diff;
    f.equity_credit_amount.value = credit_diff;
  }

  function commify(f)
  {
    var value;
    var temp;
    var count = 0;
    var start, comma_count;

    start = false;
    comma_count = 0;

    // round the value to the nearest two decimal places
    temp = parseFloat(f) + 0.005;
    temp = temp.toString();
    value = temp.substring(0, temp.indexOf('.') + 3);
    temp = "";
    // reverse the array adding the commas to it as we go
    for (count=value.length-1; count >= 0; count--)
    {
      if (value.charAt(count) == '.')
      {
        start = true;
      }
      temp += value.charAt(count);
      if (start && comma_count == 3 && count > 0)
      {
        comma_count = 1;
        temp += ',';
      }
      else if (start)
      {
        comma_count++;
      }
    }
    if (temp.length == 3)
    {
      // add a zero to the "beginning" of the array
      temp += '0';
    }
    // turn the array back around
    value = "";
    for (count=temp.length-1; count >= 0; count--)
    {
      value += temp.charAt(count);
    }

    return value;
  }

  function edit_balance_row(n)
  {
    var ac, sa, db, cr, rf;
    var f = document.balance_row_entry_form;

    // copy the contents of the array values for the selected row into the balance_row_entry_form fields
    ac = initial_balances_array[n]['ac'][0].toString();
    sa = initial_balances_array[n]['sa'][0].toString();
    db = initial_balances_array[n]['db'];
    cr = initial_balances_array[n]['cr'];
    rf = initial_balances_array[n]['rf'];
    // delete the array values for the selected row
    delete_balance_row(n);
    // assign to and process the form field values
    f.ref_num.value = rf;
    for (var count=0; count < f.account_select.options.length; count++)
    {
      if (f.account_select.options[count].value == ac)
      {
        f.account_select.selectedIndex = count;
      }
    }
    account_select_change();
    for (var count=0; count < f.sub_account_select.options.length; count++)
    {
      if (f.sub_account_select.options[count].value == sa)
      {
        f.sub_account_select.selectedIndex = count;
      }
    }
    f.debit_amount.value = db;
    f.credit_amount.value = cr;
    // place the focus on the account select
    document.balance_row_entry_form.account_select.focus();
  }

  function delete_balance_row(n)
  {
    // remove the row of data from the JavaScript array
    // move all values down one in the array
    for (var count=parseInt(n)+1; count < initial_balances_array.length; count++)
    {
      initial_balances_array[count-1]['ac'][0] = initial_balances_array[count]['ac'][0];
      initial_balances_array[count-1]['ac'][1] = initial_balances_array[count]['ac'][1];
      initial_balances_array[count-1]['sa'][0] = initial_balances_array[count]['sa'][0];
      initial_balances_array[count-1]['sa'][1] = initial_balances_array[count]['sa'][1];
      initial_balances_array[count-1]['db'] = initial_balances_array[count]['db'];
      initial_balances_array[count-1]['cr'] = initial_balances_array[count]['cr'];
      initial_balances_array[count-1]['rf'] = initial_balances_array[count]['rf'];
    }
    // shrink the array size by one
    if (initial_balances_array.length > 0)
    {
      initial_balances_array.length--;
    }
    // redraw the sheet
    rewrite_sheet();
    // calculate the difference to balance the total debits and credits being entered on the screen and place the difference
    // into the selected equity account's debit or credit field
    calculate_difference();
    // place the focus on the account select
    document.balance_row_entry_form.account_select.focus();
  }

  function enter_row_entry()
  {
    var f = document.balance_row_entry_form;

    // validate row entry
    if (!validate_row_entry())
    {
      return;
    }

    // add the row values to the values array
    var arrayElement = initial_balances_array.length;
    initial_balances_array[arrayElement] = new Array();
    initial_balances_array[arrayElement]['ac'] = new Array();
    initial_balances_array[arrayElement]['ac'][0] = f.account_select.options[f.account_select.selectedIndex].value;
    initial_balances_array[arrayElement]['ac'][1] = f.account_select.options[f.account_select.selectedIndex].text;
    initial_balances_array[arrayElement]['sa'] = new Array();
    initial_balances_array[arrayElement]['sa'][0] = f.sub_account_select.options[f.sub_account_select.selectedIndex].value;
    initial_balances_array[arrayElement]['sa'][1] = f.sub_account_select.options[f.sub_account_select.selectedIndex].text;
    initial_balances_array[arrayElement]['db'] = f.debit_amount.value;
    initial_balances_array[arrayElement]['cr'] = f.credit_amount.value;
    initial_balances_array[arrayElement]['rf'] = f.ref_num.value;

    // blank row entry
    blank_row_entry();

    // calculate the difference
    calculate_difference();

    // rewrite the sheet
    rewrite_sheet();

    f.account_select.focus();
  }

  function blank_row_entry()
  {
    var f = document.balance_row_entry_form;
    var matchfound = true;

    f.account_select.selectedIndex = 0;
    account_select_change();
    f.debit_amount.value = "";
    f.credit_amount.value = "";
    f.ref_num.value = initial_balances_array.length;

    f.account_select.focus();
  }

  function validate_row_entry()
  {
    var f = document.balance_row_entry_form;
    var errorString = "";
    var focus_element;

    if (f.account_select.selectedIndex <= 0)
    {
      errorString += "An account must be entered!\\n";
      focus_element = f.account_select;
    }
    if (f.debit_amount.value == "" && f.credit_amount.value == "")
    {
      errorString += "A debit or credit amount must be entered!\\n";
      if (!focus_element)
      {
        focus_element = f.debit_amount;
      }
    }
    else if (f.debit_amount.value != "")
    {
      // verify that the value entered is numerical
      if (isNaN(parseFloat(f.debit_amount.value)))
      {
        errorString += "Debit: " + f.debit_amount.value + " is not a valid numeric value!\\n";
        if (!focus_element)
        {
          focus_element = f.debit_amount;
        }
      }
    }
    else if (f.credit_amount.value != "")
    {
      // verify that the value entered is numerical
      if (isNaN(parseFloat(f.credit_amount.value)))
      {
        errorString += "Credit: " + f.credit_amount.value + " is not a valid numeric value!\\n";
        if (!focus_element)
        {
          focus_element = f.credit_amount;
        }
      }
    }

    if (errorString != "")
    {
      alert(errorString);
      if (focus_element)
      {
        focus_element.focus();
      }
      return false;
    }

    return true;
  }

  function validate_form()
  {
    var f = document.balance_row_entry_form;
    var sum_debits, sum_credits;
    var temp_db, temp_cr;
    var temp_diff;
    var errorString = "";

    sum_debits = parseFloat(f.equity_debit_amount.value);
    sum_credits = parseFloat(f.equity_credit_amount.value);
    if (isNaN(sum_debits))
    {
      sum_debits = 0.0;
    }
    if (isNaN(sum_credits))
    {
      sum_credits = 0.0;
    }

    // verify that an equity account has been entered to make balance transactions against
    if (f.equity_account_select.selectedIndex <= 0)
    {
      errorString += "No equity account has been selected!\\n";
    }

    // do not allow them to enter a blank set of account balances
    if (initial_balances_array.length == 0)
    {
      errorString += "No balance entries were made!  Please enter some data.\\n";
    }

    // sum the debits and credits
    for (var count=0; count < initial_balances_array.length; count++)
    {
      if (initial_balances_array[count]['db'] != "")
      {
        sum_debits += parseFloat(initial_balances_array[count]['db']);
      }
      else if (initial_balances_array[count]['cr'] != "")
      {
        sum_credits += parseFloat(initial_balances_array[count]['cr']);
      }
      else
      {
        errorString += "No debit or credit amount was entered for Ref#: " + initial_balances_array[count]['rf'] + "\\n";
      }
    }
    // check to make sure debits equal credits
    temp_db = parseFloat(sum_debits) + 0.005;
    temp_db = temp_db.toString();
    sum_debits = temp_db.substring(0, temp_db.indexOf(".") + 3);

    temp_cr = parseFloat(sum_credits) + 0.005;
    temp_cr = temp_cr.toString();
    sum_credits = temp_cr.substring(0, temp_cr.indexOf(".") + 3);

    if (sum_debits != sum_credits)
    {
      errorString += "Debits do not equal Credits!\\n     Debits: " + commify(sum_debits) + "\\n    Credits: " + commify(sum_credits) + "\\n";
    }

    if (errorString != "")
    {
      alert(errorString);
      return false;
    }

    return true;
  }

  function update_values()
  {
    var f = document.balance_row_entry_form;

    f.initial_balances_values.value = "";

    // copy the values of the initial_balances_array into the hidden form value to pass to the next script
    f.initial_balances_values.value = "";
    for (var count=0; count < initial_balances_array.length; count++)
    {
      f.initial_balances_values.value += 'ac=' + initial_balances_array[count]['ac'][0] +
      					 '|sa=' + initial_balances_array[count]['sa'][0] +
      					 '|db=' + initial_balances_array[count]['db'] +
      					 '|cr=' + initial_balances_array[count]['cr'] +
      					 '|rf=' + initial_balances_array[count]['rf'];
      if (count < initial_balances_array.length - 1)
      {
        f.initial_balances_values.value += ';';
      }
    }
  }

  function submit_form()
  {
    // validate the form entries
    if (!validate_form())
    {
      return false;
    }

    // update the hidden value that holds the form balances information to be passed to the next script
    update_values();

    return true;
  }

  function reset_balances_form()
  {
    var f = document.balance_row_entry_form;

    // delete all entries from the initial_balances_array
    initial_balances_array.length = 0;

    // clear the row entry forms
    blank_row_entry();
    f.equity_account_select.selectedIndex = 0;
    f.equity_debit_amount.value = "";
    f.equity_credit_amount.value = "";

    // copy the values from the reset_balances_array to the initial_balances_array
    for (var count=0; count < reset_balances_array.length; count++)
    {
      initial_balances_array[count] = new Array();
      initial_balances_array[count]['rf'] = reset_balances_array[count]['rf'];
      initial_balances_array[count]['ac'] = new Array();
      initial_balances_array[count]['ac'][0] = reset_balances_array[count]['ac'][0];
      initial_balances_array[count]['ac'][1] = reset_balances_array[count]['ac'][1];
      initial_balances_array[count]['sa'] = new Array();
      initial_balances_array[count]['sa'][0] = reset_balances_array[count]['sa'][0];
      initial_balances_array[count]['sa'][1] = reset_balances_array[count]['sa'][1];
      initial_balances_array[count]['db'] = reset_balances_array[count]['db'];
      initial_balances_array[count]['cr'] = reset_balances_array[count]['cr'];
    }

    // calculate difference value
    calculate_difference();

    // redraw the sheet
    rewrite_sheet();

    // set the focus to the equity account select
    f.equity_account_select.focus();
  }

  //-->
</script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left width=47%><font color=\"red\">";
print "Working with Initial Account Balances in Ledger <font color=\"$ledger_color\">$ledger_name</font></font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#initial_balance\" target=\"docs\"><img src=\"$site_dir/images/help.jpg\" alt=\"Help\" border=0></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);

print "<center>Fiscal Year: $current_working_year</center>\n";
print "<form method=post name=\"balance_row_entry_form\" action=\"$site_dir/cgi-bin/initial_balance_update_sheet.cgi\">\n";
print "<input type=hidden name=\"fiscal_year\" value=\"$current_working_year\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"initial_balances_values\" value=\"\">\n";
print "<input type=hidden name=\"ref_num\" value=\"\">\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left><input type=submit name=\"update_balances\" value=\"Update Balances\" onClick=\"return submit_form();\"></td>\n";
print "<td align=right><input type=button name=\"reset_form\" value=\"Reset\" onClick=\"reset_balances_form();\"></td>\n";
print "</tr>\n";
print "</table>\n";
# display a table with form elements to allow the user to select the retained earnings account to use for making debits/credits
# against to balance the initial balances entries
print "<table border=0 width=\"100%\" cellspacing=0 cellpadding=0 bgcolor=\"black\">\n";
print "<tr>\n";
print "<td>\n";
print "<table border=0 width=\"100%\" cellspacing=1 cellpadding=1 bgcolor=\"black\">\n";
print "<tr bgcolor=\"white\">\n";
print "<th>Equity Account</th><th>Debit</th><th>Credit</th>\n";
print "</tr>\n";
print "<tr bgcolor=\"white\" align=center>\n";
print "<td><select name=\"equity_account_select\">\n";
print "<option value=\"-1\">-------------------------\n";
foreach $count (sort keys %local_equity_accounts_names)
{
  print "<option value=\"$local_equity_accounts_names{$count}\">$count\n";
}
print "</select></td>\n";
print "<td><input type=text name=\"equity_debit_amount\" size=12 value=\"\" onFocus=\"document.balance_row_entry_form.equity_debit_amount.blur(); document.balance_row_entry_form.account_select.focus();\"></td>\n";
print "<td><input type=text name=\"equity_credit_amount\" size=12 value=\"\" onFocus=\"document.balance_row_entry_form.equity_credit_amount.blur(); document.balance_row_entry_form.account_select.focus();\"></td>\n";
print "</tr>\n";
print "</table>\n";
print "</td>\n";
print "</tr>\n";
print "</table>\n";
print "<br>\n";
# display a table with form elements to allow the user to make initial balance entries for accounts and sub-accounts
print "<table border=0 width=\"100%\" cellspacing=0 cellpadding=0 bgcolor=\"black\">\n";
print "<tr>\n";
print "<td>\n";
print "<table border=0 width=\"100%\" cellspacing=1 cellpadding=1 bgcolor=\"black\">\n";
print "<tr bgcolor=\"white\">\n";
print "<th>Account</th><th>Sub-Account</th><th>Debit</th><th>Credit</th><th>&nbsp;</th>\n";
print "</tr>\n";
print "<tr bgcolor=\"white\" align=center>\n";
print "<td><select name=\"account_select\" onChange=\"account_select_change();\">\n";
print "<option value=\"-1\">-------------------------\n";
foreach $count (sort keys %local_accounts_names)
{
  print "<option value=\"$local_accounts_names{$count}\">$count\n";
}
print "</select></td>\n";
print "<td><select name=\"sub_account_select\">\n";
print "<option value=\"-1\">--------------------\n";
print "</select></td>\n";
print "<td><input type=text name=\"debit_amount\" size=12 value=\"\" onChange=\"debit_amount_change();\"></td>\n";
print "<td><input type=text name=\"credit_amount\" size=12 value=\"\" onChange=\"credit_amount_change();\"></td>\n";
print "<td><input type=button name=\"enter_row\" value=\"Enter\" onClick=\"enter_row_entry();\">\n";
print "<br><input type=button name=\"clear_row\" value=\"Clear\" onClick=\"blank_row_entry();\"></td>\n";
print "</tr>\n";
print "</table>\n";
print "</td>\n";
print "</tr>\n";
print "</table>\n";
print "</form>\n";

# the HTML element for the cascading style sheet view
print "<div id=\"InitialBalanceView\"></div>\n";

print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
