#!/usr/bin/perl

# journal_activity_update.cgi  -  Created by Jason A. Pattie, 09/28/1999
# This script creates or updates the modified journal entry in the database.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$count';
use vars '$journal_code';
use vars '$trans_date';
use vars '$payee_payer';
use vars '$memo';
use vars '$remember';
use vars '$ref_num';
use vars '$num_rows';
use vars '$temp';
use vars '$temp1';
use vars '$temp2';
use vars '$temp3';
use vars '$amount';
use vars '$dorc';
use vars '$sub_entity_element';
use vars '$account_element';
use vars '$sub_account_element';
use vars '$sub_account_text';
use vars '$cleared';
use vars '$journal_command';
use vars '$journal_view_command';
use vars '$journal_display_string';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $mode;  # modes are "new" and "edit"
my $ledger_name;  # name of ledger to work with
my $ledger_db;
my $ledger_color;
my $journal_name;
my $result2;
my $date_from;
my $date_to;
my $sort_journal;
my $sort_by;
my $journal_bgcolor;
my $num_journal_entries;
#my $use_java;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

if (not (exists $input{'edit_submit'} or exists $input{'new_submit'}))
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Invalid options.<br>$sys_admin";
  CleanUp();
}

$ledger_db = $input{'ledger_db'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

$journal_code = $input{'journal_code'};
if (length $journal_code == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a journal to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

$journal_bgcolor = $input{'journal_bgcolor'};
if (length $journal_bgcolor == 0)
{
  $journal_bgcolor = "deepskyblue";
}

# validate we got valid input from the user.
if (exists $input{'edit_submit'})
{
  $mode = "edit";
}
elsif (exists $input{'new_submit'})
{
  $mode = "new";
}
else
{
  PrintErrorHeader();
  print "<h1>Access Denied!</h1>\n";
  print "Error: Invalid options given!\n<br>$sys_admin";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# now make sure that the journal exists that we are supposed to work with
$result = $conn->exec("select name from journal_tb where code = '$journal_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database <font color=\"blue\">$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate Journal with code = <font color=\"blue\">$journal_code</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Journal with code = <font color=\"blue\">$journal_code</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
$journal_name = $result->getvalue(0, 0);

if ($mode eq "new")
{
  # verify that check/deposit numbers that are used on the form do not already exist in the database
  $num_rows = $input{'num_rows'};
  for ($count=0; $count<$num_rows; $count++)
  {
    # find only those rows that have debits/credits entered to check for check/deposit numbers
    $temp1 = "debit_" . $count;
    $temp2 = "credit_" . $count;
    $temp3 = "account_select_" . $count;

    if (exists $input{$temp1} and exists $input{$temp2} and exists $input{$temp3} and not ($input{$temp1} eq "" and $input{$temp2} eq "" and s/\ /$input{$temp3}/g eq ""))
    {
      if (length $input{$temp1} > 0)
      {
        $amount = $input{$temp1};
        $dorc = "true";
      }
      else
      {
        $amount = $input{$temp2};
        $dorc = "false";
      }
      $sub_entity_element =~ s/^ *//g;
      $temp = "account_select_" . $count;
      $account_element = $input{$temp};
      $temp = "sub_account_select_" . $count;
      $sub_account_element = $input{$temp};
      chomp $sub_account_element;
      $sub_account_element =~ s/^\ *//g;
      if ($sub_account_element eq "")
      {
        $sub_account_element = "-1";
      }
      $temp = "sub_account_text_" . $count;
      $sub_account_text = $input{$temp};
      chomp $sub_account_text;
      $sub_account_text =~ s/^\ *//g;
      if ($sub_account_element eq "C" or $sub_account_element eq "D")
      {
        # sub_account_text contains a check or deposit number; based on dorc value, determine whether check/deposit is already in use
        # see if check/deposit number values are already in database
        $result = $conn->exec("select parent_num from debitcredit_tb where account_num = $account_element and sub_account_num = $sub_account_text and dorc = $dorc AND ledger_code = '$ledger_code'");
        if ($result->resultStatus != PGRES_TUPLES_OK)
        {
          PrintErrorHeader();
          print "<h1>Error: Accessing database failed!</h1>\n";
          print $conn->errorMessage . "<br>Tried to get check/deposit numbers already in use.<br>$sys_admin";
          CleanUp();
        }
        if ($result->ntuples > 0)
        {
          PrintErrorHeader();
          print "<h1>Error: ";
          if ($dorc eq "true")
          {
            print "Deposit";
          }
          else
          {
            print "Check";
          }
          print " number <b>$sub_account_text</b> already in use in database!</h1>\n";
          print "$back_button";
          CleanUp();
        }
      }
    }
  }
}

print "<head><title>$prog_name $prog_version Journals Activity</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
if ($mode eq "edit")
{
  print "<td align=left width=47%><font color=\"red\">";
  print "Updating Journal <font color=\"blue\">$journal_name</font>";
}
elsif ($mode eq "new")
{
  print "<td align=left width=47%><font color=\"red\">";
  print "Creating Journal Entry in Journal <font color=\"blue\">$journal_name</font>";
}
print " in Ledger <font color=\"$ledger_color\">$ledger_name</font></font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#journal\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);

if ($mode eq "edit") # update the journal entry in journal_entry_tb, delete all references to the journal entry in debitcredit_tb, add them back to debitcredit_tb
{
  # update first portion of transaction (journal_entry_tb)
  $ref_num = $input{'ref_num'};
  $journal_code = $input{'journal_code'};
  $trans_date = $input{'trans_date'};
  $payee_payer = $input{'payee_payer'};
  # escape all ('), ($), (\) characters
  $payee_payer =~ s/\\/\\\\/g;
  $payee_payer =~ s/\'/\\\'/g;
  $payee_payer =~ s/\$/\\\$/g;
  $remember = $input{'remember'};
  $memo = $input{'memo'};
  # escape all ('), ($), (\) characters
  $memo =~ s/\\/\\\\/g;
  $memo =~ s/\'/\\\'/g;
  $memo =~ s/\$/\\\$/g;
  $result = $conn->exec("update journal_entry_tb set journal_code = '$journal_code', trans_date = '$trans_date', payee_payer = '$payee_payer', memo = '$memo', remember = $remember where ref_num = $ref_num AND ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Updating database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to update the information for journal entry $ref_num.<br>$sys_admin";
    CleanUp();
  }
  print "<table border=1 cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$body_bgcolor\">\n";
  print "<tr>\n";
  print "<th>Ref#</th><th>Jnl</th><th>Date</th><th>Payee-Payer</th><th>Remember</th><th>Memo</th>\n";
  print "</tr>\n";
  print "<tr align=center>\n";
  print "<td>$ref_num</td>\n";
  print "<td>$journal_code</td>\n";
  print "<td>$trans_date</td>\n";
  if ($payee_payer eq "")
  {
    print "<td><br></td>\n";
  }
  else
  {
    print "<td>$payee_payer</td>\n";
  }
  if ($remember eq "true")
  {
    print "<td>Yes</td>\n";
  }
  else
  {
    print "<td>No</td>\n";
  }
  if ($memo eq "")
  {
    print "<td><br></td>\n";
  }
  else
  {
    print "<td>$memo</td>\n";
  }
  print "</tr>\n";
  print "</table>\n";
  # delete values from debitcredit_tb with parent_num == ref_num of journal entry
  $result = $conn->exec("delete from debitcredit_tb where parent_num = $ref_num AND ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Deleting entries from debit/credit table failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to remove the entries for journal entry $ref_num.<br>$sys_admin";
    CleanUp();
  }
  print "<table border=1 cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$body_bgcolor\">\n";
  print "<tr>\n";
  print "<th>SbE</th><th>Account</th><th>SubAccount</th><th>Debit</th><th>Credit</th><th>Cleared</th><th>Counter</th>\n";
  print "</tr>\n";
  # insert values into debitcredit_tb with old and new information
  $num_rows = $input{'num_rows'};
  for ($count=0; $count<$num_rows; $count++)
  {
    # find only those rows that have debits/credits and account entered and insert them into the database
    $temp1 = "debit_" . $count;
    $temp2 = "credit_" . $count;
    $temp3 = "account_select_" . $count;

    if (exists $input{$temp1} and exists $input{$temp2} and exists $input{$temp3} and $input{$temp3} ne "-1" and not ($input{$temp1} eq "" and $input{$temp2} eq ""))
    {
      if (length $input{$temp1} > 0)
      {
        $amount = $input{$temp1};
        $dorc = "true";
      }
      else
      {
        $amount = $input{$temp2};
        $dorc = "false";
      }
      $temp = "sub_entity_select_" . $count;
      $sub_entity_element = $input{$temp};
      chomp $sub_entity_element;
      $sub_entity_element =~ s/^ *//g;
      $temp = "account_select_" . $count;
      $account_element = $input{$temp};
      $temp = "sub_account_select_" . $count;
      $sub_account_element = $input{$temp};
      chomp $sub_account_element;
      $sub_account_element =~ s/^\ *//g;
      if ($sub_account_element eq "")
      {
        $sub_account_element = "-1";
      }
      $temp = "sub_account_text_" . $count;
      $sub_account_text = $input{$temp};
      chomp $sub_account_text;
      $sub_account_text =~ s/^\ *//g;
      if ($sub_account_text eq "")
      {
        $sub_account_text = "-1";
      }
      if ($sub_account_element ne "C" and $sub_account_element ne "D")
      {
        $sub_account_text = $sub_account_element;
      }
      else
      {
        # check for check/deposit #'s already in use
        if ($sub_account_element eq "C")
        {
          # check for duplicate check numbers; sub_account_text is check #
          $result = $conn->exec("select parent_num from debitcredit_tb where account_num = $account_element and sub_account_num = $sub_account_text and dorc = $dorc AND ledger_code = '$ledger_code'");
          if ($result->resultStatus != PGRES_TUPLES_OK)
          {
            PrintErrorHeader();
            print "<h1>Error: Accessing database failed!</h1>\n";
            print $conn->errorMessage . "<br>Tried to get check/deposit numbers already in use.<br>$sys_admin";
            CleanUp();
          }
          if ($result->ntuples > 0)
          {
            PrintErrorHeader();
            print "<h1>Error: ";
            if ($dorc eq "true")
            {
              print "Deposit";
            }
            else
            {
              print "Check";
            }
            print " number <b>$sub_account_text</b> already in use in database!</h1>\n";
            print "$back_button";
            CleanUp();
          }
        }
        elsif ($sub_account_element eq "D")
        {
          # check for duplicate deposit numbers; sub_account_text is deposit #
          $result = $conn->exec("select parent_num from debitcredit_tb where account_num = $account_element and sub_account_num = $sub_account_text and dorc = $dorc AND ledger_code = '$ledger_code'");
          if ($result->resultStatus != PGRES_TUPLES_OK)
          {
            PrintErrorHeader();
            print "<h1>Error: Accessing database failed!</h1>\n";
            print $conn->errorMessage . "<br>Tried to get check/deposit numbers already in use.<br>$sys_admin";
            CleanUp();
          }
          if ($result->ntuples > 0)
          {
            PrintErrorHeader();
            print "<h1>Error: ";
            if ($dorc eq "true")
            {
              print "Deposit";
            }
            else
            {
              print "Check";
            }
            print " number <b>$sub_account_text</b> already in use in database!</h1>\n";
            print "$back_button";
            CleanUp();
          }
        }
      }
      $temp = "cleared_" . $count;
      if (exists $input{$temp})
      {
        $cleared = $input{$temp};
      }
      else
      {
        $cleared = " ";
      }
      # insert values into database
      $result = $conn->exec("insert into debitcredit_tb (parent_num, sub_e_code, account_num, sub_account_num, amount, dorc, cleared, counter, ledger_code) values ($ref_num, '$sub_entity_element', '$account_element', '$sub_account_text', '$amount', $dorc, '$cleared', $count, '$ledger_code')");
      if ($result->resultStatus != PGRES_COMMAND_OK)
      {
        print "<h1>Error: Inserting data failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to insert row data for journal entry $ref_num.<br>$sys_admin";
        CleanUp();
      }
      print "<tr align=center>\n";
      if ($sub_entity_element eq "")
      {
        print "<td><br></td>\n";
      }
      else
      {
        print "<td>$sub_entity_element</td>\n";
      }
      print "<td>$account_element</td>\n";
      if ($sub_account_text ne "-1")
      {
        print "<td>$sub_account_text</td>\n";
      }
      else
      {
        print "<td><br></td>\n";
      }
      if ($dorc eq "true")
      {
        print "<td align=right>$amount</td>\n";
        print "<td><br></td>\n";
      }
      else
      {
        print "<td><br></td>\n";
        print "<td align=right>$amount</td>\n";
      }
      if ($cleared eq " ")
      {
        print "<td><br></td>\n";
      }
      else
      {
        print "<td>$cleared</td>\n";
      }
      print "<td>$count</td>\n";
      print "</tr>\n";
    }
  }
  print "</table>\n";
}
elsif ($mode eq "new") # create a new journal entry in journal_entry_tb with ref_num tried 10 times, create entries in debitcredit_tb
{
  # find ref_num value not in use in database; try 10 times if necessary
  $result = $conn->exec("select max(ref_num) from journal_entry_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error: Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Attempted to retrieve the maximum reference number.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    $ref_num = 0;
  }
  else
  {
    $ref_num = $result->getvalue(0, 0) + 1;
  }
  # insert values into journal_entry_tb and debitcredit_tb
  $journal_code = $input{'journal_code'};
  $trans_date = $input{'trans_date'};
  $payee_payer = $input{'payee_payer'};
  # escape all ('), ($), (\) characters
  $payee_payer =~ s/\\/\\\\/g;
  $payee_payer =~ s/\'/\\\'/g;
  $payee_payer =~ s/\$/\\\$/g;
  $memo = $input{'memo'};
  # escape all ('), ($), (\) characters
  $memo =~ s/\\/\\\\/g;
  $memo =~ s/\'/\\\'/g;
  $memo =~ s/\$/\\\$/g;
  $remember = $input{'remember'};
  $result = $conn->exec("insert into journal_entry_tb (ref_num, journal_code, trans_date, payee_payer, memo, remember, ledger_code) values ($ref_num, '$journal_code', '$trans_date', '$payee_payer', '$memo', $remember, '$ledger_code')");
  $count = 0;
  while ($result->resultStatus != PGRES_COMMAND_OK and $count < 10)
  {
    $ref_num++;
    $result = $conn->exec("insert into journal_entry_tb (ref_num, journal_code, trans_date, payee_payer, memo, remember, ledger_code) values ($ref_num, '$journal_code', '$trans_date', '$payee_payer', '$memo', $remember, '$ledger_code')");
    $count++;
  }
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Failed to insert data into database!</h1>\n";
    $temp = $count + 1;
    print $conn->errorMessage . "<br>Tried to get ref_num for " . $temp . " times.<br>$sys_admin";
    CleanUp();
  }
  print "<table border=1 cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$body_bgcolor\">\n";
  print "<tr>\n";
  print "<th>Ref#</th><th>Jnl</th><th>Date</th><th>Payee-Payer</th><th>Remember</th><th>Memo</th>\n";
  print "</tr>\n";
  print "<tr align=center>\n";
  print "<td>$ref_num</td>\n";
  print "<td>$journal_code</td>\n";
  print "<td>$trans_date</td>\n";
  if ($payee_payer eq "")
  {
    print "<td><br></td>\n";
  }
  else
  {
    print "<td>$payee_payer</td>\n";
  }
  if ($remember eq "true")
  {
    print "<td>Yes</td>\n";
  }
  else
  {
    print "<td>No</td>\n";
  }
  if ($memo eq "")
  {
    print "<td><br></td>\n";
  }
  else
  {
    print "<td>$memo</td>\n";
  }
  print "</tr>\n";
  print "</table>\n";

  print "<table border=1 cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$body_bgcolor\">\n";
  print "<tr>\n";
  print "<th>SbE</th><th>Account</th><th>SubAccount</th><th>Debit</th><th>Credit</th><th>Cleared</th><th>Counter</th>\n";
  print "</tr>\n";
  $num_rows = $input{'num_rows'};
  for ($count=0; $count<$num_rows; $count++)
  {
    # find only those rows that have debits/credits entered and insert them into the database
    $temp1 = "debit_" . $count;
    $temp2 = "credit_" . $count;
    $temp3 = "account_select_" . $count;

    if (exists $input{$temp1} and exists $input{$temp2} and exists $input{$temp3} and $input{$temp3} ne "-1" and not ($input{$temp1} eq "" and $input{$temp2} eq ""))
    {
      if (length $input{$temp1} > 0)
      {
        $amount = $input{$temp1};
        $dorc = "true";
      }
      else
      {
        $amount = $input{$temp2};
        $dorc = "false";
      }
      $temp = "sub_entity_select_" . $count;
      $sub_entity_element = $input{$temp};
      chomp $sub_entity_element;
      $sub_entity_element =~ s/^ *//g;
      $temp = "account_select_" . $count;
      $account_element = $input{$temp};
      $temp = "sub_account_select_" . $count;
      $sub_account_element = $input{$temp};
      chomp $sub_account_element;
      $sub_account_element =~ s/^\ *//g;
      if ($sub_account_element eq "")
      {
        $sub_account_element = "-1";
      }
      $temp = "sub_account_text_" . $count;
      $sub_account_text = $input{$temp};
      chomp $sub_account_text;
      $sub_account_text =~ s/^\ *//g;
      if ($sub_account_text eq "")
      {
        $sub_account_text = "-1";
      }
      if ($sub_account_element ne "C" and $sub_account_element ne "D")
      {
        $sub_account_text = $sub_account_element;
      }
      $cleared = " ";
      # insert values into database
      $result = $conn->exec("insert into debitcredit_tb (parent_num, sub_e_code, account_num, sub_account_num, amount, dorc, cleared, counter, ledger_code) values ($ref_num, '$sub_entity_element', '$account_element', '$sub_account_text', '$amount', $dorc, '$cleared', $count, '$ledger_code')");
      if ($result->resultStatus != PGRES_COMMAND_OK)
      {
        print "<h1>Error: Inserting data failed!</h1>\n";
        print $conn->errorMessage . "<br>Tried to insert row data for journal entry $ref_num.<br>$sys_admin";
        CleanUp();
      }
      print "<tr align=center>\n";
      if ($sub_entity_element eq "")
      {
        print "<td><br></td>\n";
      }
      else
      {
        print "<td>$sub_entity_element</td>\n";
      }
      print "<td>$account_element</td>\n";
      if ($sub_account_text ne "-1")
      {
        print "<td>$sub_account_text</td>\n";
      }
      else
      {
        print "<td><br></td>\n";
      }
      if ($dorc eq "true")
      {
        print "<td align=right>$amount</td>\n";
        print "<td><br></td>\n";
      }
      else
      {
        print "<td><br></td>\n";
        print "<td align=right>$amount</td>\n";
      }
      if ($cleared eq " ")
      {
        print "<td><br></td>\n";
      }
      else
      {
        print "<td>$cleared</td>\n";
      }
      print "<td>$count</td>\n";
      print "</tr>\n";
    }
  }
  print "</table>\n";
}

if ($mode eq "edit")
{
  print "<br>Update of journal entry $ref_num successful.<br>\n";
}
elsif ($mode eq "new")
{
  print "<br>Insert of new journal entry $ref_num successful.<br>\n";
}

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";
print "</center>\n";

print "<form method=post action=\"$site_dir/cgi-bin/journal_activity.cgi\" name=\"new_journal_form\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"journal_code\" value=\"$journal_code\">\n";
print "<input type=hidden name=\"journal_command\" value=\"create\">\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=center><input type=submit name=\"create\" value=\"(Another) New Transaction\"></td>\n";
print "</tr>\n";
print "</table>\n";
my $temp_str;
$temp_str = GetJournalViewInput();
if (length $temp_str > 0)
{
  print "$temp_str";
}
print "</form>\n";
$temp_str = GetJournalViewInput();
if (length $temp_str > 0)
{
  print "<form name=\"journal_view_form\" method=post action=\"$site_dir/cgi-bin/journal_mod1.cgi\">\n";
  print "$temp_str";
  print "<br><br><a href=\"$site_dir/cgi-bin/journal_mod1.cgi\" onClick=\"document.journal_view_form.submit(); return false;\">To return to the Journal View, click <u>here</u>.</a>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;

sub GetJournalViewInput
{
  my ($mode, $date_type, $date_from, $date_to, $redisplay_string, $journal_name, $journal_code, $ref_num, $error_string, $payee_payer, $memo, $account_name, $account_num, $sub_acct_name, $sub_acct_num, $sub_acct_parent);
  my ($money_type, $money_search, $money1, $money2, $sort_by, $sort_journal, $num_entries_to_display, $start_index, $ledger_db, $check_num, $deposit_num, $sort_operator);
  my %valid_entries_to_display = ( 5 => "5", 10 => "10", 15 => "15", 20 => "20", 25 => "25", 30 => "30", 35 => "35", 40 => "40", 45 => "45", 50 => "50" );
  my %valid_options = ( "open" => "open", "delete" => "delete", "add" => "add", "search" => "search", "search2" => "search2" );

  if (not exists $input{'journal_view_command'})
  {
    return "";  # nothing to do.
  }
  if (not exists $valid_options{$input{'journal_view_command'}})
  {
    PrintErrorHeader();
    print "<h1>(Journal View Re-display Code) Error:  Access Denied!</h1>\n";
    print "Invalid option given.  Option = '$input{'journal_view_command'}'.<br>$sys_admin";
    CleanUp();
  }

  $ledger_db = $input{'ledger'};
  if (length $ledger_db == 0)
  {
    PrintErrorHeader();
    print "<h1>(Journal View Re-display Code) Error: You must specify a ledger to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidString($ledger_db);
  if (length $error_string > 0)
  {
    PrintErrorHeader();
    print "<h1>(Journal View Re-display Code) Error:  Invalid Database Name!</h1>\n";
    print "<b>$error_string</b> is invalid!<br>$back_button";
    CleanUp();
  }
  $redisplay_string .= "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";

  # validate we got valid input from the user.
  if ($input{'journal_view_command'} eq "open")
  {
    $mode = 1;
    $redisplay_string .= "<input type=hidden name=\"journal_view_command\" value=\"open\">\n";
    if (not exists $input{'journal'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: You must specify a Journal to work with!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"journal\" value=\"" . $input{'journal'} . "\">\n";
    $date_from = $input{'date_from'};
    if (length $date_from == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  You must specify a From date!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $date_to = $input{'date_to'};
    if (length $date_to == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  You must specify a To date!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $sort_by = $input{'sort_by'};
    if (length $sort_by == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  You must specify a SortBy Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (not ($sort_by eq "date" || $sort_by eq "ref_num" || $sort_by eq "payee_payer"))
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid SortBy Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if ($sort_by eq "date")
    {
      $sort_by = "trans_date";
    }
    if (exists $input{'sort_journal'})
    {
      $sort_journal = 1;
      $redisplay_string .= "<input type=hidden name=\"sort_journal\" value=\"1\">\n";
    }
    else
    {
      $sort_journal = 0;
    }
    $redisplay_string .= "<input type=hidden name=\"date_from\" value=\"$date_from\">\n<input type=hidden name=\"date_to\" value=\"$date_to\">\n<input type=hidden name=\"sort_by\" value=\"" . $input{'sort_by'} . "\">\n";
  }
  elsif ($input{'journal_view_command'} eq "search")
  {
    $mode = 4;
    # first make sure all the correct variables are being passed in.
    if (not exists $input{'date_type'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Date Qualifier must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $date_type = $input{'date_type'};
    if (not exists $input{'date_from_search'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Date to search from must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (not ($date_type eq "equal" || $date_type eq "betweeni" || $date_type eq "betweene" || $date_type eq "afteri" || $date_type eq "aftere" || $date_type eq "beforei" || $date_type eq "beforee"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Date Qualifier Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $date_from = $input{'date_from_search'};
    if (not exists $input{'date_to_search'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Date to search to must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (length $date_from > 0 && (not IsValidDateSyntax($date_from)))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Date given!</h1>\n";
      print "The From date is invalid.<br>$back_button";
      CleanUp();
    }
    $date_to = $input{'date_to_search'};
    if (length $date_to > 0 && (not IsValidDateSyntax($date_to)))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Date given!</h1>\n";
      print "The To date is invalid.<br>$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"journal_view_command\" value=\"search\">\n";
    $redisplay_string .= "<input type=hidden name=\"date_type\" value=\"$date_type\">\n<input type=hidden name=\"date_from_search\" value=\"$date_from\">\n<input type=hidden name=\"date_to_search\" value=\"$date_to\">\n";
    if (not exists $input{'journal2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Journal selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    ($journal_name, $journal_code) = split /\|/, $input{'journal2'};
    if (not exists $input{'ref_num2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Reference Number field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $ref_num = $input{'ref_num2'};
    $error_string = IsValidUInteger($ref_num);
    if (length $ref_num > 0 && length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Ref Number!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if (not exists $input{'check_num'})
    {
      PrintErrorHeader();
      print "<h1>Error: Check Number field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $check_num = $input{'check_num'};
    $error_string = IsValidUInteger($check_num);
    if (length $check_num > 0 && length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid Check Number!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if (not exists $input{'deposit_num'})
    {
      PrintErrorHeader();
      print "<h1>Error: Deposit Number field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $deposit_num = $input{'deposit_num'};
    $error_string = IsValidUInteger($deposit_num);
    if (length $deposit_num > 0 && length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid Deposit Number!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if (not exists $input{'payee_payer2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Payee-Payer selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $payee_payer = $input{'payee_payer2'};
    $error_string = IsValidString($payee_payer);
    if (length $payee_payer > 0 && length $error_string > 0 && (not $error_string eq "_"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Payee-Payer!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"journal2\" value=\"" . $input{'journal2'} . "\">\n<input type=hidden name=\"ref_num2\" value=\"$ref_num\">\n<input type=hidden name=\"check_num\" value=\"$check_num\">\n<input type=hidden name=\"deposit_num\" value=\"$deposit_num\">\n<input type=hidden name=\"payee_payer2\" value=\"$payee_payer\">\n";
    if (not exists $input{'memo2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Memo field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $memo = $input{'memo2'};
    $error_string = IsValidString($memo);
    if (length $memo > 0 && length $error_string > 0 && (not ($error_string eq "*" || $error_string eq "?")))
    {
      PrintErrorHeader();
      print "<h1(Journal View Re-display Code) >Error:  Invalid Memo!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if (not exists $input{'account'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Account selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    ($account_name, $account_num) = split /\|/, $input{'account'};
    if (not exists $input{'sub_account'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Sub Account selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    ($sub_acct_name, $sub_acct_num, $sub_acct_parent) = split /\|/, $input{'sub_account'};
    if (not exists $input{'dorc'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Debit or Credit Selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $money_type = $input{'dorc'};
    if (not ($money_type eq "select_dorc" || $money_type eq "debit" || $money_type eq "credit" || $money_type eq "both"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Debit or Credit Selection type!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"memo2\" value=\"$memo\">\n<input type=hidden name=\"account\" value=\"" . $input{'account'} . "\">\n<input type=hidden name=\"sub_account\" value=\"" . $input{'sub_account'} . "\">\n";
    $redisplay_string .= "<input type=hidden name=\"dorc\" value=\"$money_type\">\n";
    if (not exists $input{'dorc_search'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Debit or Credit Search Type Selection must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $money_search = $input{'dorc_search'};
    if (not ($money_search eq "all" || $money_search eq "equal" || $money_search eq "less_than" || $money_search eq "less_than_equal" || $money_search eq "greater_than" || $money_search eq "greater_than_equal" || $money_search eq "betweene" || $money_search eq "betweeni"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Debit or Credit Search type!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $money1 = $input{'money1'};
    $error_string = IsValidInteger($money1);
    my $error_string2 = IsValidFloat($money1);
    if (length $money1 > 0)
    {
      if (length $error_string > 0 && (not $error_string eq "."))
      {
        PrintErrorHeader();
        print "<h1>(Journal View Re-display Code) Error:  Invalid First Money Field!</h1>\n";
        print "<b>$error_string</b> is invalid!<br>$back_button";
        CleanUp();
      }
      elsif (length $error_string > 0 && $error_string eq "." && length $error_string2 > 0)
      {
        PrintErrorHeader();
        print "<h1>(Journal View Re-display Code) Error:  Invalid First Money Field!</h1>\n";
        print "<b>$error_string</b> is invalid!<br>$back_button";
        CleanUp();
      }
    }
    $redisplay_string .= "<input type=hidden name=\"dorc_search\" value=\"$money_search\">\n<input type=hidden name=\"money1\" value=\"$money1\">\n<input type=hidden name=\"money2\" value=\"" . $input{'money2'} . "\">\n";
    $money2 = $input{'money2'};
    $error_string = IsValidInteger($money2);
    $error_string2 = IsValidFloat($money2);
    if (length $money2 > 0)
    {
      if (length $error_string > 0 && (not $error_string eq "."))
      {
        PrintErrorHeader();
        print "<h1>(Journal View Re-display Code) Error:  Invalid Second Money Field!</h1>\n";
        print "<b>$error_string</b> is invalid!<br>$back_button";
        CleanUp();
      }
      elsif (length $error_string > 0 && $error_string eq "." && length $error_string2 > 0)
      {
        PrintErrorHeader();
        print "<h1>(Journal View Re-display Code) Error:  Invalid Second Money Field!</h1>\n";
        print "<b>$error_string</b> is invalid!<br>$back_button";
        CleanUp();
      }
    }
    $sort_by = $input{'sort_by2'};
    if (length $sort_by == 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You must specify a SortBy Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    #print "\"$sort_by\"<br>";
    if (not ($sort_by eq "date" || $sort_by eq "ref_num" || $sort_by eq "payee_payer"))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid SortBy Option!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if ($sort_by eq "date")
    {
      $sort_by = "trans_date";
    }
    if (exists $input{'sort_journal'})
    {
      $sort_journal = 1;
      $redisplay_string .= "<input type=hidden name=\"sort_journal\" value=\"1\">\n";
    }
    else
    {
      $sort_journal = 0;
    }
    $sort_operator = $input{'sort_operator'};
    if (length $sort_operator == 0)
    {
      PrintErrorHeader();
      print "<h1>Error:  You must specify a Search Operator!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (not ($sort_operator eq "AND" || $sort_operator eq "OR"))
    {
      PrintErrorHeader();
      print "<h1>Error:  Invalid Search Operator!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"sort_by2\" value=\"" . $input{'sort_by2'} . "\">\n<input type=hidden name=\"sort_operator\" value=\"$sort_operator\">\n";

    # make sure they entered something in at least one field, but not something in both the debit and credit fields.
    if ((length $input{'date_from_search'} == 0) && (length $input{'date_to_search'} == 0) && ($input{'journal2'} eq "select_journal") && (length $input{'ref_num2'} == 0) && (length $input{'check_num'} == 0) && (length $input{'deposit_num'} == 0) && ($input{'payee_payer2'} eq "select_payee_payer") && (length $input{'memo2'} == 0) && ($input{'account'} eq "select_account") && ($input{'sub_account'} eq "select_sub_acct") && ($input{'dorc'} eq "select_dorc") && ($input{'dorc_search'} eq "all") && (length $input{'money1'} == 0) && (length $input{'money2'} == 0))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: You must specify something for your search!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    # make sure that the date_to_search field is given only if the date_type is "Between".
    if ((length $input{'date_from_search'} > 0) && (length $input{'date_to_search'} > 0) && ((not $input{'date_type'} eq "betweeni") && (not $input{'date_type'} eq "betweene")))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You must specify <b>Between</b> to use the second Date field!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    # make sure that both money fields are not both filled in if they are not doing a between search or no Dorc value selected.
    if ((length $input{'money1'} > 0) && (length $input{'money2'} > 0) && ((not $input{'dorc'} eq "select_dorc") && (not $input{'dorc_search'} eq "betweene") && (not $input{'dorc_search'} eq "betweeni")))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You can not specify both money fields when not searching with between!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (($input{'dorc_search'} eq "betweene" || $input{'dorc_search'} eq "betweeni") && (length $input{'money1'} == 0 || length $input{'money2'} == 0))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You must specify both money fields when searching with between!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    if (((not $input{'dorc'} eq "select_dorc") && (not $input{'dorc_search'} eq "all")) && ((length $input{'money1'} == 0) && (length $input{'money2'} == 0)))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You must specify an amount to search for!</h1>\n";
      print "Error: 1007<br>$back_button";
      CleanUp();
    }
    # make sure that if they enter a Ref-Num that they don't select anything else.
    if ((length $input{'ref_num2'} > 0) && ((length $input{'date_from_search'} > 0) || (length $input{'date_to_search'} > 0) || (length $input{'check_num'} > 0) || (length $input{'deposit_num'} > 0) || (not $input{'payee_payer2'} eq "select_payee_payer") || (not $input{'journal2'} eq "select_journal") || (not $input{'account'} eq "select_account") || (not $input{'sub_account'} eq "select_sub_acct") || (not $input{'dorc'} eq "select_dorc") || (not $input{'dorc_search'} eq "all") || (length $input{'money1'} > 0) || (length $input{'money2'} > 0) || (length $input{'memo2'} > 0)))
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  You can not specify anything else when specifying the Reference Number!</h1>\n";
      print "$back_button";
      CleanUp();
    }
  }
  elsif ($input{'journal_view_command'} eq "search2")
  {
    $mode = 5;
    # first make sure all the correct variables are being passed in.
    if (not exists $input{'ref_num2'})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: Reference Number field must be given!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $ref_num = $input{'ref_num2'};
    $error_string = IsValidUInteger($ref_num);
    if (length $ref_num > 0 && length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Ref Number!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"journal_view_command\" value=\"search2\">\n<input type=hidden name=\"ref_num2\" value=\"$ref_num\">\n";
  }

  if ($mode == 1 || $mode == 4 || $mode == 5)
  {
    $num_entries_to_display = $input{'num_entries_to_display'};
    if (length $num_entries_to_display == 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: You must specify the number of Journal entries to display!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $error_string = IsValidInteger($num_entries_to_display);
    if (length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Value for Number of Entries to Display!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }

    # verify it is a valid value
    if (not exists $valid_entries_to_display{$num_entries_to_display})
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Value for Number of Entries to Display!</h1>\n";
      print "<b>$num_entries_to_display</b> is not valid!<br>$back_button";
      CleanUp();
    }

    $start_index = $input{'start_index'};
    if (length $start_index == 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error: You must specify the Starting Index to work from!</h1>\n";
      print "$back_button";
      CleanUp();
    }
    $error_string = IsValidInteger($start_index);
    if (length $error_string > 0)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  Invalid Value for Start Index to display from!</h1>\n";
      print "<b>$error_string</b> is invalid!<br>$back_button";
      CleanUp();
    }
    if ($start_index < 1)
    {
      PrintErrorHeader();
      print "<h1>(Journal View Re-display Code) Error:  The Start Index can not be less than 1!</h1>\n";
      print "$sys_admin";
      CleanUp();
    }
    $redisplay_string .= "<input type=hidden name=\"num_entries_to_display\" value=\"$num_entries_to_display\">\n<input type=hidden name=\"start_index\" value=\"$start_index\">\n";
  }

  return $redisplay_string;  # return the string that will contain everything necessary to re-display the journal like how the person just left it.
}