#!/usr/bin/perl
#$Id: journal_print.cgi,v 1.1 2001/09/06 10:21:19 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# journal_print.cgi  -  Created by James Pattie, 02/04/2000
# This script presents the user with the options to print journals.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

my $ledger_name;
my $ledger_db;
my $ledger_color;
my $journal_name;
my $journal_code;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  A Ledger must be specified!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now we check and validate the ledger is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
$ledger_name = $result->getvalue(0, 0);

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# present them with the main menu

print "<head><title>$prog_name $prog_version Journals Main Menu</title>\n";
print <<"END_OF_SCRIPT";
  <script language="Javascript1.2">
  function create_select_name(index, text, start, end)
  {
    this.value = index;
    this.text = text;
    this.start = start;
    this.end = end;
  }

  var dates = new Array();

END_OF_SCRIPT

# figure out the beginning and ending date of all fiscal years and create them in the array
my ($year, $from_date, $to_date);

$result = $conn->exec("select year, start_date, end_date from known_years_tb where ledger_code = '$ledger_code' ORDER BY year");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get fiscal year dates from ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Program not configured!</h1>\n";
  print "Working Year is not set!<br>$sys_admin";
  CleanUp();
}

for (my $i=0; $i < $result->ntuples; $i++)
{
  $year = $result->getvalue($i, 0);
  $from_date = $result->getvalue($i, 1);
  $to_date = $result->getvalue($i, 2);

  print "  dates[$i] = new create_select_name($i, \"$year\", \"$from_date\", \"$to_date\");\n";
}

print <<"END_OF_SCRIPT";

  function init()
  {
    document.print_form.date_entries.selectedIndex = 0;
    date_select_fill();
  }

  function date_select_fill()
  {
    document.print_form.date_entries.options.length = 0;

    for (var count=0; count < dates.length; count++)
    {
      document.print_form.date_entries.options[count] = new Option(dates[count].text, dates[count].value);
    }
  }

  function FillDate()
  {
    var index = document.print_form.date_entries.selectedIndex;

    if (index == -1)
    {
      alert("No item selected from the Dates combo box!");
      return;
    }

    if (index > dates.length)
    {
      alert("The selected item is out of range!\\n  Value = "+index);
      return;
    }

    // replace the date row with the selected date values
    document.print_form.start_date.value = dates[index].start;
    document.print_form.end_date.value = dates[index].end;
    document.print_form.start_date.focus();

    // now unselect the selected date
    document.print_form.date_entries.selectedIndex = -1;
  }

  function SelectAllJournals()
  {
    var i;
    for (i=0; i < document.print_form.journals.length; i++)
    {
      document.print_form.journals.options[i].selected = true;
    }
  }

  function ValidatePrintForm()
  {
    var message = "";
    if (document.print_form.start_date.value.length == 0)
    {
      message += "You must specify the Start Date!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(document.print_form.start_date.value);
      if (tmp.length > 0)
      {
        message += "Start Date: " + tmp + "\\n";
      }
    }
    if (document.print_form.end_date.value.length == 0)
    {
      message += "You must specify the End Date!\\n";
    }
    else
    {
      var tmp = IsValidDateSyntax(document.print_form.end_date.value);
      if (tmp.length > 0)
      {
        message += "End Date: " + tmp + "\\n";
      }
    }
    // make sure they have selected a journal to work with.
    var i, found;
    for (i=0, found=false; i < document.print_form.journals.length; i++)
    {
      if (document.print_form.journals.options[i].selected)
      {
        found = true;
      }
    }
    if (!found)
    {
      message += "You must select a Journal to display!\\n";
    }
    if (message.length > 0)
    {
      alert(message);
      return false;
    }
    return true;
  }
  </script>
END_OF_SCRIPT
print "</head>\n";
print "<body bgcolor=$body_bgcolor onLoad=\"init();\">\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><font color=red>Working in ledger <font color=\"$ledger_color\">$ledger_name</font>.</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#journals_main_menu\" target=\"docs\"><img src=\"$site_dir/images/help.jpg\" alt=\"Help\" border=0</a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Print Journals Main Menu</h2>\n";

$result = $conn->exec("select name, code from journal_tb order by name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of journals available.<br>$sys_admin";
  CleanUp();
}

my ($i, $name, $code);
if ($result->ntuples > 0)
{
  print "<form method=\"post\" action=\"$site_dir/cgi-bin/journal_mod1.cgi\" name=\"print_form\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$input{'ledger'}\">\n";
  print "<input type=hidden name=\"journal_view_command\" value=\"print\">\n";
  print "<table border=0 cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$menu_bgcolor\">\n";
  print "<tr><th colspan=2>Custom Title</th></tr>\n";
  print "<tr><td align=center colspan=2><input type=text name=\"title\" value=\"Journal Print View\"></td></tr>\n";
  print "<tr><th colspan=2>Journal</th></tr>";
  print "<tr><td align=left><select name=\"journals\" size=4 width=200 multiple>\n";
  for ($i=0; $i < $result->ntuples; $i++)
  {
    $name = $result->getvalue($i, 0);
    $code = $result->getvalue($i, 1);
    print "<option value=\"$name|$code\">$name ($code)\n";
  }

  print "</select></td><td align=left>&nbsp;<font color=\"$menu_fgcolor\"><input type=button name=\"select_all\" value=\"Select All\" onClick=\"SelectAllJournals();\"></font></td></tr>\n";
  print "<tr><th>Date From</th><th>Date To</th></tr>\n";
  print "<tr><td align=center><input type=text name=\"start_date\" value=\"\"></td><td align=center><input type=text name=\"end_date\" value=\"\"></td></tr>";
  print "<tr><th>&nbsp;</th><th>Year</th></tr>\n";
  print "<tr><td align=center><font color=\"$menu_fgcolor\"><input type=button name=\"clear\" value=\"Clear Dates\" onClick=\"document.print_form.start_date.value='';document.print_form.end_date.value='';\"></font></td><td align=center><select name=\"date_entries\" size=3 width=50 onChange=\"FillDate();\"></select></td></tr>\n";
  print "<tr><td align=left colspan=2>Sort By &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<select name=\"sort_by\">\n";
  print "<option value=\"date\">Date\n";
  print "<option value=\"ref_num\">Reference Number\n";
  print "<option value=\"payee_payer\">Payee Payer\n";
  print "</select> And &nbsp;&nbsp;<input type=checkbox name=\"sort_journal\" value=\"ON\"> Journals</td></tr>\n";
  print "<tr><td align=center colspan=2><font color=\"$menu_fgcolor\"><input type=button name=\"print\" value=\"Print\" onClick=\"if (!ValidatePrintForm()) return false;document.print_form.submit();\">&nbsp;<input type=reset value=\"Reset\"></font></td></tr>";
  print "</table>\n";
  print "</form>\n";
}
print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
