#!/usr/bin/perl
#$Id: journal_search.cgi,v 1.4 2003/04/09 19:31:57 pcxuser Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# journal_search.cgi  -  Created by James Pattie, 11/30/1999
# This script presents the user with the journal search form.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

my $ledger_name;
my $ledger_db;
my $ledger_color;
my $journal_name;
my $journal_code;
my $ledger_code;
my $ledger_color;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  A Ledger must be specified!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now we check and validate the ledger is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
$ledger_name = $result->getvalue(0, 0);

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# figure out the beginning and ending date of the current working year
my ($working_year, $from_date, $to_date);

$result = $conn->exec("select year, start_date, end_date from known_years_tb where year in (select working_year from variables_tb WHERE ledger_code = '$ledger_code') AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get working year and fiscal year dates from ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Program not configured!</h1>\n";
  print "Working Year is not set!<br>$sys_admin";
  CleanUp();
}
$working_year = $result->getvalue(0, 0);
$from_date    = $result->getvalue(0, 1);
$to_date      = $result->getvalue(0, 2);

$result = $conn->exec("select name, code from journal_tb order by name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of journals available.<br>$sys_admin";
  CleanUp();
}

# present them with the main menu

print "<head><title>$prog_name $prog_version Journal Search From</title>\n";
print <<"END_OF_SCRIPT";
  <script language="JavaScript1.2">
  function ValidateSearchForm()
  {
    // first we validate that there is something selected or filled in and that if they entered a ref_num that nothing other than the defaults is modified.
    var doc = document.journals_form;
    var msg = "";
    if (doc.date_type.selectedIndex == 0 && doc.date_from_search.value.length == 0 && doc.date_to_search.value.length == 0 && doc.journal2.selectedIndex == 0 && doc.ref_num2.value.length == 0 && doc.check_num.value.length == 0 && doc.deposit_num.value.length == 0 && doc.payee_payer2.selectedIndex == 0 && doc.memo2.value.length == 0 && doc.account.selectedIndex == 0 && doc.sub_account.selectedIndex == 0 && doc.dorc.selectedIndex == 0 && doc.dorc_search.selectedIndex == 0 && doc.money1.value.length == 0 && doc.money2.value.length == 0 && doc.sub_entity.selectedIndex == 0)
    {
      msg += "You must specify what you want searched for!\\n";
    }
    // Make sure that they haven't specified the Ref_Num and anything else.
    if (doc.ref_num2.value.length > 0 && (doc.date_type.selectedIndex > 0 || doc.date_from_search.value.length > 0 || doc.date_to_search.value.length > 0 || doc.date_type_selectedIndex > 0 || doc.check_num.value.length > 0 || doc.deposit_num.value.length > 0 || doc.payee_payer2.selectedIndex > 0 || doc.memo2.value.length > 0 || doc.account.selectedIndex > 0 || doc.sub_account.selectedIndex > 0 || doc.dorc.selectedIndex > 0 || doc.dorc_search.selectedIndex > 0 || doc.money1.value.length > 0 || doc.money2.value.length > 0))
    {
      msg += "You can not specify any other options when specifying the Reference Number to search for!\\n";
    }
    else
    {
      // now check to make sure that the only time that both date fields are filled in is if they selected between.
      if ((doc.date_from_search.value.length > 0 && doc.date_to_search.value.length > 0) && (doc.date_type.selectedIndex != 1 && doc.date_type.selectedIndex != 2))
      {
        msg += "You can only fill in the second Date when Searching with Between (Inclusive or Exclusive)!\\n";
      }
      if (doc.date_type.selectedIndex == 1 || doc.date_type.selectedIndex == 2)
      {
        if (doc.date_from_search.value.length == 0)
        {
          msg += "You must specify the From Date to Search between!\\n";
        }
        if (doc.date_to_search.value.length == 0)
        {
          msg += "You must specify the To Date to Search between!\\n";
        }
      }
      if (doc.date_type.selectedIndex > 0 && doc.date_from_search.value.length == 0 && doc.date_type.selectedIndex > 2)
      {
        msg += "You must specify a beginning Date!\\n";
      }
      if (doc.date_from_search.value.length == 0 && doc.date_to_search.value.length > 0)
      {
        msg += "You must specify the beginning Date before the end Date!\\n";
      }
      // If the dates are filled in, validate them.
      if (doc.date_from_search.value.length > 0)
      {
        var tmp = IsValidDateSyntax(doc.date_from_search.value);
        if (tmp.length > 0)
        {
          msg += "The First Date is Invalid!\\n" + tmp + "\\n";
        }
      }
      if (doc.date_to_search.value.length > 0)
      {
        var tmp = IsValidDateSyntax(doc.date_to_search.value);
        if (tmp.length > 0)
        {
          msg += "The Second Date is Invalid!\\n" + tmp + "\\n";
        }
      }
      if (doc.dorc.selectedIndex > 0)
      {
        if (doc.dorc_search.selectedIndex > 0)
        {
          if (doc.money1.value.length == 0 && doc.money2.value.length == 0)
          {
            msg += "You must specify the Money amounts to search for!\\n";
          }
          if (doc.money1.value.length == 0)
          {
            msg += "You must specify the Money amount to search for!\\n";
          }
          if (doc.dorc_search.selectedIndex == 6 || doc.dorc_search.selectedIndex == 7)
          {
            if (doc.money2.value.length == 0)
            {
              msg += "You must specify the top value for the Money amount to search between!\\n";
            }
          }
          else
          {
            if (doc.money2.value.length != 0)
            {
              msg += "You can not specify both Money values unless using the Between (Inclusive or Exclusive) Search!\\n";
            }
          }
          if (doc.money1.value.length > 0)
          {
            doc.money1.value = MakeValidAmount(doc.money1.value);
          }
          if (doc.money2.value.length > 0)
          {
            doc.money2.value = MakeValidAmount(doc.money2.value);
          }
        }
        else
        {
          if (doc.money1.value.length > 0 || doc.money2.value.length > 0)
          {
            msg += "You can not specify any Money values when searching for All Debits, Credits or Both!\\n";
          }
        }
      }
      else
      {
        // verify that they haven't changed the dorc_search or money fields, else it is an error.
        if (doc.dorc_search.selectedIndex > 0)
        {
          if (doc.money1.value.length > 0 || doc.money2.value.length > 0)
          {
            msg += "You must Select a Debit, Credit or Both option before you can specify Money Values!\\n";
          }
          else
          {
            msg += "You must Select a Debit, Credit or Both option and a Money value before you can Search the Amounts!\\n";
          }
        }
        else
        {
          if (doc.money1.value.length > 0 || doc.money2.value.length > 0)
          {
            msg += "You must Select a Debit, Credit or Both option before you can specify Money Values!\\n";
          }
        }
      }
    }
    if (doc.ref_num2.value.length > 0)
    {
      doc.ref_num2.value = MakeValidUInt(doc.ref_num2.value);
    }
    if (msg.length > 0)
    {
      alert(msg);
      return false;
    }
    return true;
  }
  </script>
END_OF_SCRIPT
print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

my ($i, $name, $code);

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><font color=red>Working in ledger <font color=\"$ledger_color\">$ledger_name</font>. &nbsp;FY is <font color=\"black\">$working_year</font>.</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#journal_search\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Journal Search Form</h2>\n";
print "<form method=\"post\" action=\"\" name=\"journals_form\" onSubmit=\"return ValidateSearchForm();\">\n";
print "<input type=hidden name=\"ledger\" value=\"$input{'ledger'}\">\n";


# Create the Search Journal interface
print "<br>\n";
print "Any fields not filled in or selected will be ignored.<br>\n";
print "<table border=0 cellspacing=0 cellpadding=$table_cellpadding bgcolor=\"$menu_bgcolor\">\n";
print "<tr><td align=right>Date&nbsp;&nbsp;&nbsp;</td><td align=left><select name=\"date_type\">\n";
print "<option value=\"equal\">Equal\n";
print "<option value=\"betweeni\">Between (I)\n";
print "<option value=\"betweene\">Between (E)\n";
print "<option value=\"beforei\">Before (I)\n";
print "<option value=\"beforee\">Before (E)\n";
print "<option value=\"afteri\">After (I)\n";
print "<option value=\"aftere\">After (E)\n";
print "</select>";
print "<input type=text name=\"date_from_search\"> and <input type=text name=\"date_to_search\">&nbsp;<i>The second date field is only valid with <b>Between</b>.</i></td></tr>\n";
print "<tr><td align=right>Journal = </td><td align=left><select name=\"journal2\">\n";
print "<option value=\"select_journal\">Select Journal\n";
for ($i=0; $i < $result->ntuples; $i++)
{
  $name = $result->getvalue($i, 0);
  $code = $result->getvalue($i, 1);
  print "<option value=\"$name|$code\">$name ($code)\n";
}
print "</select></td></tr>\n";
print "<tr><td align=right>Ref-Num = </td><td align=left><input type=text name=\"ref_num2\">&nbsp;(Enter this, only if you know the Reference Number.)</td></tr>\n";
print "<tr><td align=right>Check-Number = </td><td align=left><input type=text name=\"check_num\"></td></tr>\n";
print "<tr><td align=right>Deposit-Number = </td><td align=left><input type=text name=\"deposit_num\"></td></tr>\n";
print "<tr><td align=right>Payee-Payer = </td><td align=left><select name=\"payee_payer2\">\n";
$result = $conn->exec("select distinct(payee_payer) from journal_entry_tb WHERE ledger_code = '$ledger_code' order by payee_payer");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "</select></td></tr></table><h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of Payee-Payers available.<br>$sys_admin";
  CleanUp();
}
print "<option value=\"select_payee_payer\">Select Payee-Payer\n";
for ($i=0; $i < $result->ntuples; $i++)
{
  $name = $result->getvalue($i,0);
  print "<option value=\"$name\">";
  if ($name eq "")
  {
    print "_\n";
  }
  else
  {
    print "$name\n";
  }
}
print "</select> <i>( _ represents an empty string)</i></td></tr>\n";
print "<tr><td align=right>Memo = </td><td align=left><input type=text name=\"memo2\">&nbsp;(<b>*</b> and <b>?</b> are the wild card characters to use.)</td></tr>\n";

print "<tr><td align=right>sub_entity = </td><td align=left><select name=\"sub_entity\">\n";
$result = $conn->exec("select code from sub_entity_tb WHERE ledger_code = '$ledger_code' order by code");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "</select></td></tr></table><h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of sub_entities available.<br>$sys_admin";
  CleanUp();
}
print "<option value=\"select_sub_entity\">Select Sub Entity\n";
for ($i=0; $i < $result->ntuples; $i++)
{
  $code = $result->getvalue($i,0);
  print "<option value=\"$code\">$code\n";
}
print "</select></td></tr>\n";

print "<tr><td align=right>Account = </td><td align=left><select name=\"account\">\n";
$result = $conn->exec("select name, number from account_tb order by name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "</select></td></tr></table><h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of accounts available.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  print "</select></td></tr></table><h1>Error:  There are no accounts!</h1>\n";
  print "Tried to get list of accounts to work with.<br>$sys_admin";
  CleanUp();
}
print "<option value=\"select_account\">Select Account\n";
for ($i=0; $i < $result->ntuples; $i++)
{
  $name = $result->getvalue($i,0);
  $code = $result->getvalue($i, 1);
  print "<option value=\"$name|$code\">$name ($code)\n";
}
print "</select></td></tr>\n";
print "<tr><td align=right>Sub Account = </td><td align=left><select name=\"sub_account\">\n";
$result = $conn->exec("select sub_acc_name, sub_acc_number from sub_acc_tb order by sub_acc_name");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "</select></td></tr></table><h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get list of sub accounts available.<br>$sys_admin";
  CleanUp();
}
print "<option value=\"select_sub_acct\">Select Sub Account\n";
for ($i=0; $i < $result->ntuples; $i++)
{
  $name = $result->getvalue($i,0);
  $code = $result->getvalue($i, 1);
  print "<option value=\"$name|$code\">$name ($code)\n";
}
print "</select></td></tr>\n";
print "<tr><td align=right>Amount&nbsp;&nbsp;&nbsp;</td><td align=left><select name=\"dorc\">\n";
print "<option value=\"select_dorc\">Select Debit or Credit\n";
print "<option value=\"debit\">Debit\n";
print "<option value=\"credit\">Credit\n";
print "<option value=\"both\">Both\n";
print "</select>\n<select name=\"dorc_search\">\n";
print "<option value=\"all\">All\n";
print "<option value=\"equal\">=\n";
print "<option value=\"less_than\">&lt;\n";
print "<option value=\"less_than_equal\">&lt;=\n";
print "<option value=\"greater_than\">&gt;\n";
print "<option value=\"greater_than_equal\">&gt;=\n";
print "<option value=\"betweene\">Between (E)\n";
print "<option value=\"betweeni\">Between (I)\n";
print "</select>\n<input type=text name=\"money1\"> and <input type=text name=\"money2\"></td></tr>\n";
print "<tr><td align=right>&nbsp;</td><td align=center><i>Selecting the <b>All</b> operator could potentially select every transaction in the database!</i></td></tr>\n";
print "<tr><td align=right>Sort By &nbsp;&nbsp;&nbsp;</td><td align=left><select name=\"sort_by2\">\n";
print "<option value=\"date\">Date\n";
print "<option value=\"ref_num\">Reference Number\n";
print "<option value=\"payee_payer\">Payee Payer\n";
print "</select> And &nbsp;&nbsp;<input type=checkbox name=\"sort_journal\" value=\"ON\"> Journals</td></tr>\n";
print "<tr><td align=right>Search Operator&nbsp;&nbsp;&nbsp;</td><td align=left><select name=\"sort_operator\">\n";
print "<option value=\"AND\"> AND\n";
print "<option value=\"OR\"> OR\n";
print "</select></td></tr>\n";
print "<tr><td align=left colspan=2>";
print "How many entries to display at the same time:&nbsp;<select name=\"num_entries_to_display\">\n";
print "<option value=\"0\" selected>all\n";
print "<option value=\"10\">10\n";
print "<option value=\"30\">30\n";
print "<option value=\"50\">50\n";
print "</select></td></tr>\n";
print "<tr><td align=left colspan=2><font color=\"$menu_fgcolor\"><input type=submit name=\"search\" value=\"Find\" onClick=\"submit_my_form(document.journals_form, '$site_dir/cgi-bin/journal_mod1.cgi')\">&nbsp;<input type=reset value=\"Reset\"></font></td></tr>\n";
print "</table>\n";
print "<input type=hidden name=\"journal_view_command\" value=\"search\">\n";
print "<input type=hidden name=\"start_index\" value=\"1\">\n";
print "</form>\n";
print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
