#!/usr/bin/perl
#$Id: new_delete_ledger.cgi,v 1.2 2001/10/22 23:24:44 pcxuser Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# new_delete_ledger.cgi  -  Created by James Pattie, 05/18/1999
# This script allows the user to either delete
# a specified ledger or create a new ledger.  This script does
# the actual database work.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $mode;  # modes are 2=delete, 3=new
my $ledger_name;  # name of ledger to work with
my ($ledger_code, $ledger_db, $ledger_color, $cash_accrual, $business_type, $fiscal_month, $fiscal_day, $fiscal_current, $fiscal_year, $start_year, $yes_no, $old_ledger_name);
my ($q1_start, $q1_end, $q2_start, $q2_end, $q3_start, $q3_end, $q4_start, $q4_end);
my $error_string;  # used to store the returned value from IsValidString().

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# validate we got valid input from the user.
if (exists $input{'delete2'})
{
  $mode = 2;
  $ledger_name = $input{'ledger_name'};
  $error_string = IsValidString($ledger_name);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  $ledger_db   = $input{'db_name'};
  $error_string = IsValidString($ledger_db);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Database Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if (not (exists $input{'yes_no'}))
  {
    PrintErrorHeader();
    print "<h1>Error: You must say Yes or No!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $yes_no = $input{'yes_no'};
  if (not ($yes_no eq "yes" or $yes_no eq "no"))
  {
    PrintErrorHeader();
    print "<h1>Error: Invalid Option for Confirmation of deletion!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($yes_no eq "no")
  {
    PrintErrorHeader();
    print "<h1>I am not going to delete Ledger <font color=\"blue\">$ledger_name</font> until you say Yes!</h1>\n";
    print "$back_button";
    CleanUp();
  }
}
elsif (exists $input{'new2'})
{
  $ledger_name = $input{'ledger_name'};
  $error_string = IsValidString($ledger_name);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  $ledger_db   = $input{'db_name'};
  if (length $ledger_db == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must specify the Ledger Database name!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidString($ledger_db);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Database Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  $ledger_code = $input{'ledger_code'};
  if (length $ledger_code < 1)
  {
    PrintErrorHeader();
    print "<h1>Error: You have to enter at least one character for your code!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidString($ledger_code);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Code!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if (not exists $input{'color'})
  {
    PrintErrorHeader();
    print "<h1>Error: You must select a Color!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $ledger_color = $input{'color'};
  if (not exists $input{'cash_accrual'})
  {
    PrintErrorHeader();
    print "<h1>Error: You must select either Cash or Accrual!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $cash_accrual = $input{'cash_accrual'};
  if (not ($cash_accrual eq "cash" or $cash_accrual eq "accrual"))
  {
    PrintErrorHeader();
    print "<h1>Error: Invalid value for Cash or Accrual!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  $cash_accrual = "true" if ($input{'cash_accrual'} eq "cash");
  $cash_accrual = "false" if ($input{'cash_accrual'} eq "accrual");
  $business_type = $input{'business_type'};
  if (length $business_type == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must select a Business Type!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $fiscal_month = $input{'fiscal_month'};
  if (length $fiscal_month == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must enter a month between 1 and 12!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidInteger($fiscal_month);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Fiscal Month Value!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if ($fiscal_month < 1 or $fiscal_month > 12)
  {
    PrintErrorHeader();
    print "<h1>Error: You must enter a month between 1 and 12!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $fiscal_day = $input{'fiscal_day'};
  if (length $fiscal_day == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must enter a valid day for the month specified!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidInteger($fiscal_day);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Fiscal Day Value!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if ($fiscal_day < 1 or $fiscal_day > 31)
  {
    PrintErrorHeader();
    print "<h1>Error: The day you entered is obviously wrong!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 2 && $fiscal_day > 28)
  {
    PrintErrorHeader();
    print "<h1>Error:  February's official last day is the 28<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 4 && $fiscal_day == 31)
  {
    PrintErrorHeader();
    print "<h1>Error:  April's last day is the 30<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 6 && $fiscal_day == 31)
  {
    PrintErrorHeader();
    print "<h1>Error:  June's last day is the 30<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 9 && $fiscal_day == 31)
  {
    PrintErrorHeader();
    print "<h1>Error:  September's last day is the 30<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if ($fiscal_month == 11 && $fiscal_day == 31)
  {
    PrintErrorHeader();
    print "<h1>Error:  November's last day is the 30<sup>th</sup>!</h1>\n";
    print "$back_button";
    CleanUp();
  }
#  $fiscal_current = $input{'fiscal_year'};
#  if ($fiscal_current eq "Current")
#  {
#    $fiscal_current = "true";
#  }
#  elsif ($fiscal_current eq "Last")
#  {
#    $fiscal_current = "false";
#  }
#  else
#  {
#    PrintErrorHeader();
#    print "<h1>Error:  Invalid value for Fiscal Year Current or Previous!</h1>\n";
#    print "$sys_admin";
#    CleanUp();
#  }
  $fiscal_year = $input{'fiscal_year'};
  if ($fiscal_year <= 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid value for Fiscal Year!</h1>\n";
    print "You have to enter a year > 0.<br>$back_button";
    CleanUp();
  }
  $error_string = IsValidInteger($fiscal_year);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Fiscal Year Value!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  $start_year = $input{'start_year'};
  if ($start_year <= 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid value for Starting Book Year!</h1>\n";
    print "You have to enter a year > 0.<br>$back_button";
    CleanUp();
  }
  $error_string = IsValidInteger($start_year);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Start Year Value!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  # if fiscal_year is equal to start_year, fiscal_current is true
  if ($fiscal_year == $start_year)
  {
    $fiscal_current = "true";
  }
  else
  {
    $fiscal_current = "false";
  }

  $q1_start = $input{'q1_start'};
  $q1_end = $input{'q1_end'};
  $q2_start = $input{'q2_start'};
  $q2_end = $input{'q2_end'};
  $q3_start = $input{'q3_start'};
  $q3_end = $input{'q3_end'};
  $q4_start = $input{'q4_start'};
  $q4_end = $input{'q4_end'};

  if (length $q1_start == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Quarter 1 Start Date!</h1>\n";
    print "You must enter a date.<br>$back_button";
    CleanUp();
  }
  if (not IsValidDateSyntax($q1_start))
  {
    PrintErrorHeader();
    print "<h1>Error:  Quarter 1 Start Date is invalid!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if (length $q1_end == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Quarter 1 End Date!</h1>\n";
    print "You must enter a date.<br>$back_button";
    CleanUp();
  }
  if (not IsValidDateSyntax($q1_end))
  {
    PrintErrorHeader();
    print "<h1>Error:  Quarter 1 End Date is invalid!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if (length $q2_start == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Quarter 2 Start Date!</h1>\n";
    print "You must enter a date.<br>$back_button";
    CleanUp();
  }
  if (not IsValidDateSyntax($q2_start))
  {
    PrintErrorHeader();
    print "<h1>Error:  Quarter 2 Start Date is invalid!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if (length $q2_end == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Quarter 2 End Date!</h1>\n";
    print "You must enter a date.<br>$back_button";
    CleanUp();
  }
  if (not IsValidDateSyntax($q2_end))
  {
    PrintErrorHeader();
    print "<h1>Error:  Quarter 2 End Date is invalid!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if (length $q3_start == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Quarter 3 Start Date!</h1>\n";
    print "You must enter a date.<br>$back_button";
    CleanUp();
  }
  if (not IsValidDateSyntax($q3_start))
  {
    PrintErrorHeader();
    print "<h1>Error:  Quarter 3 Start Date is invalid!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if (length $q3_end == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Quarter 3 End Date!</h1>\n";
    print "You must enter a date.<br>$back_button";
    CleanUp();
  }
  if (not IsValidDateSyntax($q3_end))
  {
    PrintErrorHeader();
    print "<h1>Error:  Quarter 3 End Date is invalid!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if (length $q4_start == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Quarter 4 Start Date!</h1>\n";
    print "You must enter a date.<br>$back_button";
    CleanUp();
  }
  if (not IsValidDateSyntax($q4_start))
  {
    PrintErrorHeader();
    print "<h1>Error:  Quarter 4 Start Date is invalid!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  if (length $q4_end == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Quarter 4 End Date!</h1>\n";
    print "You must enter a date.<br>$back_button";
    CleanUp();
  }
  if (not IsValidDateSyntax($q4_end))
  {
    PrintErrorHeader();
    print "<h1>Error:  Quarter 4 End Date is invalid!</h1>\n";
    print "$back_button";
    CleanUp();
  }

  $mode = 3;
}
elsif (exists $input{'edit2'})
{
  $ledger_name = $input{'ledger_name'};
  $error_string = IsValidString($ledger_name);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  $old_ledger_name = $input{'old_ledger_name'};
  $error_string = IsValidString($old_ledger_name);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Old Ledger Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  $ledger_db   = $input{'db_name'};
  if (length $ledger_db == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must specify the Ledger Database name!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidString($ledger_db);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Database Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  $ledger_code = $input{'ledger_code'};
  if (length $ledger_code < 1)
  {
    PrintErrorHeader();
    print "<h1>Error: You have to enter at least one character for your code!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $error_string = IsValidString($ledger_code);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid Ledger Code!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if (not exists $input{'color'})
  {
    PrintErrorHeader();
    print "<h1>Error: You must select a Color!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $ledger_color = $input{'color'};
  if (not exists $input{'cash_accrual'})
  {
    PrintErrorHeader();
    print "<h1>Error: You must select either Cash or Accrual!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $cash_accrual = $input{'cash_accrual'};
  if (not ($cash_accrual eq "cash" or $cash_accrual eq "accrual"))
  {
    PrintErrorHeader();
    print "<h1>Error: Invalid value for Cash or Accrual!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  $cash_accrual = "true" if ($input{'cash_accrual'} eq "cash");
  $cash_accrual = "false" if ($input{'cash_accrual'} eq "accrual");
  $business_type = $input{'business_type'};
  if (length $business_type == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: You must select a Business Type!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  #$fiscal_month = $input{'fiscal_month'};
  #if (length $fiscal_month == 0)
  #{
  #  PrintErrorHeader();
  #  print "<h1>Error: You must enter a month between 1 and 12!</h1>\n";
  #  print "$back_button";
  #  CleanUp();
  #}
  #if ($fiscal_month < 1 or $fiscal_month > 12)
  #{
  #  PrintErrorHeader();
  #  print "<h1>Error: You must enter a month between 1 and 12!</h1>\n";
  #  print "$back_button";
  #  CleanUp();
  #}
  #$fiscal_day = $input{'fiscal_day'};
  #if (length $fiscal_day == 0)
  #{
  #  PrintErrorHeader();
  #  print "<h1>Error: You must enter a valid day for the month specified!</h1>\n";
  #  print "$back_button";
  #  CleanUp();
  #}
  #if ($fiscal_day < 1 or $fiscal_day > 31)
  #{
  #  PrintErrorHeader();
  #  print "<h1>Error: The day you entered is obviously wrong!</h1>\n";
  #  print "$back_button";
  #  CleanUp();
  #}
  #$fiscal_current = $input{'fiscal_year'};
  #if ($fiscal_current eq "Current")
  #{
  #  $fiscal_current = "true";
  #}
  #elsif ($fiscal_current eq "Last")
  #{
  #  $fiscal_current = "false";
  #}
  #else
  #{
  #  PrintErrorHeader();
  #  print "<h1>Error:  Invalid value for Fiscal Year Current or Previous!</h1>\n";
  #  print "$sys_admin";
  #  CleanUp();
  #}
  $mode = 4;
}
else
{
  PrintErrorHeader();
  print "<h1>Access Denied!</h1>\n";
  print "Error: Invalid options given!\n";
  CleanUp();
}

if (length $ledger_name == 0)
{
  PrintErrorHeader();
  print "Error: You must specify a ledger to work with!<br>\n";
  print "$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid, if in mode 2
# or that it doesn't already exist if in mode 3.
if ($mode == 4 && (not $old_ledger_name eq $ledger_name))
{
  $result = $conn->exec("select database_name from entity_tb where name = '$old_ledger_name'");
}
else
{
  $result = $conn->exec("select database_name from entity_tb where name = '$ledger_name'");
}
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($mode == 2)
{
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: Ledger does not exist!</h1>\n";
    print "The Ledger may have already been deleted!\n";
    print "$sys_admin";
    CleanUp();
  }
  else
  {
    $ledger_db = $result->getvalue(0, 0);
  }
}
elsif ($mode == 3)
{ # we want to create a new one so make sure it doesn't exist
  if ($result->ntuples > 0)
  {
    PrintErrorHeader();
    print "<h1>Error: Ledger already exists!</h1>\n";
    print "$back_button";
    CleanUp();
  }
}
elsif ($mode == 4)
{
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Error: Ledger does not exist!</h1>\n";
    print "The Ledger may have already been deleted!\n";
    print "$sys_admin";
    CleanUp();
  }
}

# if the mode is create or edit ledger, then validate that the business type is valid.
if ($mode == 3 || $mode == 4)
{
  $result = $conn->exec("select name from business_type_tb where num = $business_type");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing database failed!</h1>\n";
    print "Trying to check business type validity.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples == 0)
  {
    PrintErrorHeader();
    print "<h1>Internal Error: No Business Types defined!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  if ($result->ntuples != 1)
  {
    PrintErrorHeader();
    print "<h1>Error: Invalid option for Business Type!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
}

# present them with the main menu 

print "<head><title>$prog_name $prog_version Ledger Maintenance</title>\n";
if ($mode == 2)
{
  print "     <Meta http-equiv=\"refresh\" content=\"3; URL=/portal/cgi-bin/index.cgi?app=Accounting&state=Main&command=close_ledger&ledger=$ledger_name\">\n";
}

if ($mode == 3 || $mode == 4)  # need some Javascript here
{
  print <<"END_OF_STYLE";
  <script language="Javascript1.2">
  <!--
    function SubmitForm()
    { // This function will submit the assign_account_form with the needed stuff set.
      document.assign_account_form.submit();
    }
  //-->
  </script>
END_OF_STYLE
}

print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left width=47%><font color=\"red\">";
if ($mode == 2)
{
  print "Deleting Ledger <font color=\"blue\">$ledger_name</font>";
}
elsif ($mode == 3)
{
  print "Creating Ledger <font color=\"blue\">$ledger_name</font>";
}
elsif ($mode == 4)
{
  print "Editing Ledger <font color=\"blue\">$ledger_name</font>";
}
print "</font></td>\n";
if ($mode == 2)
{
  print "<td align=left><a href=\"$site_dir/help.html#ledger_deletion\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
}
elsif ($mode == 3)
{
  print "<td align=left><a href=\"$site_dir/help.html#ledger_creation\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
}
elsif ($mode == 4)
{
  print "<td align=left><a href=\"$site_dir/help.html#ledger_editing\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
}
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);

# for now I'm assuming that we really delete the ledger or create it and
# don't support the feature of just removing it from the list of entities, etc.

if ($mode == 2) # delete the ledger
{
  $result = $conn->exec("begin work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: starting transaction failed!</h1>\n";
    print "Before ledger deletion.<br>\n";
    print "$sys_admin";
    CleanUp();
  }
  else
  {
    $transaction = 1;
  }

  # get the ledger_code from the ledger table
  $result = $conn->exec("SELECT code FROM ledger_tb WHERE name = '$ledger_name'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get ledger code for ledger = '$ledger_name'.<br>$sys_admin";
    CleanUp();
  }
  $ledger_code = $result->getvalue(0, 0);


  # first we need to remove the ledger name from the main database's tables.
  $result = $conn->exec("delete from entity_tb where name = '$ledger_name'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Deleting Ledger <font color=\"blue\">$ledger_name</font> from main database failed!</h1>\n";
    print "Tried to remove ledger.<br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    $result = $conn->exec("rollback work");
    CleanUp();
  }

  # now we need to remove all entries from the time_tracker_config_tb and tips_tb tables that reference this ledger.
  $result = $conn->exec("DELETE FROM time_tracker_config_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing Time Tracker Configuration data for Ledger '$ledger_name' failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  $result = $conn->exec("DELETE FROM tips_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing Tips data for Ledger '$ledger_name' failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # in the future all other ledger related tables will also be purged.
  # ledger table
  $result = $conn->exec("DELETE FROM ledger_tb WHERE code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing ledger_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # variables table
  $result = $conn->exec("DELETE FROM variables_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing variables_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # known_years table
  $result = $conn->exec("DELETE FROM known_years_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing known_years_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # sub_entity table
  $result = $conn->exec("DELETE FROM sub_entity_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing sub_entity_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # journal_entry table
  $result = $conn->exec("DELETE FROM journal_entry_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing journal_entry_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # debitcredit table
  $result = $conn->exec("DELETE FROM debitcredit_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing debitcredit_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # invoice table
  $result = $conn->exec("DELETE FROM invoice_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing invoice_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # invoice_items table
  $result = $conn->exec("DELETE FROM invoice_items_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing invoice_items_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # billing_tb table
  $result = $conn->exec("DELETE FROM billing_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing billing_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # group table
  $result = $conn->exec("DELETE FROM group_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing group_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # client_group table
  $result = $conn->exec("DELETE FROM client_group_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing client_group_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # asset table
  $result = $conn->exec("DELETE FROM asset_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing asset_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # reminder table
  $result = $conn->exec("DELETE FROM reminder_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing reminder_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # custom_report table
  $result = $conn->exec("DELETE FROM custom_report_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing custom_report_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # acct_bal table
  $result = $conn->exec("DELETE FROM acct_bal_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing acct_bal_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # beginning_balance_entry table
  $result = $conn->exec("DELETE FROM beginning_balance_entry_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing beginning_balance_entry_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # tax_non_tax table
  $result = $conn->exec("DELETE FROM local_accounts_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing local_accounts_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # sec_inv table
  $result = $conn->exec("DELETE FROM sec_inv_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing sec_inv_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # sec_event table
  $result = $conn->exec("DELETE FROM sec_event_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing sec_event_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # dep_sched table
  $result = $conn->exec("DELETE FROM dep_sched_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing dep_sched_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # report_index table
  $result = $conn->exec("DELETE FROM report_index_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing report_index_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # report table
  $result = $conn->exec("DELETE FROM report_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing report_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # memorized_report table
  $result = $conn->exec("DELETE FROM memorized_report_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing memorized_report_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # tax_loss table
  $result = $conn->exec("DELETE FROM tax_loss_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing tax_loss_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # scratch table
  $result = $conn->exec("DELETE FROM scratch_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing scratch_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
  
  # lot_node_tb
  $result = $conn->exec("DELETE FROM lot_node_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing lot_node_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
  
  # sec_event_tb
  $result = $conn->exec("DELETE FROM sec_event_tb WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Removing sec_event_tb entries failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # now we commit our deletion
  $result = $conn->exec("commit work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: committing work failed!</h1>\n";
    print "After ledger removal.<br>\n";
    print "$sys_admin";
    CleanUp();
  }
  else
  {
    $transaction = 0;
  }

  print "<h2>Deletion of Ledger <font color=\"blue\">$ledger_name</font> Succeeded!</h2><br>\n";
  print "<br><br><br>You will be returned to the Main Menu in 3 seconds.<br>\n";
}
elsif ($mode == 3) # create the ledger
{
  # now we store the ledger info in the entity_tb table in the main database and then connect to the new database
  # and store the detailed info there for this ledger.
  $result = $conn->exec("begin work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Starting a database transaction failed!</h1>\n";
    print "Before adding new ledger info in main database.<br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
  else
  {
    $transaction = 1;
  }

  $result = $conn->exec("insert into entity_tb (name, database_name) values ('$ledger_name', '$ledger_db')");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Updating ledger tables in main database failed!</h1>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    $result = $conn->exec("rollback work");
    CleanUp();
  }

  # create the entries in the time_tracker_config_tb table
  $result = $conn->exec("INSERT INTO time_tracker_config_tb (ledger_code) values ('$ledger_code')");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Inserting Default Time Tracker Configuration Data failed!</h1>\n";
    print $conn->errorMessage . "<br>Ledger Code = '$ledger_code'.<br>$sys_admin";
    CleanUp();
  }

  # ****************************
  # insert default data

  $result = $conn->exec("insert into scratch_tb (validate, ledger_code) values ('01-01-2000', '$ledger_code')");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Inserting Default Data into Scratch Table!</h1>\n";
    print $conn->errorMessage . "<br>Ledger Code = '$ledger_code'.<br>$sys_admin";
    CleanUp();
  }

  $result = $conn->exec("insert into ledger_tb (name, code, color, cash_or_accrual, business_type, fy_mo, fy_day, fy_current) values ('$ledger_name', '$ledger_code', '$ledger_color', $cash_accrual, $business_type, $fiscal_month, $fiscal_day, $fiscal_current)");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Inserting ledger info into database failed!</h1>\n";
    print "After connecting to new Ledger <font color=\"blue\">$ledger_db</font> and starting transaction.<br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # create the entries for the working year.
  $result = $conn->exec("insert into variables_tb (working_year, ledger_code) values ($start_year, '$ledger_code')");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Creating the Working Year value failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to set working_year = <font color=\"blue\">$start_year</font>.<br>$sys_admin";
    CleanUp();
  }
#***************************************************!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  # I need to calculate the beginning and ending dates for the current fiscal year and insert them into the known_years_tb table.
  my ($fiscal_start_date, $fiscal_end_date);
  if ($fiscal_month == 1 && $fiscal_day == 31)
  {
    $fiscal_start_date = "02/01/";
  }
  elsif ($fiscal_month == 2 && $fiscal_day == 28)
  {
    $fiscal_start_date = "03/01/";
  }
  elsif ($fiscal_month == 3 && $fiscal_day == 31)
  {
    $fiscal_start_date = "04/01/";
  }
  elsif ($fiscal_month == 4 && $fiscal_day == 30)
  {
    $fiscal_start_date = "05/01/";
  }
  elsif ($fiscal_month == 5 && $fiscal_day == 31)
  {
    $fiscal_start_date = "06/01/";
  }
  elsif ($fiscal_month == 6 && $fiscal_day == 30)
  {
    $fiscal_start_date = "07/01/";
  }
  elsif ($fiscal_month == 7 && $fiscal_day == 31)
  {
    $fiscal_start_date = "08/01/";
  }
  elsif ($fiscal_month == 8 && $fiscal_day == 31)
  {
    $fiscal_start_date = "09/01/";
  }
  elsif ($fiscal_month == 9 && $fiscal_day == 30)
  {
    $fiscal_start_date = "10/01/";
  }
  elsif ($fiscal_month == 10 && $fiscal_day == 31)
  {
    $fiscal_start_date = "11/01/";
  }
  elsif ($fiscal_month == 11 && $fiscal_day == 30)
  {
    $fiscal_start_date = "12/01/";
  }
  elsif ($fiscal_month == 12 && $fiscal_day == 31)
  {
    $fiscal_start_date = "01/01/";
  }
  else # increment the day by one as we are not at a month ending.
  {
    $fiscal_start_date = "";
    $fiscal_start_date = "0" if ($fiscal_month < 10);
    $fiscal_start_date .= $fiscal_month;
    $fiscal_start_date .= "/";
    my $temp = $fiscal_day + 1;
    $fiscal_start_date .= "0" if ($temp < 10);
    $fiscal_start_date .= $temp;
    $fiscal_start_date .= "/";
  }

  # now we figure out the year to use for the start date.
  if ($fiscal_month == 12 && $fiscal_day == 31) # we are on a calendar year
  {
    $fiscal_start_date .= $start_year;
  }
  else # we have to use the fiscal_current value to determine the year.
  {
    if ($fiscal_current eq "true")
    {
      my $temp = $start_year - 1;
      $fiscal_start_date .= $temp;
    }
    else
    {
      $fiscal_start_date .= $start_year;
    }
  }

  # create the fiscal_end_date value
  $fiscal_end_date = "0" if ($fiscal_month < 10);
  $fiscal_end_date .= $fiscal_month;
  $fiscal_end_date .= "/";
  $fiscal_end_date .= "0" if ($fiscal_day < 10);
  $fiscal_end_date .= $fiscal_day;
  $fiscal_end_date .= "/";
  # now we figure out the year to use for the end date.
  if ($fiscal_month == 12 && $fiscal_day == 31) # we are on a calendar year
  {
    $fiscal_end_date .= $start_year;
  }
  else # we have to use the fiscal_current value to determine the year.
  {
    if ($fiscal_current eq "true")
    {
      $fiscal_end_date .= $start_year;
    }
    else
    {
      my $temp = $start_year + 1;
      $fiscal_end_date .= $temp;
    }
  }

  $result = $conn->exec("insert into known_years_tb (year, closed, start_date, end_date, q1_start, q1_end, q2_start, q2_end, q3_start, q3_end, q4_start, q4_end, ledger_code) values ($start_year, false, '$fiscal_start_date', '$fiscal_end_date', '$q1_start', '$q1_end', '$q2_start', '$q2_end', '$q3_start', '$q3_end', '$q4_start', '$q4_end', '$ledger_code')");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error:  Creating the Known Years values failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to set year = <font color=\"blue\">$start_year</font>, closed = false.<br>$sys_admin";
    CleanUp();
  }

  # commit work
  $result = $conn->exec("commit work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Committing work failed!</h1>\n";
    print "After adding new ledger info to new database <font color=\"blue\">$ledger_db</font>.<br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  print "<h1>Ledger <font color=\"blue\">$ledger_name</font> Successfully Created!</h1>\n";
  print "<form name=\"assign_account_form\" method=post action=\"$site_dir/cgi-bin/assign_accounts2.cgi\">\n";
  print "<input type=hidden name=\"select_ledger\" value=\"select_ledger\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
  print "<br><br><a href=\"\" onClick=\"SubmitForm(); return false;\">You need to assign accounts to this Ledger <u>here</u></a><br>\n";
  print "</form>\n";
}
elsif ($mode == 4)
{ # we are just going to update the ledger_tb entries for this ledger.

  # first we check to see if they changed the ledger name.
  if (not $old_ledger_name eq $ledger_name)
  {
    # now we store the ledger info in the entity_tb table in the main database and then connect to the new database
    # and store the detailed info there for this ledger.
    $result = $conn->exec("begin work");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
      print "<h1>Error: Starting a database transaction failed!</h1>\n";
      print "Before updating ledger info in main database.<br>\n";
      print $conn->errorMessage . "<br>$sys_admin";
      CleanUp();
    }
    else
    {
      $transaction = 1;
    }

    $result = $conn->exec("update entity_tb set name = '$ledger_name' where database_name = '$ledger_db'");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
      print "<h1>Error: Updating ledger tables in main database failed!</h1>\n";
      print $conn->errorMessage . "<br>$sys_admin";
      $result = $conn->exec("abort transaction");
      CleanUp();
    }

    # now we commit the work
    $result = $conn->exec("commit work");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
      print "<h1>Error: Committing work failed!</h1>\n";
      print "After updating ledger info in main database.<br>\n";
      print $conn->errorMessage . "<br>$sys_admin";
      CleanUp();
    }
    else
    {
      $transaction = 0;
    }
  }

  # begin a transaction first
  $result = $conn->exec("begin work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Starting a database transaction failed!</h1>\n";
    print "After connecting to Ledger <font color=\"blue\">$ledger_db</font>.<br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  $result = $conn->exec("update ledger_tb set name = '$ledger_name', color = '$ledger_color', cash_or_accrual = $cash_accrual, business_type = $business_type WHERE code = '$ledger_code'"); #, fy_mo = $fiscal_month, fy_day = $fiscal_day, fy_current = $fiscal_current");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Updating ledger info in database failed!</h1>\n";
    print "After connecting to Ledger <font color=\"blue\">$ledger_db</font> and starting transaction.<br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  # commit work
  $result = $conn->exec("commit work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Committing work failed!</h1>\n";
    print "After updating ledger info in Ledger <font color=\"blue\">$ledger_db</font>.<br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }

  print "<h1>Ledger <font color=\"blue\">$ledger_name</font> Successfully Updated!</h1>\n";
  print "<form name=\"assign_account_form\" method=post action=\"$site_dir/cgi-bin/assign_accounts2.cgi\">\n";
  print "<input type=hidden name=\"select_ledger\" value=\"select_ledger\">\n";
  print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><br><a href=\"\" onClick=\"SubmitForm(); return false;\">To assign accounts to this Ledger go <u>here</u></a><br>\n";
  print "</form>\n";
}

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if ($mode == 4)
{
  if (exists $input{'ledger_open'})
  {
    print "<form method=post action=\"$site_dir/cgi-bin/ledger.cgi\" name=\"menu_bar\">\n";
    print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
    print "<input type=hidden name=\"open\" value=\"\">\n";
    print "<div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"document.menu_bar.submit(); return false;\">Return to Ledger Summary Screen</a></div>\n";
    print "</form>\n";
  }
}
else
{
  print "<br>";
}

print "</body>\n";
print "</html>\n";
exit 0;
