#!/usr/bin/perl
#$Id: register_display.cgi,v 1.4 2003/07/22 21:03:41 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# register_display.cgi  -  Created by Jason A. Pattie, 10/18/1999
# This script does the work of displaying the account register for the given
# account and range of dates entered on the previous form.  It displays the
# information and allows the user to click on a particular item in order to
# be allowed to edit it (if it is within the current working date range) and
# to update its cleared or reconciled status.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$count';
use vars '$count2';
use vars '$from_search_date';
use vars '$to_search_date';
use vars '$reverse_from';
use vars '$reverse_to';
use vars '$start_known';
use vars '$end_known';
use vars '$reverse_start_known';
use vars '$reverse_end_known';
use vars '$temp_day';
use vars '$temp_month';
use vars '$temp_year';
use vars '$account_name';
use vars '$account_num';
use vars '$account_type';
use vars '$debit_increases';
use vars '$current_fiscal_year';
use vars '$start_fiscal';
use vars '$stop_fiscal';
use vars '$start_month_fiscal';
use vars '$stop_month_fiscal';
use vars '$start_day_fiscal';
use vars '$stop_day_fiscal';
use vars '$start_year_fiscal';
use vars '$stop_year_fiscal';
use vars '@ref_num_array';
use vars '@trans_date_array';
use vars '@payee_payer_array';
use vars '@memo_array';
use vars '@sub_account_array';
use vars '@sub_account_name_array';
use vars '@amount_array';
use vars '@dorc_array';
use vars '@cleared_array';
use vars '@counter_array';
use vars '@o_ref_num_array';
use vars '@o_trans_date_array';
use vars '@o_payee_payer_array';
use vars '@o_memo_array';
use vars '@o_sub_account_array';
use vars '@o_sub_account_name_array';
use vars '@o_amount_array';
use vars '@o_dorc_array';
use vars '@o_cleared_array';
use vars '@o_counter_array';
use vars '$temp_month';
use vars '$temp_day';
use vars '$temp_year';
use vars '$balance_so_far';
use vars '$balance_date';
use vars '$beginning_balance_date';
use vars '$ending_balance_date';
use vars '$sum_debits';
use vars '$sum_credits';
use vars '$sum_outstanding_debits';
use vars '$sum_outstanding_credits';
use vars '$reverse_balance';
use vars '$temp_amount';
use vars '$count_temp';
my $error_string;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

my $ledger_name;  # name of ledger to work with
my $ledger_db;
my $ledger_color;
my $result2;
my $ledger_code;
my $report_obj;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

$ledger_db = $input{'ledger_db'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}
$error_string = IsValidString($ledger_db);
if (length $error_string > 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Database Name!</h1>\n";
  print "<b>$error_string</b> is invalid!<br>$back_button";
  CleanUp();
}

if (not exists $input{'register_select_submit'})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Invalid options.<br>$sys_admin";
  CleanUp();
}

if (not exists $input{'account_select'})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Missing necessary form value (account select).<br>Invalid options.<br>$sys_admin";
  CleanUp();
}
if (not exists $input{'from_search_date'})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Missing necessary form value (from search date).<br>Invalid options.<br>$sys_admin";
  CleanUp();
}
if (not IsValidDateSyntax($input{'from_search_date'}))
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid From Date!</h1>\n";
  print "$back_button";
  CleanUp();
}
if (not exists $input{'to_search_date'})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "Missing necessary form value (to search date).<br>Invalid options.<br>$sys_admin";
  CleanUp();
}
if (not IsValidDateSyntax($input{'to_search_date'}))
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid To Date!</h1>\n";
  print "$back_button";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now validate that the ledger they specified is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# get the account name - number pair and the from/to search dates
$from_search_date = $input{'from_search_date'};
# validate from_search_date
if (!IsValidDateSyntax($from_search_date))
{
  PrintErrorHeader();
  print "<h1>Error: Invalid From Date Syntax!</h1>\n";
  print "$back_button";
  CleanUp();
}
# substitute all '-'s for '/'s
$from_search_date =~ s/\-/\//g;
$reverse_from = `date -d "$from_search_date" +%Y%m%d`;
chomp $reverse_from;
$from_search_date = `date -d "$from_search_date" +%m-%d-%Y`;
chomp $from_search_date;
$to_search_date = $input{'to_search_date'};
# validate to_search_date
if (!IsValidDateSyntax($to_search_date))
{
  PrintErrorHeader();
  print "<h1>Error: Invalid To Date Syntax!</h1>\n";
  print "$back_button";
  CleanUp();
}
# substitute all '-'s for '/'s
$to_search_date =~ s/\-/\//g;
$reverse_to = `date -d "$to_search_date" +%Y%m%d`;
chomp $reverse_to;
$to_search_date = `date -d "$to_search_date" +%m-%d-%Y`;
chomp $to_search_date;
$ending_balance_date = $to_search_date;
($account_name, $account_num) = split /\|/, $input{'account_value'};

# get the type of the account
# and whether debits increase or decrease the value of the account
# bank account  - dont dereference sub account numbers
# investment account - only select -1 sub account rows
$result = $conn->exec("select account_tb.type, acct_type_tb.debit_increases where account_tb.number = $account_num and account_tb.type = acct_type_tb.num");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the account type and debit direction (+/-) from the database for account $account_num.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 1)
{
  PrintErrorHeader();
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print $result->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
  CleanUp();
}
$account_type = $result->getvalue(0, 0);
$debit_increases = $result->getvalue(0, 1);

# make sure that a search outside the known years in the database is not being attempted
$result = $conn->exec("select min(start_date) from known_years_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the minimum starting date from the known_years_tb table.<br>$sys_admin";
  CleanUp();
}
if ($result->getvalue(0, 0) ne "")
{
  $start_known = $result->getvalue(0, 0);
  ($temp_month, $temp_day, $temp_year) = split "-", $start_known;
  $reverse_start_known = $temp_year . $temp_month . $temp_day;
}
else
{
  $start_known = $from_search_date;
  $reverse_start_known = $reverse_from;
}
$result = $conn->exec("select max(end_date) from known_years_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get the maximum ending date from the known_years_tb table.<br>$sys_admin";
  CleanUp();
}
if ($result->getvalue(0, 0) ne "")
{
  $end_known = $result->getvalue(0, 0);
  ($temp_month, $temp_day, $temp_year) = split "-", $end_known;
  $reverse_end_known = $temp_year . $temp_month . $temp_day;
}
else
{
  $end_known = $to_search_date;
  $reverse_end_known = $reverse_to;
}
if ($reverse_from < $reverse_start_known) # || $reverse_to > $reverse_end_known)
{
  PrintErrorHeader();
  print "<h1>Error: Data for Range of Search Does Not Exist!</h1>\n";
  print "The date(s) entered is less than the beginning date of the first known year in the database.<br>\n";
  print "$back_button";
  CleanUp();
}

# get all journal entries that are not cleared if the show outstanding transactions button has been checked on the previous form
if (exists $input{'display_outstanding'})
{
  $sum_outstanding_debits = 0.0;
  $sum_outstanding_credits = 0.0;

  my $sqlselect = "select journal_entry_tb.ref_num, journal_entry_tb.trans_date, journal_entry_tb.payee_payer, journal_entry_tb.memo, debitcredit_tb.sub_account_num, debitcredit_tb.amount, debitcredit_tb.dorc, debitcredit_tb.cleared, debitcredit_tb.counter, debitcredit_tb.account_num FROM journal_entry_tb, debitcredit_tb where debitcredit_tb.account_num = $account_num and journal_entry_tb.trans_date < '$from_search_date' and journal_entry_tb.ref_num = debitcredit_tb.parent_num and debitcredit_tb.cleared <> 'r' AND journal_entry_tb.ledger_code = '$ledger_code' AND debitcredit_tb.ledger_code = '$ledger_code'";
  if ($account_type == 3)
  {
    $sqlselect .= " and debitcredit_tb.sub_account_num = '-1'";
  }
  $sqlselect .= " order by journal_entry_tb.trans_date, debitcredit_tb.dorc desc, debitcredit_tb.sub_account_num, journal_entry_tb.ref_num";
  $result = $conn->exec($sqlselect);
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get all the info. from the journal_entry_tb and debitcredit_tb tables for account $account_num<br>for outstanding transactions before the date range specified.<br>$sys_admin";
    CleanUp();
  }
  for ($count=0; $count < $result->ntuples; $count++)
  {
    # load the oustanding results arrays with the values of the result set
    $o_ref_num_array[$count] = $result->getvalue($count, 0);
    $o_trans_date_array[$count] = $result->getvalue($count, 1);
    $o_payee_payer_array[$count] = $result->getvalue($count, 2);
    $o_memo_array[$count] = $result->getvalue($count, 3);
    $o_sub_account_array[$count] = $result->getvalue($count, 4);
    # do not lookup the sub-account name if the account type is cash/bank/sales revenue
    if ($account_type == 0 || $account_type == 10 || $account_type == 3)
    {
      $o_sub_account_name_array[$count] = "";
    }
    else
    {
      $result2 = $conn->exec("select sub_acc_name from sub_acc_tb where sub_acc_number = $o_sub_account_array[$count]");
      if ($result2->resultStatus != PGRES_TUPLES_OK)
      {
        PrintErrorHeader();
        print "<h1>Error: Accessing database failed!</h1>\n";
        print $conn->errorMessage . "<br>1036: Tried to get the sub-account name for sub-account: $o_sub_account_array[$count].<br>$sys_admin";
        CleanUp();
      }
      if ($result2->ntuples > 1)
      {
        PrintErrorHeader();
        print "<h1>Error: Wrong number of records returned!</h1>\n";
        print "1037: " . $result2->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
        CleanUp();
      }
      if ($result2->ntuples != 0)
      {
        $o_sub_account_name_array[$count] = $result2->getvalue(0, 0);
      }
      else
      {
        $o_sub_account_name_array[$count] = "";
      }
    }
    $o_amount_array[$count] = $result->getvalue($count, 5);
    $o_amount_array[$count] =~ s/\$//g;
    $o_dorc_array[$count] = $result->getvalue($count, 6);
    $o_cleared_array[$count] = $result->getvalue($count, 7);
    $o_counter_array[$count] = $result->getvalue($count, 8);
    if ($o_dorc_array[$count] eq "t")
    {
      $sum_outstanding_debits += $o_amount_array[$count];
    }
    else
    {
      $sum_outstanding_credits += $o_amount_array[$count];
    }
  }
}

# get all journal entries that have an entry that uses the account and are between the from and to search dates
# for investment accounts only show -1 subaccount entries
my $sqlstatement;
$sqlstatement = "select journal_entry_tb.ref_num, journal_entry_tb.trans_date, journal_entry_tb.payee_payer, journal_entry_tb.memo, debitcredit_tb.sub_account_num, debitcredit_tb.amount, debitcredit_tb.dorc, debitcredit_tb.cleared, debitcredit_tb.counter, debitcredit_tb.account_num FROM journal_entry_tb, debitcredit_tb where debitcredit_tb.account_num = $account_num and journal_entry_tb.trans_date between '$from_search_date' and '$to_search_date' and journal_entry_tb.ref_num = debitcredit_tb.parent_num AND journal_entry_tb.ledger_code = '$ledger_code' AND debitcredit_tb.ledger_code = '$ledger_code'";
  if ($account_type == 3)
  {
    $sqlstatement .= " and debitcredit_tb.sub_account_num = '-1'";
  }
  $sqlstatement .= " order by journal_entry_tb.trans_date, debitcredit_tb.dorc desc, debitcredit_tb.sub_account_num, journal_entry_tb.ref_num";
$result = $conn->exec($sqlstatement);
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get all the info. from the journal_entry_tb and debitcredit_tb tables for account $account_num.<br>$sys_admin";
  CleanUp();
}
for ($count=0; $count < $result->ntuples; $count++)
{
  # load the results arrays with the values of the result set
  $ref_num_array[$count] = $result->getvalue($count, 0);
  $trans_date_array[$count] = $result->getvalue($count, 1);
  $payee_payer_array[$count] = $result->getvalue($count, 2);
  $memo_array[$count] = $result->getvalue($count, 3);
  $sub_account_array[$count] = $result->getvalue($count, 4);
  # do not lookup the sub-account name if the account type is cash/bank/sales revenue
  if ($account_type == 0 || $account_type == 10)
  {
    $sub_account_name_array[$count] = "";
  }
  else
  {
    $result2 = $conn->exec("select sub_acc_name from sub_acc_tb where sub_acc_number = $sub_account_array[$count]");
    if ($result2->resultStatus != PGRES_TUPLES_OK)
    {
      PrintErrorHeader();
      print "<h1>Error: Accessing database failed!</h1>\n";
      print $conn->errorMessage . "<br>1038: Tried to get the sub-account name for sub-account: $sub_account_array[$count].<br>$sys_admin";
      CleanUp();
    }
    if ($result2->ntuples > 1)
    {
      PrintErrorHeader();
      print "<h1>Error: Wrong number of records returned!</h1>\n";
      print "1039: " . $result2->ntuples . " records were returned.  There should be only one.<br>$sys_admin";
      CleanUp();
    }
    if ($result2->ntuples != 0)
    {
      $sub_account_name_array[$count] = $result2->getvalue(0, 0);
    }
    else
    {
      $sub_account_name_array[$count] = "";
    }
  }
  $amount_array[$count] = $result->getvalue($count, 5);
  $amount_array[$count] =~ s/\$//g;
  $dorc_array[$count] = $result->getvalue($count, 6);
  $cleared_array[$count] = $result->getvalue($count, 7);
  $counter_array[$count] = $result->getvalue($count, 8);
}

# get current fiscal year and verify that it is not closed
$result = $conn->exec("select working_year from variables_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get current fiscal year from database.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 1)
{
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "There should only be one record in the variables_tb.  " . $result->ntuples . " were returned.<br>$sys_admin";
  CleanUp();
}
$current_fiscal_year = $result->getvalue(0, 0);
# get the date ranges for the current fiscal year if it is not closed
$result = $conn->exec("select closed, start_date, end_date from known_years_tb where year = $current_fiscal_year AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error: Accessing database <font color=\"blue\"$ledger_db</font> failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get current fiscal year info. from database.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples != 1)
{
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "There should only be one record for each individual fiscal year.<br>$sys_admin";
  CleanUp();
}
$start_fiscal = $result->getvalue(0, 1);
$stop_fiscal = $result->getvalue(0, 2);
($start_month_fiscal, $start_day_fiscal, $start_year_fiscal) = split "-", $start_fiscal;
($stop_month_fiscal, $stop_day_fiscal, $stop_year_fiscal) = split "-", $stop_fiscal;

# get the current balance for the account from the database that is less than the from_search_date; if there is no balance less than the
# from_search_date, start with an initial balance of 0
# if account is investment get the -1 balance
my $sub_number;
if ($account_type == 3)
{
  $sub_number = "-1";
}
else
{
  $sub_number = "-2";
}
my $sqlstring;
$sqlstring = "select balance, bal_date from acct_bal_tb where ledger_code = '$ledger_code' AND acct_num = $account_num and sub_acct_num = '$sub_number' and bal_date = (select max(bal_date) from acct_bal_tb where ledger_code = '$ledger_code' AND bal_date < '$from_search_date' and acct_num = $account_num and sub_acct_num = '$sub_number')";
$result = $conn->exec($sqlstring);
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n<br>$sqlstring<br>\n";
  print $conn->errorMessage . "<br>Tried to get the balance for the account $account_num.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples > 1)
{
  PrintErrorHeader();
  print "<h1>Error: Wrong number of records returned!</h1>\n";
  print "Tried to get the balance and date of the most recent balance less than the from search date.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  $balance_date = $start_known;
  ($temp_month, $temp_day, $temp_year) = split "-", $balance_date;
  $reverse_balance = $temp_year . $temp_month . $temp_day;
  $balance_so_far = 0;
}
else
{
  $balance_so_far = $result->getvalue(0, 0);
  $balance_date = $result->getvalue(0, 1);
  ($temp_month, $temp_day, $temp_year) = split "-", $balance_date;
  $reverse_balance = $temp_year . $temp_month . $temp_day;
}
# get the beginning_balance_date usually 1 day before the from_search_date
if ($reverse_start_known < $reverse_from)
{
  $beginning_balance_date = $from_search_date;
  $beginning_balance_date =~ s/\-/\//g;
  $beginning_balance_date = `/bin/date -d '$beginning_balance_date 1 day ago' +%m-%d-%Y`;
  chomp $beginning_balance_date;
}
elsif ($reverse_start_known == $reverse_from)
{
  $beginning_balance_date = $from_search_date;
}
# calculate the correct balance up to the from_search_date
if ($reverse_balance < $reverse_from)
{
  # select all records from the balance_date to the from_search_date with dorc == true
  my $sql_string = "select sum(db.amount) from debitcredit_tb as db, journal_entry_tb as je where (je.trans_date ";
  if ($balance_date eq $start_known)
  {
    $sql_string .= ">=";
  }
  else
  {
    $sql_string .= ">";
  }
  $sql_string .= " '$balance_date' and je.trans_date < '$from_search_date') and je.ref_num = db.parent_num and db.dorc = true and db.account_num = $account_num AND je.ledger_code = '$ledger_code' AND db.ledger_code = '$ledger_code'";
  if ($account_type == 3)
  {
    $sql_string .= " AND db.sub_account_num = '-1'";
  }
  $result = $conn->exec($sql_string);
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the sum of debits for account $account_num.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples != 1)
  {
    PrintErrorHeader();
    print "<h1>Error: Wrong number of records returned!</h1>\n";
    print $result->ntuples . " records were returned. There should be only one.<br>$sys_admin";
    CleanUp();
  }
  $sum_debits = $result->getvalue(0, 0);
  # select all records from the balance_date to the from_search_date with dorc == false
  $sql_string = "select sum(debitcredit_tb.amount) from debitcredit_tb, journal_entry_tb where (journal_entry_tb.trans_date ";
  if ($balance_date eq $start_known)
  {
    $sql_string .= ">=";
  }
  else
  {
    $sql_string .= ">";
  }
  $sql_string .= " '$balance_date' and journal_entry_tb.trans_date < '$from_search_date') and journal_entry_tb.ref_num = debitcredit_tb.parent_num and debitcredit_tb.dorc = false and debitcredit_tb.account_num = $account_num AND journal_entry_tb.ledger_code = '$ledger_code' AND debitcredit_tb.ledger_code = '$ledger_code'";
  if ($account_type == 3)
  {
    $sql_string .= " AND debitcredit_tb.sub_account_num = '-1'";
  }
  $result = $conn->exec($sql_string);
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error: Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get the sum of credits for account $account_num.<br>$sys_admin";
    CleanUp();
  }
  if ($result->ntuples != 1)
  {
    PrintErrorHeader();
    print "<h1>Error: Wrong number of records returned!</h1>\n";
    print $result->ntuples . " records were returned. There should be only one.<br>$sys_admin";
    CleanUp();
  }
  $sum_credits = $result->getvalue(0, 0);

  if ($debit_increases eq "t")
  {
    $balance_so_far += $sum_debits;
    $balance_so_far -= $sum_credits;
  }
  else
  {
    $balance_so_far -= $sum_debits;
    $balance_so_far += $sum_credits;
  }
}

# remove the sum of the debits/credits for outstanding transactions from the balance to be displayed
if (exists $input{'display_outstanding'})
{
  if ($debit_increases eq "t")
  {
    $balance_so_far -= $sum_outstanding_debits;
    $balance_so_far += $sum_outstanding_credits;
  }
  else
  {
    $balance_so_far += $sum_outstanding_debits;
    $balance_so_far -= $sum_outstanding_credits;
  }
}

print "<head><title>$prog_name $prog_version Account Registers</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }

td { font-size: xx-small; } 

-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
<script language=\"Javascript1.2\">
  <!--

  function go_here2(n)
  {
    // set the form to go to the journal_mod1 script
    document.register_display_form.action = '$site_dir/cgi-bin/journal_mod1.cgi';
    document.register_display_form.ref_num2.value = n;
    window.open('','edit_window','resizable,status,scrollbars');
    document.register_display_form.target = 'edit_window';
    document.register_display_form.submit();
    return false;
  }

  function cycle_button(button, cleared)
  {
    if (button.value == "_")
    {
      button.value = "x";
      cleared.value = "x";
    }
    else if (button.value == "x")
    {
      button.value = "r";
      cleared.value = "r";
    }
    else if (button.value == "r")
    {
      button.value = "_";
      cleared.value = " ";
    }
    else
    {
      button.value = "_";
      cleared.value = " ";
    }
  }

  function reset_click()
  {
END_OF_STYLE
for ($count=0; $count < @o_cleared_array; $count++)
{
  print "    if (document.register_display_form.cleared_button_$count.value != null)\n";
  print "    {\n";
  print "      document.register_display_form.cleared_button_$count.value = \"";
  if ($o_cleared_array[$count] eq " ")
  {
    print "_";
  }
  else
  {
    print "$o_cleared_array[$count]";
  }
  print "\";\n";
  print "      document.register_display_form.cleared_$count.value = \"$o_cleared_array[$count]\";\n";
  print "    }\n";
}
$count_temp = $count;
for ($count=0, $count2=$count_temp; $count < @cleared_array; $count++, $count2++)
{
  print "    if (document.register_display_form.cleared_button_$count2.value != null)\n";
  print "    {\n";
  print "      document.register_display_form.cleared_button_$count2.value = \"";
  if ($cleared_array[$count] eq " ")
  {
    print "_";
  }
  else
  {
    print "$cleared_array[$count]";
  }
  print "\";\n";
  print "      document.register_display_form.cleared_$count2.value = \"$cleared_array[$count]\";\n";
  print "    }\n";
}
print <<"END_OF_STYLE";
  }

  //-->
</script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td align=left width=47%><font color=\"red\">";
print "Displaying Account Register for <font color=\"blue\">$account_name</font> in Ledger <font color=\"$ledger_color\">$ledger_name</font>. &nbsp;FY is <font color=\"black\">$current_fiscal_year</font>.";
print "</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#register\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";

print "<h3>Register for <font color=\"$menu_bgcolor\">$account_name - $account_num</font></h3>\n";
print "<h4>From: <font color=\"blue\">$from_search_date</font>, To: <font color=\"blue\">$to_search_date</font></h4><br>\n";

print "<form method=post name=\"register_display_form\" action=\"\">\n";
print "<input type=hidden name=\"ledger_db\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"account_name\" value=\"$account_name\">\n";
print "<input type=hidden name=\"account_num\" value=\"$account_num\">\n";
print "<input type=hidden name=\"account_type\" value=\"$account_type\">\n";
if (exists $input{'display_outstanding'})
{
  print "<input type=hidden name=\"display_outstanding\" value=\"\">\n";
}
print "<input type=hidden name=\"ref_num2\" value=\"\">\n";
print "<input type=hidden name=\"journal_view_command\" value=\"search2\">\n";
print "<input type=hidden name=\"num_entries_to_display\" value=\"10\">\n";
print "<input type=hidden name=\"start_index\" value=\"1\">\n";
print "<input type=hidden name=\"num_rows\" value=\"" . (@ref_num_array + @o_ref_num_array) . "\">\n";
print "<input type=hidden name=\"debit_increases\" value=\"$debit_increases\">\n";
print "<input type=hidden name=\"beginning_date\" value=\"$beginning_balance_date\">\n";
print "<input type=hidden name=\"ending_date\" value=\"$ending_balance_date\">\n";
print "<input type=hidden name=\"from_search_date\" value=\"$from_search_date\">\n";
print "<input type=hidden name=\"to_search_date\" value=\"$to_search_date\">\n";
if ($account_type == 0)
{
  print "<input type=hidden name=\"bank_account\" value=\"\">\n";
}

if ((@ref_num_array != 0 || @o_ref_num_array != 0) && exists $input{'display_outstanding'})
{
  print "<table border=0 width=\"100%\" cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
  print "<tr>\n";
  print "<td align=left><input type=submit name=\"update_register_submit\" value=\"Mark Entries\" onClick=\"submit_my_form(document.register_display_form, '$site_dir/cgi-bin/register_update.cgi');\"></td>\n";
  print "<td align=right><input type=button name=\"reset_button_1\" value=\"Reset\" onClick=\"reset_click();\"></td>\n";
  print "</tr>\n";
  print "</table>\n";
}

# set count to zero so that the next loop that uses it will have it set to zero just in case it does not have a non-zero value and the
# displaying of outstanding transactions is turned off
$count = 0;
if (@o_ref_num_array != 0)
{
  # display outstanding transactions
  print "<table border=0 width=100% cellspacing=0 cellpadding=1 bgcolor=\"$body_bgcolor\">\n";
  print "<tr><td align=left width=50%><b>Outstanding Transactions</b></td><td align=right width=50%><b>". int (scalar @o_ref_num_array) . "</b> Transactions Returned</td></tr>\n";
  print "</table>\n";
  print "<table border=1 width=\"100%\" cellspacing=0 cellpadding=1 bgcolor=\"$body_bgcolor\">\n";
  print "<tr bgcolor=\"$menu_bgcolor\">\n";
  print "<th width=\"5%\">Ref#</th><th width=\"8%\">Date</th><th align=\"left\">Description</th><th>Memo</th>";
  if ($account_type == 0)
  {
    print "<th width=\"1%\">Chck#</th><th width=\"1%\">Dpst#</th>";
  }
  else
  {
    print "<th>Sub-Acct</th>";
  }
  print "<th width=\"1%\">Cld</th><th width=\"7%\">Debit</th><th width=\"7%\">Credit</th>\n";
  print "</tr>\n";
  for ($count=0; $count < @o_ref_num_array; $count++)
  {
    print "<input type=hidden name=\"row_ref_num_$count\" value=\"$o_ref_num_array[$count]\">\n";
    print "<tr align=center>\n";
    print "<td><a href=\"\" onClick=\"return go_here2($o_ref_num_array[$count]);\">$o_ref_num_array[$count]</a></td>\n";
    print "<td>$o_trans_date_array[$count]<input type=hidden name=\"trans_date_$count\" value=\"$o_trans_date_array[$count]\"></td>\n";
    if ($o_payee_payer_array[$count] eq "")
    {
      print "<td><br></td>\n";
    }
    else
    {
      print "<td align=\"left\">$o_payee_payer_array[$count]</td>\n";
    }
    print "<input type=hidden name=\"payee_payer_$count\" value=\"$o_payee_payer_array[$count]\">\n";
    if (substr($o_memo_array[$count], 0, 15) eq "")
    {
      print "<td><br></td>\n";
    }
    else
    {
      print "<td align=\"left\">" . substr($o_memo_array[$count], 0, 15) . "</td>\n";
    }
    if ($account_type == 0 || $account_type == 10)
    {
      if ($o_dorc_array[$count] eq "t" && $o_sub_account_array[$count] ne "-1")
      {
        print "<td><br></td>\n";
        print "<td>$o_sub_account_array[$count]</td>\n";
        print "<input type=hidden name=\"check_deposit_$count\" value=\"$o_sub_account_array[$count]\">\n";
      }
      elsif ($o_dorc_array[$count] ne "t" && $o_sub_account_array[$count] ne "-1")
      {
        print "<td>$o_sub_account_array[$count]</td>\n";
        print "<td><br></td>\n";
        print "<input type=hidden name=\"check_deposit_$count\" value=\"$o_sub_account_array[$count]\">\n";
      }
      else
      {
        print "<td><br></td>\n";
        print "<td><br></td>\n";
        print "<input type=hidden name=\"check_deposit_$count\" value=\"<br>\">\n";
      }
    }
    else
    {
      if ($o_sub_account_array[$count] ne "-1")
      {
        print "<td>$o_sub_account_name_array[$count] - $o_sub_account_array[$count]</td>\n";
      }
      else
      {
        print "<td><br></td>\n";
      }
    }
    print "<td><input type=button name=\"cleared_button_$count\" value=\"";
    if ($o_cleared_array[$count] eq " ")
    {
      print "_";
    }
    else
    {
      print "$o_cleared_array[$count]";
    }
    print "\" onClick=\"cycle_button(document.register_display_form.cleared_button_$count, document.register_display_form.cleared_$count);\">";
    if ($o_dorc_array[$count] eq "t")
    {
      print "<td align=right>" . commify(sprintf("%.2f", $o_amount_array[$count])) . "</td>\n";
      print "<td><br></td>\n";
    }
    else
    {
      print "<td><br></td>\n";
      print "<td align=right>" . commify(sprintf("%.2f", $o_amount_array[$count])) . "</td>\n";
    }
    print "<input type=hidden name=\"amount_$count\" value=\"$o_amount_array[$count]\">\n";
    print "<input type=hidden name=\"dorc_$count\" value=\"$o_dorc_array[$count]\">\n";
    print "<input type=hidden name=\"cleared_$count\" value=\"$o_cleared_array[$count]\"><input type=hidden name=\"counter_$count\" value=\"$o_counter_array[$count]\"></td>\n";
    print "</tr>\n";
  }
  print "</table>\n";
  print "<br><br>\n";
}
print "<div align=right><b>" . int (scalar @ref_num_array) . "</b> Transactions Returned</div>\n";
print "<table border=1 width=\"100%\" cellspacing=0 cellpadding=1 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
if ($account_type == 0)
{
  print "<th colspan=7 align=right>";
}
else
{
  print "<th colspan=6 align=right>";
}
print "Beginning Balance</th>\n";
print "<td align=center colspan=2>$beginning_balance_date</td>\n";
print "<td align=right>";
$balance_so_far = sprintf "%.2f", $balance_so_far;
print "<input type=hidden name=\"beginning_balance\" value=\"$balance_so_far\">\n";
if ($balance_so_far < 0)
{
  printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($balance_so_far)));
}
else
{
  printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($balance_so_far)));
}
print "</td>\n";
print "</tr>\n";
print "<tr bgcolor=\"$menu_bgcolor\">\n";
print "<th width=\"5%\">Ref#</th><th width=\"8%\">Date</th><th align=\"left\">Description</th><th>Memo</th>";
if ($account_type == 0)
{
  print "<th width=\"1%\">Chck#</th><th width=\"1%\">Dpst#</th>";
}
elsif ($account_type == 10)
{
}
else
{
  print "<th>Sub-Acct</th>";
}
print "<th width=\"1%\">Cld</th><th width=\"7%\">Debit</th><th width=\"7%\">Credit</th><th width=\"10%\">Balance</th>\n";
print "</tr>\n";
$count_temp = $count;
for ($count=0, $count2=$count_temp; $count<@ref_num_array; $count++, $count2++)
{
  print "<input type=hidden name=\"row_ref_num_$count2\" value=\"$ref_num_array[$count]\">\n";
  print "<tr align=center>\n";
  print "<td><a href=\"\" onClick=\"return go_here2($ref_num_array[$count]);\">$ref_num_array[$count]</a></td>\n";
  print "<td>$trans_date_array[$count]<input type=hidden name=\"trans_date_$count2\" value=\"$trans_date_array[$count]\"></td>\n";
  if ($payee_payer_array[$count] eq "")
  {
    print "<td><br></td>\n";
  }
  else
  {
    print "<td align=\"left\">$payee_payer_array[$count]</td>\n";
  }
  print "<input type=hidden name=\"payee_payer_$count2\" value=\"$payee_payer_array[$count]\">\n";
  if (substr($memo_array[$count], 0, 15) eq "")
  {
    print "<td><br></td>\n";
  }
  else
  {
    print "<td align=\"left\">" . substr($memo_array[$count], 0, 15) . "</td>\n";
  }
  if ($account_type == 0 || $account_type == 10)
  {
    if ($dorc_array[$count] eq "t" && $sub_account_array[$count] ne "-1")
    {
      print "<td><br></td>\n";
      print "<td>$sub_account_array[$count]</td>\n";
      print "<input type=hidden name=\"check_deposit_$count2\" value=\"$sub_account_array[$count]\">\n";
    }
    elsif ($dorc_array[$count] ne "t" && $sub_account_array[$count] ne "-1")
    {
      print "<td>$sub_account_array[$count]</td>\n";
      print "<td><br></td>\n";
      print "<input type=hidden name=\"check_deposit_$count2\" value=\"$sub_account_array[$count]\">\n";
    }
    else
    {
      print "<td><br></td>\n";
      print "<td><br></td>\n";
      print "<input type=hidden name=\"check_deposit_$count2\" value=\"<br>\">\n";
    }
  }
  else
  {
    if ($sub_account_array[$count] ne "-1")
    {
      print "<td>$sub_account_name_array[$count] - $sub_account_array[$count]</td>\n";
    }
    else
    {
      print "<td><br></td>\n";
    }
  }
  if (exists $input{'display_outstanding'})
  {
    print "<td><input type=button name=\"cleared_button_$count2\" value=\"";
    if ($cleared_array[$count] eq " ")
    {
      print "_";
    }
    else
    {
      print "$cleared_array[$count]";
    }
    print "\" onClick=\"cycle_button(document.register_display_form.cleared_button_$count2, document.register_display_form.cleared_$count2);\">";
  }
  else
  {
    if ($cleared_array[$count] eq " ")
    {
      print "<td>&nbsp;</td>\n";
    }
    else
    {
      print "<td>$cleared_array[$count]</td>\n";
    }
  }
  if ($dorc_array[$count] eq "t")
  {
    print "<td align=right>" . commify(sprintf("%.2f", $amount_array[$count])) . "</td>\n";
    print "<td><br></td>\n";
  }
  else
  {
    print "<td><br></td>\n";
    print "<td align=right>" . commify(sprintf("%.2f", $amount_array[$count])) . "</td>\n";
  }
  print "<input type=hidden name=\"amount_$count2\" value=\"$amount_array[$count]\">\n";
  print "<input type=hidden name=\"dorc_$count2\" value=\"$dorc_array[$count]\">\n";
  print "<input type=hidden name=\"cleared_$count2\" value=\"$cleared_array[$count]\"><input type=hidden name=\"counter_$count2\" value=\"$counter_array[$count]\"></td>\n";
  $temp_amount = $amount_array[$count];
  $temp_amount =~ s/\$//g;
  $temp_amount =~ s/\,//g;
  if ($debit_increases eq "t" && $dorc_array[$count] eq "t")
  {
    $balance_so_far += $temp_amount;
  }
  elsif ($debit_increases eq "t" && $dorc_array[$count] ne "t")
  {
    $balance_so_far -= $temp_amount;
  }
  elsif ($debit_increases ne "t" && $dorc_array[$count] eq "t")
  {
    $balance_so_far -= $temp_amount;
  }
  elsif ($debit_increases ne "t" && $dorc_array[$count] ne "t")
  {
    $balance_so_far += $temp_amount;
  }
  print "<td align=right>";
  if ($balance_so_far < 0)
  {
    printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($balance_so_far)));
  }
  else
  {
    printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($balance_so_far)));
  }
  print "</td>\n";
  print "</tr>\n";
}
if ($account_type == 0)
{
  print "<th colspan=7 align=right>";
}
else
{
  print "<th colspan=6 align=right>";
}
print "Ending Balance</th>\n";
print "<td align=center colspan=2>$ending_balance_date</td>\n";
print "<td align=right>";
if ($balance_so_far < 0)
{
  printf "<font color=red>(%s)</font>", commify(sprintf("%.2f", abs($balance_so_far)));
}
else
{
  printf "%s&nbsp;&nbsp;", commify(sprintf("%.2f", abs($balance_so_far)));
}
print "</td>\n";
print "</tr>\n";
print "</table>\n";

if ((@ref_num_array != 0 || @o_ref_num_array != 0) && exists $input{'display_outstanding'})
{
  print "<table border=0 width=\"100%\" cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
  print "<tr>\n";
  print "<td align=left><input type=submit name=\"update_register_submit\" value=\"Mark Entries\" onClick=\"submit_my_form(document.register_display_form, '$site_dir/cgi-bin/register_update.cgi');\"></td>\n";
  print "<td align=right><input type=button name=\"reset_button_2\" value=\"Reset\" onClick=\"reset_click();\"></td>\n";
  print "</tr>\n";
  print "</table>\n";
}
print "</form>\n";

# print a horizontal rule
#print "<br><hr>\n";
print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
