#!/usr/bin/perl
#$Id: reopen_fiscal_year2.cgi,v 1.2 2001/10/22 23:24:44 pcxuser Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# reopen_fiscal_year2.cgi  -  Created by James Pattie, 01/05/2000
# This script takes the selected fiscal year and goes about re-opening it and making it the current fiscal year.
use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

my $ledger_name;
my $ledger_db;
my $ledger_color;
my $result2;
my $period;
my $report_obj;
my $ledger_code;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  A Ledger must be specified!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

$period = $input{'period'};
if (length $period == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must select a Fiscal Year/Period to work with!</h1>\n";
  print $back_button;
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# now we check and validate the ledger is valid
$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "Tried to validate Ledger <font color=\"blue\">$ledger_db</font>.<br>$sys_admin";
  CleanUp();
}
$ledger_name = $result->getvalue(0, 0);

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

# figure out the beginning and ending date of the year in question
my ($working_year, $from_date, $to_date, $month, $day, $year);

if ($period =~ /(-|\/)/)
{
  ($month, $day, $year) = split (/[-\/]/g, $period);
}
else
{
  $year = $period;
}

$result = $conn->exec("select year, start_date, end_date from known_years_tb where year = $year AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get year and fiscal year dates from ledger <font color=\"$ledger_color\">$ledger_db</font> for Fiscal Year = $year.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Program not configured!</h1>\n";
  print "Fiscal Year <font color=\"black\">$year</font> does not exist!<br>$sys_admin";
  CleanUp();
}
$working_year = $result->getvalue(0, 0);
$from_date    = $result->getvalue(0, 1);
$to_date      = $result->getvalue(0, 2);

if ($period =~ /(-|\/)/)
{
  $to_date = $period;

  # get a list of all closeout dates that are within the current fiscal period.
  my $result2 = $conn->exec("SELECT DISTINCT(bal_date) FROM acct_bal_tb WHERE ledger_code = '$ledger_code' AND ledger_code = known_years_tb.ledger_code AND known_years_tb.ledger_code = variables_tb.ledger_code AND known_years_tb.year = variables_tb.working_year AND bal_date >= known_years_tb.start_date AND bal_date < known_years_tb.end_date ORDER BY bal_date DESC");
  if ($result2->resultStatus != PGRES_TUPLES_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to get list of all Close Out Periods in the current fiscal year.<br>$sys_admin";
    CleanUp();
  }
  for (my $j=0; $j < $result2->ntuples; $j++)
  {
    if ($result2->getvalue($j, 0) eq $period)
    {
      if ($j < $result2->ntuples - 1)  # This means that there was another CloseOut period before us.  So use it as the start date instead of the Fiscal Year's Start Date.
      {
        $from_date = $result2->getvalue($j+1, 0);
      }
    }
  }
}

print "<head><title>$prog_name $prog_version Re-Open Fiscal Year/Period $period - Almost Done!</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

my ($i, $name, $code);

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%><font color=red>Working in ledger <font color=\"$ledger_color\">$ledger_name</font>.  Re-Opening Fiscal Year/Period <font color=\"black\">$period</font> (<font color=\"black\">$from_date</font> to <font color=\"black\">$to_date</font>).</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#reopen_fiscal_year\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);

# start a transaction with the ledger
$result = $conn->exec("BEGIN WORK");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  print "<h1>Error:  Starting Ledger Transaction failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to BEGIN WORK.<br>$sys_admin";
  CleanUp();
}

# remove any balance table entries that exist for the last day of this fiscal year.
$result = $conn->exec("DELETE FROM acct_bal_tb WHERE bal_date >= '$to_date' AND ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  print "<h1>Error:  Deleting Balance Table Entries failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to delete any existing Balance Table Entries in Fiscal Year = $working_year.<br>$sys_admin";
  CleanUp();
}

$result2 = $conn->exec("UPDATE known_years_tb SET closed = false WHERE year >= $working_year AND ledger_code = '$ledger_code'");
if ($result2->resultStatus != PGRES_COMMAND_OK)
{
  print "<h1>Error:  Opening fiscal year $working_year failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to update the fiscal year record.  Marking all years >= '$working_year' as not being closed.<br>$sys_admin";
  CleanUp();
}

# update the variables_tb to reflect the new fiscal year
$result2 = $conn->exec("UPDATE variables_tb SET working_year = $working_year WHERE ledger_code = '$ledger_code'");
if ($result2->resultStatus != PGRES_COMMAND_OK)
{
  print "<h1>Error:  Changing the working year failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to update the variables table.<br>$sys_admin";
  CleanUp();
}

# commit the transaction
$result = $conn->exec("COMMIT WORK");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  print "<h1>Error:  Ending Ledger Transaction failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to COMMIT WORK.<br>$sys_admin";
  CleanUp();
}

print "<h1>Fiscal Year/Period <font color=\"red\">$period</font> Successfully Opened!<br></h1>\n";
print "<br>You will be returned to the <b>Ledger Summary</b> in <b><i>5</i></b> seconds.<br>\n";
print "<form name=\"success_form\" method=post action=\"$site_dir/cgi-bin/ledger.cgi\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<input type=hidden name=\"open\" value=\"\">\n";
print "</form>\n";
print "<script Language=\"JavaScript1.2\">\n";
print "function SubmitForm()\n";
print "{\n";
print "  document.success_form.submit();\n";
print "}\n";
print "setTimeout('SubmitForm();', 5000);\n";
print "</script>\n";

print "</center>\n";

print "</body>\n";
print "</html>\n";
exit 0;
