#!/usr/bin/perl
#$Id: reports2.cgi,v 1.1 2001/09/06 10:21:20 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# reports2.cgi  -  Created by James Pattie, 07/20/1999
# This script presents the user with the sub list of reports
# based upon the main report they chose.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';
use vars '$ledger_db';
use vars '$ledger_name';
use vars '$conn2';
use vars '$report_group';

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

if (not exists $input{'open'})
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Access!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

$report_group = $input{'report_group'};
if (length $report_group == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must select a report group to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}

if (not ($report_group eq "bookkeeping" || $report_group eq "payroll" || $report_group eq "asset" || $report_group eq "investment" || $report_group eq "analysis"))
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Report Group specified!</h1>\n";
  print "Value specified was '$report_group'.<br>$sys_admin";
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print "Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

print "<head><title>$prog_name $prog_version Reporting Main Menu</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%>Working with Ledger <font color=\"blue\">$ledger_name</font></td>\n";
print "<td align=left><a href=\"$site_dir/help.html#reporting\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Reporting Main Menu</h2>\n";

my $group_name;
if ($report_group eq "bookkeeping")
{
  $group_name = "Bookkeeping";
}
elsif ($report_group eq "payroll")
{
  $group_name = "Payroll";
}
elsif ($report_group eq "asset")
{
  $group_name = "Asset";
}
elsif ($report_group eq "investment")
{
  $group_name = "Investment";
}
elsif ($report_group eq "analysis")
{
  $group_name = "Analysis";
}

# create the form that lets them select from the group reports available.
my $temp;
$temp = $result->ntuples;
print <<"END_OF_FORM";
<form method=post action=\"$site_dir/cgi-bin/reports3.cgi\">
<input type=hidden name=\"ledger\" value=\"$ledger_db\">
<input type=hidden name=\"report_group\" value=\"$report_group\">
<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">
  <tr>
    <td align=right><font color=\"$menu_fgcolor\">Reports for $group_name</font></td>
    <td><select name=\"report\">
END_OF_FORM

if ($report_group eq "bookkeeping")
{
  print <<"END_OF_FORM";
    <option value=\"income_statement\">Income Statement
    <option value=\"cash_flow\">Cash Flow
    <option value=\"balance_sheet\">Balance Sheet
    <option value=\"transaction\">Transaction
    <option value=\"missing_checks\">Missing Checks
    <option value=\"comparison_reports\">Comparison Reports
    <option value=\"bank_reconciliation\">Bank Reconciliation
    <option value=\"consolidated_statements\">Consolidated Statements
    <option value=\"taxes_paid\">Taxes paid
END_OF_FORM
}
elsif ($report_group eq "payroll")
{
  print <<"END_OF_FORM";
    <option value=\"fica_and_medicare\">FICA and Medicare
    <option value=\"payroll_record\">Payroll Record
    <option value=\"941_quarterly_payroll\">941 Quarterly Payroll
END_OF_FORM
}
elsif ($report_group eq "asset")
{
  print <<"END_OF_FORM";
    <option value=\"asset_valuation\">Asset Valuation
    <option value=\"depreciation\">Depreciation
END_OF_FORM
}
elsif ($report_group eq "investment")
{
  print <<"END_OF_FORM";
    <option value=\"1099_reconciliation\">1099 Reconciliation
    <option value=\"investment_performance\">Investment Performance
    <option value=\"capital_gains\">Capital Gains
    <option value=\"investment_transactions\">Investment Transactions
    <option value=\"price_comparison\">Price Comparison
    <option value=\"portfolio_value\">Portfolio Value
END_OF_FORM
}
elsif ($report_group eq "analysis")
{
  print <<"END_OF_FORM";
    <option value=\"quick_ratio\">Quick Ratio
    <option value=\"current_ratio\">Current Ratio
    <option value=\"inventory_turnover\">Inventory Turnover
    <option value=\"accounts_receivable_turnover\">Accounts Receivable Turnover
    <option value=\"days_sales_in_receivables\">Day's Sales in Receivables
    <option value=\"debt_ratio\">Debt Ratio
    <option value=\"times_interest_earned_ratio\">Times-Interest-Earned Ratio
    <option value=\"return_on_net_sales\">Return on Net Sales
    <option value=\"return_on_total_assets\">Return on Total Assets
    <option value=\"return_on_common_stockholders_equity\">Return on Common Stockholder's Equity
    <option value=\"eps_common_stock\">EPS Common Stock
    <option value=\"pe\">PE
    <option value=\"dividend_yield\">Dividend Yield
    <option value=\"book_value\">Book Value
END_OF_FORM
}

print <<"END_OF_FORM";
    </select>
  </tr>
</table>
<input type=submit name=\"customize\" value=\"Customize Report\">
</form>
END_OF_FORM

# print a horizontal rule to break the menu from the behind the scenes links
#print "<br><hr>\n";

print "</body>\n";
print "</html>\n";
exit 0;