#!/usr/bin/perl
#$Id: sub_entities2.cgi,v 1.1 2001/09/06 10:21:20 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# sub_entities2.cgi  -  Created by James Pattie, 07/20/1999
# This script creates, edits or deletes sub entities.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features
use reports;

use vars '$date';
use vars '$ledger_db';
use vars '$ledger_name';
use vars '$working_mode';
use vars '$entry_code';
use vars '$entry_name';
my $error_string;  # used to store the result of IsValidString();
my $onLoad;
$onLoad = "";  # used to signal when an onLoad= function needs to be called.
my $ledger_code;
my $report_obj;
my $ledger_color;
my %commands;
%commands = ( "create" => "create", "edit" => "edit", "delete" => "delete" );

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

if (not exists $input{'sub_entity_command'})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "You must specify the command to work with.<br>$sys_admin";
  CleanUp();
}

if (not exists $commands{$input{'sub_entity_command'}})
{
  PrintErrorHeader();
  print "<h1>Error:  Access Denied!</h1>\n";
  print "You must specify a mode to work in.<br>$sys_admin";
  CleanUp();
}

if ($input{'sub_entity_command'} eq "create")
{
  $working_mode = 1;
}
elsif ($input{'sub_entity_command'} eq "edit")
{
  $working_mode = 2;
}
elsif ($input{'sub_entity_command'} eq "delete")
{
  $working_mode = 3;
}

$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error: You must specify a ledger to work with!</h1>\n";
  print "$back_button";
  CleanUp();
}
$error_string = IsValidString($ledger_db);
if (length $error_string != 0)
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Ledger Database Name!</h1>\n";
  print "<b>$error_string</b> is invalid.<br>$back_button";
  CleanUp();
}

# if we are editing, make sure we only have 1 value.
if ($working_mode == 2)
{
  if (not exists $input{'sub_entity'})
  {
    PrintErrorHeader();
    print "<h1>Error:  You must select an entry for Editing!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  # get the entries info
  my $temp = "sub_entity";
  $temp = $input{$temp};
  ($entry_code, $entry_name) = split /\|/, $temp;
  if (length $entry_code == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  The entry code must be specified!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  $error_string = IsValidString($entry_code);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid SubEntity Code!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if (length $entry_name == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  The entry name must be specified!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  $error_string = IsValidString($entry_name);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid SubEntity Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
}

# if we are deleting make sure we have at least 1 value and <= num_entries.
if ($working_mode == 3)
{
  if (not exists $input{'sub_entity'})
  {
    PrintErrorHeader();
    print "<h1>Error:  You must select an entry to Delete!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  # get the entries info
  my $temp = "sub_entity";
  $temp = $input{$temp};
  ($entry_code, $entry_name) = split /\|/, $temp;
  if (length $entry_code == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  The entry code must be specified!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  $error_string = IsValidString($entry_code);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid SubEntity Code!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
  if (length $entry_name == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  The entry name must be specified!</h1>\n";
    print "$sys_admin";
    CleanUp();
  }
  $error_string = IsValidString($entry_name);
  if (length $error_string != 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Invalid SubEntity Name!</h1>\n";
    print "<b>$error_string</b> is invalid.<br>$back_button";
    CleanUp();
  }
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

$result = $conn->exec("select name from entity_tb where database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error: Accessing database failed!</h1>\n";
  print "Trying to check entity table.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  PrintErrorHeader();
  print "<h1>Error: Ledger <font color=\"blue\">$ledger_db</font> does not exist!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
else
{
  $ledger_name = $result->getvalue(0, 0);
}

# create the reports object
$report_obj = reports->new($conn, $ledger_db);
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_code = $report_obj->getLedgerCode();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}
$ledger_color = $report_obj->getLedgerColor();
if ($report_obj->didErrorOccur())
{
  PrintErrorHeader();
  print $report_obj->getErrorMessage();
  CleanUp();
}

print "<head><title>$prog_name $prog_version Sub Entity Administration</title>\n";
my $Error_Occurred;
$Error_Occurred = 0;  # lets me know when an error was encountered that the timeout needs to be longer because of.

if ($working_mode == 3)  # print the refresh header
{
  print <<"END_OF_FORM";
  <script language="Javascript1.2">
  <!--
    var ErrorOccurred = 0;
    function SubmitForm()
    {
      document.menu_bar2.submit();
    }

    function doTimer()
    {
      if (ErrorOccurred)
      {
        setTimeout("SubmitForm()", 5000);  // in 5 seconds
      }
      else
      {
        setTimeout("SubmitForm()", 3000);  // in 3 seconds
      }
    }
  //-->
  </script>
END_OF_FORM
$onLoad = " onLoad=doTimer();";
}
else
{
  print <<"END_OF_SCRIPT";
  <script language="JavaScript1.2">
  function ValidateSubEntityForm()
  {
    var msg = "";
    var temp = document.sube_form.name.value;
    if (temp.length == 0)
    {
      msg += "You must specify the Sub Entity Name!\\n";
    }
    temp = document.sube_form.code.value;
    if (temp.length == 0)
    {
      msg += "You must specify the Sub Entity Code!\\n";
    }
    if (msg.length > 0)
    {
      alert(msg);
      return false;
    }
    return true;
  }
  </script>
END_OF_SCRIPT
}

print "</head>\n";
print "<body bgcolor=$body_bgcolor$onLoad>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: red }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%>Working with Ledger <font color=\"$ledger_color\">$ledger_name</font>.&nbsp;&nbsp;";
if ($working_mode == 1)
{
  print "Creating Sub Entity";
}
elsif ($working_mode == 2)
{
  print "Editing Sub Entity <font color=\"red\">$entry_name</font>";
}
elsif ($working_mode == 3)
{
  print "Deleting Sub Entity <font color=\"red\">$entry_name</font>";
}
print "</td>\n";
print "<td align=left><a href=\"$site_dir/help.html#sub_entities\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardLedgerMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Sub Entity Administration</h2>\n";

if ($working_mode == 1 || $working_mode == 2)
{
  # create the form that lets them select from the sub entities to work with.
  print <<"END_OF_FORM";
  <form method=post action=\"$site_dir/cgi-bin/sub_entities3.cgi\" name=\"sube_form\" onSubmit=\"return ValidateSubEntityForm();\">
  <input type=hidden name=\"ledger\" value=\"$ledger_db\">
  <table border=1 cellspacing=1 cellpadding=2 bgcolor=\"$menu_bgcolor\" width=50%>
  <tr><th>Name</th><th>Code</th></tr>
END_OF_FORM
}

if ($working_mode == 1)
{ # create the entries for entering the name and code.
  print "<tr><td align=center><input type=text name=\"name\"></td>\n";
  print "<td align=center><input type=text name=\"code\"></td></tr>\n";
}
elsif ($working_mode == 2)
{
  print "<tr><td align=center><input type=hidden name=\"old_name\" value=\"$entry_name\"><input type=text name=\"name\" value=\"$entry_name\"></td>\n";
  print "<td align=center><input type=hidden name=\"code\" value=\"$entry_code\"><font color=\"$menu_fgcolor\">$entry_code</font></td></tr>\n";
}
elsif ($working_mode == 3)
{
  # start a transaction
  $result = $conn->exec("begin work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Starting Transaction failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to start transaction with Ledger <font color=\"blue\">$ledger_name</font>.<br>$sys_admin";
    CleanUp();
  }

  # first check and make sure that this sub-entity is not in use.
  $result = $conn->exec("SELECT sub_e_code FROM debitcredit_tb WHERE sub_e_code = '$entry_code' AND ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_TUPLES_OK)
  {
    print "<h1>Error:  Accessing database failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to see if the Sub-Entity is being used.<br>$sys_admin";
    CleanUp();
  }

  if ($result->ntuples > 0)
  {
    print "Sub Entity, &nbsp;Name = '$entry_name', Code = '$entry_code' is <b>in Use!</b>  You must delete the transactions to delete this sub-entity.<br>\n";
    print "<script language=\"Javascript1.2\">\nErrorOccurred = 1;\n</script>\n";
    $Error_Occurred = 1;
  }
  else  # It's not being used so we can delete it.
  {
    $result = $conn->exec("delete from sub_entity_tb where name = '$entry_name' and code = '$entry_code' AND ledger_code = '$ledger_code'");
    if ($result->resultStatus != PGRES_COMMAND_OK)
    {
      print "<h1>Error:  Deleting entry <font color=\"blue\">$entry_name</font> failed!</h1>\n";
      print $conn->errorMessage . "<br>$sys_admin";
      $result = $conn->exec("rollback work");
      CleanUp();
    }
    print "Successfully deleted Sub Entity,&nbsp; Name = '$entry_name', Code = '$entry_code'<br>\n";
  }

  # commit the changes
  $result = $conn->exec("commit work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Committing Transaction failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to commit transaction to Ledger <font color=\"blue\">$ledger_name</font>.<br>$sys_admin";
    $result = $conn->exec("rollback work");
    CleanUp();
  }

  # end the transaction
  $result = $conn->exec("end work");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    print "<h1>Error: Ending Transaction failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to end transaction with Ledger <font color=\"blue\">$ledger_name</font>.<br>$sys_admin";
    CleanUp();
  }
  print "<br><br><br>You will be returned to the Sub Entity Administration screen in <b>";
  if (not $Error_Occurred)
  {
    print "3";
  }
  else
  {
    print "5";
  }
  print "</b> seconds.<br>\n";
}

if ($working_mode == 1 || $working_mode == 2)
{
  my ($button_name, $button_val);
  if ($working_mode == 1)
  {
    $button_name = "create";
    $button_val = "Create Sub Entity";
  }
  elsif ($working_mode == 2)
  {
    $button_name = "update";
    $button_val = "Update Sub Entity";
  }
  print <<"END_OF_FORM";
  </table>
  <table border=0 cellspacing=0 cellpadding=2 width=50%>
    <tr>
      <td align=center><input type=submit name=\"$button_name\" value=\"$button_val\">&nbsp;<input type=reset value=\"Reset\"></td>
    </tr>
  </table>
  </form>
END_OF_FORM
}

# print a horizontal rule to break the menu from the behind the scenes links
#print "<br><hr>\n";

print "<form method=post action=\"$site_dir/cgi-bin/sub_entities.cgi\" name=\"menu_bar2\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<div align=left><a href=\"$site_dir/cgi-bin/sub_entities.cgi\" onClick=\"javascript: document.menu_bar2.submit(); return false;\">Return to Sub Entity Administration</a></div>\n";
print "</form>\n";

print "</body>\n";
print "</html>\n";
exit 0;
