#!/usr/bin/perl
# time_tracker_config1.cgi  -  Created by James Pattie, 01/12/2000
# This script allows the user to specify if the Accounting Application
# will interact with a Time Tracker application and if so, where it
# is located at and what is necessary to access it.
#$Id: time_tracker_config1.cgi,v 1.1 2001/09/06 10:21:20 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';
my $really_in_ledger;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$really_in_ledger = $input{'really_in_ledger'};

my $ledger_db;
$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify a Ledger to work with!</h1>\n";
  print $back_button;
  CleanUp();
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print $conn->errorMessage . "<br>$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# verify the ledger is valid
$result = $conn->exec("SELECT name FROM entity_tb WHERE database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger '$ledger_db' is invalid!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate the given ledger.<br>$sys_admin";
  CleanUp();
}
my $ledger_name;
$ledger_name = $result->getvalue(0, 0);

# get the Ledger Code.
$result = $conn->exec("SELECT code, color FROM ledger_tb WHERE name = '$ledger_name'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Couldn't get Ledger Code from Ledger '$ledger_name'!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
my $ledger_code;
$ledger_code = $result->getvalue(0, 0);
my $ledger_color;
$ledger_color = $result->getvalue(0, 1);

# present them with the main menu

print "<head><title>$prog_name $prog_version Time Tracker Configuration</title>\n";
print <<"END_OF_SCRIPT";
  <script language="JavaScript1.2">
  function ValidateTimeForm()
  {
    var msg = "";
    if (document.time_form.use_time_tracker[0].checked)
    {
      // validate that the text fields appear to be valid.
      if (document.time_form.tk_host.value.length == 0)
      {
        msg += "You must specify the Host!\\n";
      }
      if (document.time_form.tk_db.value.length == 0)
      {
        msg += "You must specify the Database Name!\\n";
      }
      if (document.time_form.tk_user.value.length == 0)
      {
        msg += "You must specify the Database User name!\\n";
      }

      if (msg.length > 0)
      {
        alert(msg);
        return false;
      }
    }

    return true;
  }
  </script>
END_OF_SCRIPT
print "</head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: blue }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%>Working with Ledger <font color=\"$ledger_color\">$ledger_name</font>.</td>\n";
print "<td align=left><a href=\"$site_dir/help.html#tk_configuration\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Time Tracker Configuration</h2>\n";

$result = $conn->exec("SELECT use_time_tracker, tk_host, tk_db, tk_user, tk_passwd FROM time_tracker_config_tb WHERE ledger_code = '$ledger_code'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  print "<h1>Error:  Accessing database failed!</h1>\n";
  print $conn->errorMessage . "<br>Tried to get Time Tracker Configuration info.<br>$sys_admin";
  CleanUp();
}
if ($result->ntuples == 0)
{
  print "<h1>Internal Error:  Time Tracker Configuration is not present!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
my $use_time_tracker;
$use_time_tracker = $result->getvalue(0, 0);
my $tk_host;
$tk_host = $result->getvalue(0, 1);
my $tk_db;
$tk_db = $result->getvalue(0, 2);
my $tk_user;
$tk_user = $result->getvalue(0, 3);
my $tk_passwd;
$tk_passwd = $result->getvalue(0, 4);

# create the form to allow the user to select a ledger to work with.
print "<form method=post action=\"$site_dir/cgi-bin/time_tracker_config2.cgi\" name=\"time_form\" onSubmit=\"return ValidateTimeForm();\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
print "<table border=0 cellspacing=0 cellpadding=2 bgcolor=\"$menu_bgcolor\">\n";
print "<tr><td align=right><font color=\"$menu_fgcolor\">Use Time Tracker Integration:</font></td><td align=left>";
if ($use_time_tracker eq "t")
{
  print "Yes&nbsp;<input type=radio name=\"use_time_tracker\" value=\"ON\" checked>&nbsp;&nbsp;No&nbsp;<input type=radio name=\"use_time_tracker\" value=\"OFF\">\n";
}
else
{
  print "Yes&nbsp;<input type=radio name=\"use_time_tracker\" value=\"ON\">&nbsp;&nbsp;No&nbsp;<input type=radio name=\"use_time_tracker\" value=\"OFF\" checked>\n";
}
print "</td></tr>\n";
print "<tr><td align=right><font color=\"$menu_fgcolor\">Host:</font></td><td align=left><input type=text name=\"tk_host\" value=\"$tk_host\"></td></tr>\n";
print "<tr><td align=right><font color=\"$menu_fgcolor\">Database Name:</font></td><td align=left><input type=text name=\"tk_db\" value=\"$tk_db\"></td></tr>\n";
print "<tr><td align=right><font color=\"$menu_fgcolor\">User Name:</font></td><td align=left><input type=text name=\"tk_user\" value=\"$tk_user\"></td></tr>\n";
print "<tr><td align=right><font color=\"$menu_fgcolor\">Password:</font></td><td align=left><input type=password name=\"tk_passwd\" value=\"$tk_passwd\"></td></tr>\n";
print "</table>\n";
#print "<input type=submit name=\"open_ledger\" value=\"Open Ledger\">&nbsp;";
if ($really_in_ledger eq "yes")
{
  print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
}
print "<input type=submit name=\"update\" value=\"Update Configuration\">&nbsp;";
print "<input type=reset value=\"Reset\">\n";
print "</form>\n";

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if ($really_in_ledger ne "yes")
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/time_tracker_config.cgi\">Time Tracker Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;
