#!/usr/bin/perl
# time_tracker_config2.cgi  -  Created by James Pattie, 01/12/2000
# This script updates the Accounting Application
# with Time Tracker configuration info.
#$Id: time_tracker_config2.cgi,v 1.1 2001/09/06 10:21:20 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

use strict;

use Pg;  # PostgreSQL Database Module
use cgi_lib;  # CGI Library routines
use custom;  # our customizable features

use vars '$date';
my $use_time_tracker;
my $tk_host;
my $tk_db;
my $tk_user;
my $tk_passwd;
my $really_in_ledger;

use PortalWrapper; # The Portal Wrapper library.

my $portalObj;
$portalObj = PortalWrapper->new;
if ($portalObj->didErrorOccur)
{
  # begin the HTML Output
  print cgi_lib::PrintHeader();
  print "<html>\n";

  PrintErrorHeader();
  print $portalObj->errorMessage;
  CleanUp();
}
my $dbObj;
$dbObj = $portalObj->getDBInfo;
if (!defined $dbObj)
{
  if ($portalObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $portalObj->errorMessage;
    CleanUp();
  }
}
else
{
  if ($dbObj->didErrorOccur)
  {
    # begin the HTML Output
    print cgi_lib::PrintHeader();
    print "<html>\n";
  
    PrintErrorHeader();
    print $dbObj->errorMessage;
    CleanUp();
  }
}
$db_host = $dbObj->dbHost;
$db_name = $dbObj->dbName;
$db_user = $dbObj->dbUser;
$db_passwd = $dbObj->dbPasswd;

my $database_connected;
$database_connected = 0;

# get the values being passed in from the client
%input = ();  # make sure it is empty!
cgi_lib::ReadParse(*input);

# begin the HTML Output
print cgi_lib::PrintHeader();
print "<html>\n";

$really_in_ledger = $input{'really_in_ledger'};

my $ledger_db;
$ledger_db = $input{'ledger'};
if (length $ledger_db == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  The Ledger must be specified to work with!</h1>\n";
  print "$sys_admin";
  CleanUp();
}

if (length $input{'use_time_tracker'} == 0)
{
  PrintErrorHeader();
  print "<h1>Error:  You must specify if you are going to use Time Tracker or not!</h1>\n";
  print "$sys_admin";
  CleanUp();
}
$use_time_tracker = $input{'use_time_tracker'};
if ($use_time_tracker eq "ON")
{
  $use_time_tracker = "true";
}
elsif ($use_time_tracker eq "OFF")
{
  $use_time_tracker = "false";
}
else
{
  PrintErrorHeader();
  print "<h1>Error:  Invalid Value for Specifying the use of Time Tracker!</h1>\n";
  print "Value = '$use_time_tracker'<br>$sys_admin";
  CleanUp();
}

if ($use_time_tracker eq "true")
{
  my $temp_str;
  # verify that we have the host, db, user name, and password values
  if (length $input{'tk_host'} == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must specify a Host to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $tk_host = $input{'tk_host'};
  $temp_str = IsValidString($tk_host);
  if (length $temp_str > 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Host value is invalid!</h1>\n";
    print "<b>$temp_str</b> is invalid.<br>$back_button";
    CleanUp();
  }

  if (length $input{'tk_db'} == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must specify a Database to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $tk_db = $input{'tk_db'};
  $temp_str = IsValidString($tk_db);
  if (length $temp_str > 0 && $temp_str ne "_")
  {
    PrintErrorHeader();
    print "<h1>Error:  Database value is invalid!</h1>\n";
    print "<b>$temp_str</b> is invalid.<br>$back_button";
    CleanUp();
  }

  if (length $input{'tk_user'} == 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  You must specify a User to work with!</h1>\n";
    print "$back_button";
    CleanUp();
  }
  $tk_user = $input{'tk_user'};
  $temp_str = IsValidString($tk_user);
  if (length $temp_str > 0 && $temp_str ne "_")
  {
    PrintErrorHeader();
    print "<h1>Error:  User value is invalid!</h1>\n";
    print "<b>$temp_str</b> is invalid.<br>$back_button";
    CleanUp();
  }

  $tk_passwd = $input{'tk_passwd'};
  $temp_str = IsValidString($tk_passwd);
  if (length $temp_str > 0)
  {
    PrintErrorHeader();
    print "<h1>Error:  Password value is invalid!</h1>\n";
    print "<b>$temp_str</b> is invalid.<br>$back_button";
    CleanUp();
  }
}

# check to see if we are running under mod_perl
if (exists $ENV{MOD_PERL})
{
  # check to see if the database is still connected
  if (defined $conn)
  {
    # check the status of the connection
    if ($conn->status != PGRES_CONNECTION_OK)
    {
      $database_connected = 0;
    }
    else
    {
      # connection still good!
      $database_connected = 1;
    }
  }
  else
  {
    $database_connected = 0;
  }
}

if (not $database_connected)
{
  # connect to the database.
  $conn = Pg::connectdb("host = $db_host dbname = $db_name user = $db_user password = $db_passwd");
  if ($conn->status == PGRES_CONNECTION_BAD)
  {
    PrintErrorHeader();
    print "<h1>Error:  Couldn't connect to the database!</h1><br>\n";
    print "$sys_admin";
    CleanUp();
  }
}

# set the DateStyle for the Database.
$result = setDBDateStyle($conn);
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Setting Date Style failed!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}

# verify the ledger is valid
$result = $conn->exec("SELECT name FROM entity_tb WHERE database_name = '$ledger_db'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Ledger '$ledger_db' is invalid!</h1>\n";
  print $conn->errorMessage . "<br>Tried to validate the given ledger.<br>$sys_admin";
  CleanUp();
}
my $ledger_name;
$ledger_name = $result->getvalue(0, 0);

# get the Ledger Code.
$result = $conn->exec("SELECT code, color FROM ledger_tb WHERE name = '$ledger_name'");
if ($result->resultStatus != PGRES_TUPLES_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Couldn't get Ledger Code from Ledger '$ledger_name'!</h1>\n";
  print $conn->errorMessage . "<br>$sys_admin";
  CleanUp();
}
my $ledger_code;
$ledger_code = $result->getvalue(0, 0);
my $ledger_color;
$ledger_color = $result->getvalue(0, 1);

# start a transaction
$result = $conn->exec("BEGIN WORK");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Failed to Start a Transaction!</h1>\n";
  print $conn->errorMessage . "<br>Tried to Begin Work.<br>$sys_admin";
  CleanUp();
}

# make the changes
if ($use_time_tracker eq "true")
{
  $result = $conn->exec("UPDATE time_tracker_config_tb SET use_time_tracker = true, tk_host = '$tk_host', tk_db = '$tk_db', tk_user = '$tk_user', tk_passwd = '$tk_passwd' WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Updating Time Tracker Configuration failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to Turn on Using Time Tracker, Host = '$tk_host', DB = '$tk_db', User = '$tk_user'.<br>$sys_admin";
    CleanUp();
  }
}
else
{
  $result = $conn->exec("UPDATE time_tracker_config_tb SET use_time_tracker = false WHERE ledger_code = '$ledger_code'");
  if ($result->resultStatus != PGRES_COMMAND_OK)
  {
    PrintErrorHeader();
    print "<h1>Error:  Updating Time Tracker Configuration failed!</h1>\n";
    print $conn->errorMessage . "<br>Tried to Turn off Using Time Tracker.<br>$sys_admin";
    CleanUp();
  }
}

# commit the changes
$result = $conn->exec("COMMIT WORK");
if ($result->resultStatus != PGRES_COMMAND_OK)
{
  PrintErrorHeader();
  print "<h1>Error:  Failed to Commit the Transaction!</h1>\n";
  print $conn->errorMessage . "<br>Tried to Commit Work.<br>$sys_admin";
  CleanUp();
}

# present them with the main menu 

print "<head><title>$prog_name $prog_version Time Tracker Configuration</title></head>\n";
print "<body bgcolor=$body_bgcolor>\n";
print <<"END_OF_STYLE";
<style type="text/css"><!--
  a:hover   { color: blue }
  a:link    { text-decoration: none; color: blue }
  a:visited { text-decoration: none; color: blue }
  a:active  { text-decoration: none; color: blue }-->
</style>
<script language="Javascript1.2" src="../js/common.js"></script>
END_OF_STYLE
# get the date
$date = `/bin/date +%m-%d-%Y`;
chomp $date;

print "<center>\n";
print "<table border=0 width=100% cellspacing=0 cellpadding=2 bgcolor=\"$body_bgcolor\">\n";
print "<tr>\n";
print "<td width=47%>Working with Ledger <font color=\"$ledger_color\">$ledger_name</font>.</td>\n";
print "<td align=left><a href=\"$site_dir/help.html#tk_configuration\" target=\"docs\"><img border=0 src=\"$site_dir/images/help.jpg\" alt=\"Help\"></a></td>\n";
print "<td align=right>$date</td></tr></table>\n<hr><br>\n";
print PrintStandardMainMenu($ledger_db);
print "<h2><font color=$prog_fgcolor>$prog_name $prog_version</font> Time Tracker Configuration</h2>\n";

print "<br><br><br><h1>Time Tracker Configuration Updated Successfully!</h1>\n";

print "<form method=post action=\"$site_dir/cgi-bin/time_tracker_config1.cgi\" name=\"redo_form\">\n";
print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
if ($really_in_ledger eq "yes")
{
  print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
}
print "<a href=\"$site_dir/cgi-bin/time_tracker_config1.cgi\" onClick=\"document.redo_form.submit(); return false;\">To Edit Configuration again go <u>here</u>.</a>\n";
print "</form>\n";

# print a horizontal rule to break the menu from the behind the scenes links
print "<br><hr>\n";

if ($really_in_ledger ne "yes")
{
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/time_tracker_config.cgi\">Time Tracker Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\">Program Configuration</a></div>\n";
}
else
{
  print "<form name=\"form1\" method=post action=\"\">\n";
  print "<input type=hidden name=\"ledger\" value=\"$ledger_db\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/time_tracker_config1.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/time_tracker_config1.cgi'); return false;\">Time Tracker Configuration</a></div>\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/prg_config.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/prg_config.cgi'); return false;\">Program Configuration</a></div>\n";
  print "<input type=hidden name=\"open\" value=\"Open\">\n";
  print "<input type=hidden name=\"really_in_ledger\" value=\"yes\">\n";
  print "<br><div align=left><a href=\"$site_dir/cgi-bin/ledger.cgi\" onClick=\"go_here(document.form1, '$site_dir/cgi-bin/ledger.cgi'); return false;\">Return to Ledger Summary</a></div>\n";
  print "</form>\n";
}

print "</body>\n";
print "</html>\n";
exit 0;
